package com.mh.user.config;

import com.mh.user.component.ReceiveHandler;
import com.mh.user.constants.Constant;
import lombok.extern.slf4j.Slf4j;
import org.apache.log4j.Logger;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.connection.CachingConnectionFactory;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.autoconfigure.amqp.SimpleRabbitListenerContainerFactoryConfigurer;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.task.SimpleAsyncTaskExecutor;
import org.springframework.transaction.PlatformTransactionManager;

/**
 *队列绑定队列、绑定交换机、绑定路由KEY
 */
@Configuration
public class RabbitmqConfig {

    private static final Logger log = Logger.getLogger(RabbitmqConfig.class);

//    public static final String QUEUE_NAME = "queue_zjjd";                //采集队列,珠江酒店

//    public static final String QUEUE_NAME = "queue_gdkj";                //采集队列,广大科技
    public static final String QUEUE_NAME = "queue_ghkj";                //采集队列,广和科技
//    public static final String QUEUE_NAME = "queue_zjgj";                //采集队列,珠江国际
//    public static final String QUEUE_NAME = "queue_blsz";                //采集队列,保利山庄
//    public static final String QUEUE_NAME = "queue_dgybg";                //采集队列,东莞迎宾馆
    public static final String EXCHANGE_NAME="topic.exchange";           //topics类型交换机==============
//    public static final String ROUTINGKEY_NAME="topic.#.zjjd.#";         //路由KEY,珠江酒店
//    public static final String ROUTINGKEY_NAME="topic.#.gdkj.#";         //路由KEY,广大科技
    public static final String ROUTINGKEY_NAME="topic.#.ghkj.#";         //路由KEY,广和科技
//    public static final String ROUTINGKEY_NAME="topic.#.zjgj.#";         //路由KEY,珠江国际
//    public static final String ROUTINGKEY_NAME="topic.#.blsz.#";         //路由KEY,保利山庄
//    public static final String ROUTINGKEY_NAME="topic.#.dgybg.#";         //路由KEY,东莞迎宾馆

    @Autowired
    private CachingConnectionFactory connectionFactory;

    //声明交换机
    @Bean(EXCHANGE_NAME)
    public Exchange exchange(){
        //durable(true) 持久化，mq重启之后交换机还在
        return ExchangeBuilder.topicExchange(EXCHANGE_NAME).durable(true).build();
    }

    //声明队列
    /*
     *   new Queue(QUEUE_EMAIL,true,false,false)
     *   durable="true" 持久化 rabbitmq重启的时候不需要创建新的队列
     *   auto-delete 表示消息队列没有在使用时将被自动删除 默认是false
     *   exclusive  表示该消息队列是否只在当前connection生效,默认是false
     */
    //声明队列
    @Bean(QUEUE_NAME)
    public Queue queue(){
        log.info("声明队列"+QUEUE_NAME);
        return new Queue(QUEUE_NAME);
    }

    //ROUTINGKEY_WX队列绑定交换机，指定routingKey
    @Bean
    public Binding bindingQueue(@Qualifier(QUEUE_NAME) Queue queue,
                                @Qualifier(EXCHANGE_NAME) Exchange exchange){
        log.info(QUEUE_NAME+"队列绑定到交换机topic.exchange");
        return BindingBuilder.bind(queue).to(exchange).with(ROUTINGKEY_NAME).noargs();
    }

    //应该是处理多线程消费的
    @Bean
    public SimpleRabbitListenerContainerFactory simpleRabbitListenerContainerFactory(){
        SimpleRabbitListenerContainerFactory listenerContainerFactory = new SimpleRabbitListenerContainerFactory();
        listenerContainerFactory.setConnectionFactory(connectionFactory);
        listenerContainerFactory.setConcurrentConsumers(3);
        listenerContainerFactory.setMaxConcurrentConsumers(5);
        listenerContainerFactory.setPrefetchCount(1);//预处理消息个数
        listenerContainerFactory.setAcknowledgeMode(AcknowledgeMode.MANUAL);//开启消息确认机制
        return listenerContainerFactory;
    }

}
