package com.mh.user.controller;

import com.mh.common.http.HttpResult;
import com.mh.user.annotation.SysLogger;
import com.mh.user.entity.BuildingEntity;
import com.mh.user.model.BuildingModel;
import com.mh.user.service.BuildingService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("building")
public class BuildingController {

   @Autowired
    private BuildingService buildingService;

   //保存
   @SysLogger(value="楼栋管理",optDesc = "添加楼栋信息")
   @PostMapping(value="/save")
    public HttpResult saveBuilding(@RequestBody BuildingEntity buildingEntity) {
       try{
           return HttpResult.ok(buildingService.saveBuilding(buildingEntity));
       }catch (Exception e){
           e.printStackTrace();
           return HttpResult.error();
       }

    }

    //修改
    @SysLogger(value="楼栋管理",optDesc = "编辑楼栋信息")
    @PostMapping(value="/update")
    public HttpResult updateBuilding(@RequestBody BuildingEntity buildingEntity) {
        return HttpResult.ok(buildingService.updateBuilding(buildingEntity));
    }

    //查询所有
    @SysLogger(value="楼栋管理",optDesc = "查询楼栋信息")
    @PostMapping(value = "/query")
    public HttpResult queryBuilding(@RequestParam(value = "buildingId", required = false)String buildingId,
                                    @RequestParam(value= "page", required=true)Integer page,
                                    @RequestParam(value= "limit", required=true)Integer limit) {
        try{
            int count=buildingService.getCount(buildingId, page,limit);
            List<BuildingEntity> records=buildingService.queryBuilding(buildingId, page,limit);
            return HttpResult.ok(count,records);
        }catch (Exception e){
            e.printStackTrace();
            return HttpResult.error();
        }
    }

    //查询楼栋名称
    @PostMapping(value="/name")
    public HttpResult selectBuildingName() {
       try{
           List<BuildingModel> list=buildingService.selectBuildingName();
           return HttpResult.ok(list);
       }catch (Exception e){
           e.printStackTrace();
           return HttpResult.error();
       }

    }

    // 删除多
    @PostMapping(value="/deletes")
    public HttpResult delete(@RequestBody List<BuildingEntity> records) {
        return HttpResult.ok(buildingService.deleteBuilding(records));
    }

    // 删除单个
    @SysLogger(value="楼栋管理",optDesc = "删除楼栋信息")
    @PostMapping(value="/delete")
    public HttpResult delete(@RequestParam String id ) {
        return HttpResult.ok(buildingService.deleteBuilding(id));
    }

    // 查询编号
    @PostMapping(value="/selectBuildingId")
    public HttpResult selectBuildingId(@RequestParam String buildingName ) {
        String id=buildingService.selectBuildingId(buildingName);
        return HttpResult.ok(id);
    }

}
