package com.mh.user.controller;

import com.alibaba.fastjson.JSONObject;
import com.mh.common.http.HttpResult;
import com.mh.user.dto.DisplayBoxDataDTO;
import com.mh.user.dto.TableInfoDTO;
import com.mh.user.entity.GaugeEntity;
import com.mh.user.entity.HostValue;
import com.mh.user.entity.TableDataEntity;
import com.mh.user.entity.TableInfoEntity;
import com.mh.user.service.chillers.ChillersService;
import com.mh.user.service.chillers.DeviceDisplayService;
import com.mh.user.service.chillers.GaugeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author ljf
 * @title ：
 * @description ： 设备监控操作控制接口
 * @updateTime 2020-05-20
 * @throws ：
 */
@RestController
public class ChillersController {

    private final ChillersService chillersService;
    private final DeviceDisplayService deviceDisplayService;

    public ChillersController(ChillersService chillersService, DeviceDisplayService deviceDisplayService) {
        this.chillersService = chillersService;
        this.deviceDisplayService = deviceDisplayService;
    }

    // 查询冷水机组各个部位的状态值
//    @GetMapping("/chillers")

    /**
     * 查询冷水机组进出水温度值等设备数据详情
     * @param deviceType
     * @param type
     * @return
     */
    @GetMapping("/charts/getDisplayBoxData")
    public HttpResult getDisplayBoxData(@RequestParam(value = "deviceType") String deviceType,
                                        @RequestParam(value = "type") Integer type) {
        List<DisplayBoxDataDTO> displayBoxDataDTOS = deviceDisplayService.deviceDisplayBoxData(deviceType, String.valueOf(type));
        return HttpResult.ok("success", displayBoxDataDTOS);
    }

    /**
     * 主机监测详情左侧表格数据
     * @param deviceName
     * @return
     */
    @GetMapping("/charts/getHostTableOneData")
    public HttpResult queryChillersOneTable(@RequestParam(value = "deviceName") String deviceName) {
        TableInfoDTO<TableInfoEntity> tableInfoDTO = new TableInfoDTO<>();
        tableInfoDTO.setTableInfo(chillersService.queryLeftAndRightData(deviceName, 1));
        return HttpResult.ok("success", tableInfoDTO);
    }

    /**
     * 主机监测详情顶部表格数据
     * @param deviceName
     * @return
     */
    @GetMapping("/charts/getHostTopData")
    public HttpResult queryChillersTopTable(@RequestParam(value = "deviceName", required = true) String deviceName) {
        TableInfoDTO<TableDataEntity> tableInfoDTO = new TableInfoDTO<>();
        tableInfoDTO.setTableInfo(chillersService.queryTopData(deviceName, 1));
        return HttpResult.ok("success", tableInfoDTO);
    }

    /**
     * 主机监测详情右侧一表格数据
     * @param deviceName
     * @return
     */
    @GetMapping("/charts/getHostTableTwoData")
    public HttpResult queryChillersRightTable(@RequestParam(value = "deviceName", required = true) String deviceName) {
        TableInfoDTO<TableInfoEntity> tableInfoDTO = new TableInfoDTO<>();
        tableInfoDTO.setTableInfo(chillersService.queryLeftAndRightData(deviceName, 2));
        return HttpResult.ok("success", tableInfoDTO);
    }

    /**
     * 主机监测详情右侧二表格数据
     * @param deviceName
     * @return
     */
    @GetMapping("/charts/getHostTableThreeData")
    public HttpResult queryChillersRightTable1(@RequestParam(value = "deviceName", required = true) String deviceName) {
        TableInfoDTO<TableInfoEntity> tableInfoDTO = new TableInfoDTO<>();
        tableInfoDTO.setTableInfo(chillersService.queryLeftAndRightData(deviceName, 3));
        return HttpResult.ok("success", tableInfoDTO);
    }

    // 冷却泵、冷冻泵、冷却塔详情页顶部表格数据
    @GetMapping("/charts/getPumpOrTowerTopTableData")
    public HttpResult getPumpOrTowerTopData(@RequestParam(value = "deviceName", required = true) String deviceName) {
        TableInfoDTO<TableInfoEntity> tableInfoDTO = new TableInfoDTO<>();
        tableInfoDTO.setTableInfo(chillersService.queryLeftAndRightData(deviceName, 1));
        return HttpResult.ok("success", tableInfoDTO);
    }

    // 冷却泵、冷冻泵、冷却塔详情页顶部数据
//    @GetMapping("/charts/getPumpOrTowerTopTableData")
    @GetMapping("/charts/getPumpOrTowerTopData")
    public HttpResult getPumpOrTowerTopTableData(@RequestParam(value = "deviceName", required = true) String deviceName) {
        TableInfoDTO<TableDataEntity> tableInfoDTO = new TableInfoDTO<>();
        tableInfoDTO.setTableInfo(chillersService.queryTopData(deviceName, 1));
        return HttpResult.ok("success", tableInfoDTO);
    }


    // 获取冷却/冷冻水进出水温度、主机/冷却塔温度
    @GetMapping("/operation/getTempValue")
    public HttpResult getTempValue(@RequestParam(value = "deviceType", required = true) String deviceType) {
        List<HostValue> hostValueList = chillersService.queryTempValue(deviceType);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("values", hostValueList);
        return HttpResult.ok("success",jsonObject);
    }

    @Autowired
    private GaugeService gaugeService;

    // 设备监控页表盘分析图
    @GetMapping("/charts/getGaugeData")
    public HttpResult getGaugeData(@RequestParam(value = "type") String type, @RequestParam(value = "deviceName") String deviceName) {
        GaugeEntity gaugeEntity = gaugeService.getGaugeData(type,deviceName);
        return HttpResult.ok("success",gaugeEntity);
    }

}
