package com.mh.user.controller;

import com.mh.common.http.HttpResult;
import com.mh.user.annotation.SysLogger;
import com.mh.user.entity.DataResultChEntity;
import com.mh.user.entity.DataResultEntity;
import com.mh.user.service.BuildingService;
import com.mh.user.service.DataResultService;
import com.mh.user.utils.ExchangeStringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("dataResult")
public class DataResultController {

    @Autowired
    DataResultService dataResultService;

    @Autowired
    BuildingService buildingService;


    @SysLogger(value="中央空调",optDesc = "数据分析查询")
    @PostMapping(value="/query")
    public HttpResult queryDataResult(@RequestParam(value = "projectID", required = false) String projectID,
                                      @RequestParam(value= "startDate", required=false)String startDate,
                                      @RequestParam(value= "endDate", required=false)String endDate,
                                      @RequestParam(value= "deviceType", required=false)String deviceType,
                                      @RequestParam(value= "page", required=true)Integer page,
                                      @RequestParam(value= "limit", required=true)Integer limit) {

        try{

           int count=dataResultService.getDataResultCount(projectID,startDate,endDate,deviceType,page,limit);
           List<DataResultEntity> records=dataResultService.queryDataResult(projectID,startDate,endDate,deviceType,page,limit);
           return HttpResult.ok(count,records);

        }catch (Exception e){
            e.printStackTrace();
            return HttpResult.error();
        }
    }

    @SysLogger(value="中央空调",optDesc = "能耗分析按小时查询")
    @PostMapping(value="/hour")
    public HttpResult queryDataResultMi(@RequestParam(value= "projectID", required=false)String projectID,
                                        @RequestParam(value= "dateType", required=false)String dateType,
                                        @RequestParam(value= "curDate", required=false)String curDate,
                                        @RequestParam(value="page", required=false) int page,
                                        @RequestParam(value="limit", required=false) int limit) {
        try{
            if(projectID!=null && projectID.length()>0){
                String startDate="";
                curDate=curDate.substring(0,16)+":00";
                if (dateType.equals("1小时")){
                    startDate= ExchangeStringUtil.dateRoll(1,curDate);
                    List<DataResultChEntity> records=dataResultService.queryDataResultMi(projectID,startDate,curDate,page,limit,0);
                    int count=dataResultService.dataResultOneMiCount(projectID,startDate,curDate);
                    return HttpResult.ok(count,records);
                }else if (dateType.equals("2小时")){
                    startDate=ExchangeStringUtil.dateRoll(2,curDate);
                    List<DataResultChEntity> records=dataResultService.queryDataResultMi(projectID,startDate,curDate,page,limit,0);
                    int count=dataResultService.dataResultOneMiCount(projectID,startDate,curDate);
                    return HttpResult.ok(count,records);
                }else if (dateType.equals("8小时")){
                    startDate=ExchangeStringUtil.dateRoll(8,curDate);
                    List<DataResultChEntity> records=dataResultService.queryDataResultMi(projectID,startDate,curDate,page,limit,1);
                    int count=dataResultService.dataResultFiveMiCount(projectID,startDate,curDate);
                    return HttpResult.ok(count,records);
                }else if (dateType.equals("12小时")){
                    startDate=ExchangeStringUtil.dateRoll(12,curDate);
                    List<DataResultChEntity> records=dataResultService.queryDataResultMi(projectID,startDate,curDate,page,limit,1);
                    int count=dataResultService.dataResultFiveMiCount(projectID,startDate,curDate);
                    return HttpResult.ok(count,records);
                }else if (dateType.equals("24小时")){
                    startDate=ExchangeStringUtil.dateRoll(24,curDate);
                    List<DataResultChEntity> records=dataResultService.queryDataResultMi(projectID,startDate,curDate,page,limit,2);
                    int count=dataResultService.dataResultFifteenMiCount(projectID,startDate,curDate);
                    return HttpResult.ok(count,records);
                }else{ //1小时
                    startDate=ExchangeStringUtil.dateRoll(1,curDate);
                    List<DataResultChEntity> records=dataResultService.queryDataResultMi(projectID,startDate,curDate,page,limit,0);
                    int count=dataResultService.dataResultOneMiCount(projectID,startDate,curDate);
                    return HttpResult.ok(count,records);
                }
            }else{
                return HttpResult.error();
            }
        }catch (Exception e){
//            e.printStackTrace();
            return HttpResult.error();
        }
    }

    @SysLogger(value="中央空调",optDesc = "能耗分析按分钟查询")
    @PostMapping(value="/minute")
    public HttpResult queryDataResultSec(@RequestParam(value= "projectID", required=false)String projectID,
                                         @RequestParam(value= "dateType", required=false)String dateType,
                                         @RequestParam(value= "curDate", required=false)String curDate,
                                         @RequestParam(value="page",required=false) int page,
                                         @RequestParam(value="limit",required=false) int limit) {

        try{
            SimpleDateFormat df=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String startDate=""; //起始时间
            String strDate="";   //遍历时间
            String strDate2="";
            curDate=curDate.substring(0,16)+":00";
            if (dateType.equals("1分钟")){
                startDate= ExchangeStringUtil.dateTime(1,curDate);
                List<DataResultChEntity> records=dataResultService.queryDataResultSec(projectID,startDate,curDate);
                String deviceAddr=records.get(0).getDeviceAddr();
                String projectName=records.get(0).getProjectName();

//                Date date=df.parse(startDate); //顺序
                Date date=df.parse(curDate); //倒序
                List<Map<String,Object>> listMap=new ArrayList<Map<String,Object>>();
                for (int i=0;i<60;i++){
                    Map<String,Object>map=new HashMap<String,Object>();
                    //date.setTime(date.getTime()+1000);//顺序
                    date.setTime(date.getTime()-1000);//倒序
                    strDate=df.format(date);
                    map.put("deviceAddr",deviceAddr);
                    map.put("projectName",projectName);
                    map.put("curDate",strDate);
                    map.put("curValue","0");
                    for (DataResultChEntity dataResult: records) {
                        strDate2=df.format(dataResult.getCurDate());
                        if (strDate2.equals(strDate)){
                            map.put("curValue",dataResult.getCurValue());
                        }
                    }
                    listMap.add(map);
                }
                return HttpResult.ok(60,listMap);

            }else if (dateType.equals("2分钟")){
                startDate=ExchangeStringUtil.dateTime(2,curDate);
                List<DataResultChEntity> records=dataResultService.queryDataResultSec(projectID,startDate,curDate);
                String deviceAddr=records.get(0).getDeviceAddr();
                String projectName=records.get(0).getProjectName();
                //Date date=df.parse(startDate);//顺序
                Date date=df.parse(curDate); //倒序
                List<Map<String,Object>> listMap=new ArrayList<Map<String,Object>>();
                for (int i=0;i<120;i++){
                    Map<String,Object>map=new HashMap<String,Object>();
                    //date.setTime(date.getTime()+1000);
                    date.setTime(date.getTime()-1000);//倒序
                    strDate=df.format(date);
                    map.put("deviceAddr",deviceAddr);
                    map.put("projectName",projectName);
                    map.put("curDate",strDate);
                    map.put("curValue","0");
                    for (DataResultChEntity dataResult: records) {
                        strDate2=df.format(dataResult.getCurDate());
                        if (strDate2.equals(strDate)){
                          map.put("curValue",dataResult.getCurValue());
                        }
                    }
                    listMap.add(map);
                }
                return HttpResult.ok(120,listMap);
            }else if (dateType.equals("5分钟")){
                startDate=ExchangeStringUtil.dateTime(5,curDate);
                List<DataResultChEntity> records=dataResultService.queryDataResultSec(projectID,startDate,curDate);
                String deviceAddr=records.get(0).getDeviceAddr();
                String projectName=records.get(0).getProjectName();
                String s=startDate.substring(18,19);
                if (Integer.parseInt(s)>=0 && Integer.parseInt(s)<5){
                    startDate=startDate.substring(0,18)+"0";
                }else{
                    startDate=startDate.substring(0,18)+"5";
                }

                //Date date=df.parse(startDate);//顺序
                Date date=df.parse(curDate); //倒序
                List<Map<String,Object>> listMap=new ArrayList<Map<String,Object>>();
                for (int i=0;i<60;i++){
                    Map<String,Object>map=new HashMap<String,Object>();
                    //date.setTime(date.getTime()+5000);
                    date.setTime(date.getTime()-5000);
                    strDate=df.format(date);
                    map.put("deviceAddr",deviceAddr);
                    map.put("projectName",projectName);
                    map.put("curDate",strDate);
                    map.put("curValue","0");
                    for (DataResultChEntity dataResult: records) {
                        strDate2=df.format(dataResult.getCurDate());
                        if (strDate2.equals(strDate)){
//                            map.put("curDate",curDate);
                            map.put("curValue",dataResult.getCurValue());
                        }
                    }
                    listMap.add(map);
                }

                return HttpResult.ok(60,listMap);
            }else if (dateType.equals("15分钟")){
                startDate=ExchangeStringUtil.dateTime(15,curDate);
                List<DataResultChEntity> records=dataResultService.queryDataResultSec(projectID,startDate,curDate);
                String deviceAddr=records.get(0).getDeviceAddr();
                String projectName=records.get(0).getProjectName();
                String s=startDate.substring(18,19);
                if (Integer.parseInt(s)>=0 && Integer.parseInt(s)<5){
                    startDate=startDate.substring(0,18)+"0";
                }else{
                    startDate=startDate.substring(0,18)+"5";
                }

                //Date date=df.parse(startDate);
                Date date=df.parse(curDate);
                List<Map<String,Object>> listMap=new ArrayList<Map<String,Object>>();
                for (int i=0;i<180;i++){
                    Map<String,Object>map=new HashMap<String,Object>();
                    //date.setTime(date.getTime()+5000);
                    date.setTime(date.getTime()-5000);
                    strDate=df.format(date);
                    map.put("deviceAddr",deviceAddr);
                    map.put("projectName",projectName);
                    map.put("curDate",strDate);
                    map.put("curValue","0");
                    for (DataResultChEntity dataResult: records) {
                        strDate2=df.format(dataResult.getCurDate());
                        if (strDate2.equals(strDate)){
//                            map.put("curDate",curDate);
                            map.put("curValue",dataResult.getCurValue());
                        }
                    }
                    listMap.add(map);
                }

                return HttpResult.ok(180,listMap);
            }else if (dateType.equals("30分钟")){
                startDate=ExchangeStringUtil.dateTime(30,curDate);
                List<DataResultChEntity> records=dataResultService.queryDataResultSec(projectID,startDate,curDate);
                String deviceAddr=records.get(0).getDeviceAddr();
                String projectName=records.get(0).getProjectName();
                String s=startDate.substring(18,19);
                if (Integer.parseInt(s)>=0 && Integer.parseInt(s)<5){
                    startDate=startDate.substring(0,18)+"0";
                }else{
                    startDate=startDate.substring(0,18)+"5";
                }

                //Date date=df.parse(startDate);
                Date date=df.parse(curDate);
                List<Map<String,Object>> listMap=new ArrayList<Map<String,Object>>();
                for (int i=0;i<180;i++){
                    Map<String,Object>map=new HashMap<String,Object>();
                    //date.setTime(date.getTime()+10000);
                    date.setTime(date.getTime()-10000);
                    strDate=df.format(date);
                    map.put("deviceAddr",deviceAddr);
                    map.put("projectName",projectName);
                    map.put("curDate",strDate);
                    map.put("curValue","0");
                    for (DataResultChEntity dataResult: records) {
                        strDate2=df.format(dataResult.getCurDate());
                        if (strDate2.equals(strDate)){
//                            map.put("curDate",curDate);
                            map.put("curValue",dataResult.getCurValue());
                        }
                    }
                    listMap.add(map);
                }
                return HttpResult.ok(180,listMap);
            }else{ //1分钟
                startDate= ExchangeStringUtil.dateTime(1,curDate);
                List<DataResultChEntity> records=dataResultService.queryDataResultSec(projectID,startDate,curDate);
                String deviceAddr=records.get(0).getDeviceAddr();
                String projectName=records.get(0).getProjectName();
                //Date date=df.parse(startDate);
                Date date=df.parse(curDate);
                List<Map<String,Object>> listMap=new ArrayList<Map<String,Object>>();
                for (int i=0;i<60;i++){
                    Map<String,Object>map=new HashMap<String,Object>();
                    //date.setTime(date.getTime()+1000);
                    date.setTime(date.getTime()-1000);
                    strDate=df.format(date);
                    map.put("deviceAddr",deviceAddr);
                    map.put("projectName",projectName);
                    map.put("curDate",strDate);
                    map.put("curValue","0");
                    for (DataResultChEntity dataResult: records) {
                        strDate2=df.format(dataResult.getCurDate());
                        if (strDate2.equals(strDate)){
//                            map.put("curDate",strDate);
                            map.put("curValue",dataResult.getCurValue());
                        }
                    }
                    listMap.add(map);
                }
                return HttpResult.ok(listMap);
            }
        }catch (Exception e){
            e.printStackTrace();
            return HttpResult.error();
        }
    }

    @SysLogger(value="中央空调",optDesc = "查询冷水机参数实时值")
    @PostMapping(value="/dataNow")
    public HttpResult selectDataResultNow(@RequestParam(value= "deviceAddr", required=false)String deviceAddr,
                                          @RequestParam(value= "deviceType", required=false)String deviceType,
                                          @RequestParam(value= "projectID", required=false)String projectID) {

        List<DataResultChEntity> dataNow=dataResultService.selectDataResultNow(deviceAddr, deviceType, projectID);
        return HttpResult.ok("success",dataNow);
    }

    @SysLogger(value="中央空调",optDesc = "查询冷水机参数值")
    @PostMapping(value="/dataChiller")
    public HttpResult queryDataResultChiller(@RequestParam("projectID")String projectID,
                                             @RequestParam("deviceAddr")String deviceAddr,
                                             @RequestParam("registerName")String registerName,
                                             @RequestParam(value= "startDate", required=false)String startDate,
                                             @RequestParam(value="curDate", required=false)String curDate,
                                             @RequestParam("page")int page,
                                             @RequestParam("limit")int limit) {

        int count=dataResultService.dataResultChillerCount(projectID,deviceAddr, registerName, startDate, curDate);
        List<DataResultChEntity> dataChiller=dataResultService.queryDataResultChiller(projectID,deviceAddr,registerName,startDate,curDate,page,limit);
        return HttpResult.ok(count,dataChiller);
    }
}
