package com.mh.user.controller;

import com.alibaba.fastjson.JSONObject;
import com.mh.common.http.HttpResult;
import com.mh.user.annotation.SysLogger;
import com.mh.user.dto.DeviceMessageDTO;
import com.mh.user.entity.*;
import com.mh.user.manage.QuartzManager;
import com.mh.user.netty.NettyChillerControlClient;
import com.mh.user.service.chillers.DeviceDisplayService;
import com.mh.user.constants.Constant;
import com.mh.user.service.chillers.DeviceManageService;
import com.mh.user.service.chillers.DeviceParamService;
import com.mh.user.service.chillers.GatewayManageService;
import com.mh.user.utils.GetReadOrder485;
import com.mh.user.utils.QuerySendThread;
import com.mh.user.constants.SocketMessage;
import com.mh.user.utils.TimeDifferenceUtil;
import org.quartz.SchedulerException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.text.ParseException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author ljf
 * @title ：
 * @description ： 设备管理接口
 * @updateTime 2020-05-29
 * @updateTime 2020-07-17
 * @throws ：
 */
@RestController
public class DeviceManageController {

    private final DeviceDisplayService deviceDisplayService;
    private final GatewayManageService gatewayManageService;
    private final DeviceManageService deviceManageService;
    private final DeviceParamService deviceParamService;

    public DeviceManageController(DeviceDisplayService deviceDisplayService, GatewayManageService gatewayManageService, DeviceManageService deviceManageService, DeviceParamService deviceParamService) {
        this.deviceDisplayService = deviceDisplayService;
        this.gatewayManageService = gatewayManageService;
        this.deviceManageService = deviceManageService;
        this.deviceParamService = deviceParamService;
    }

    @Resource
    QuartzManager quartzManager;

    @Autowired
    private SocketMessage socketMessage;

    // 查询设备信息状态
    @GetMapping("/operation/getColdStation")
    public HttpResult getColdStation(@RequestParam(value = "deviceType", required = true) String deviceType) {
        List<DeviceMessageDTO> deviceMessageList = deviceDisplayService.queryDeviceStatus(deviceType);
        Map<String, Object> formValues = new HashMap<>();
        formValues.put("formValues",deviceMessageList);
        return HttpResult.ok("success", formValues);
    }

    // 对设备进行操作处理
    @PostMapping("/operation/operationDevice")
    public HttpResult operationDevice(@RequestBody List<OrderEntity> changeValues) {
      String result;
      try {
        List<OrderMessageEntity> orderMessageEntityList;
        // type值 0：修改频率， 1：修改开关状态, 2: 关闭冷却泵之前，查询最近关闭的冷却塔时间，3: 群控手自动切换类型，4: 修改温度, 5: 修改压力
//        // 添加网页发送指令状态 update by ljf on 2020-08-07
        Constant.CONTROL_WEB_FLAG = true;
        // 暂停采集
//        quartzManager.pauseJob("DDC","JobDDCGroup");
//        Thread.sleep(2000);
          // 修改成不用暂停采集处理
        GetReadOrder485 getReadOrder485 = new GetReadOrder485();
        // 判断是否是去关闭冷却泵，如果是，需要检查最近冷却塔有没有关闭并且关闭时间大于8分钟
        // 判断changeValues大小
        int size = changeValues.size();
        int type = changeValues.get(0).getType();
        if (type == 3 && size == 1) {
            // 生成指令
            orderMessageEntityList = getReadOrder485.createOrder(changeValues);
            if (orderMessageEntityList.size() != 0) {
                // 开启发送指令
                NettyChillerControlClient nettyChillerControlClient = new NettyChillerControlClient();
//            nettyChillerControlClient.connect(8081, "192.168.1.131", orderMessageEntityList);
                nettyChillerControlClient.connect(socketMessage.getPort(), socketMessage.getIP(), orderMessageEntityList);
                // 开启线程监测标志
                QuerySendThread querySendThread = new QuerySendThread();
                querySendThread.start();
                if (Constant.SEND_STATUS) {
                    result = "success";
                } else {
                    result = "fail";
                }
            } else {
                result = "fail";
            }
        } else if (type == 2 && size == 1) {
            TimeDifferenceUtil timeDifferenceUtil = new TimeDifferenceUtil();
            Boolean a = timeDifferenceUtil.timeDifference(socketMessage.getOverTime());
            if (a) {
                // 生成指令
                orderMessageEntityList = getReadOrder485.createOrder(changeValues);
                if (orderMessageEntityList.size() != 0) {
                    // 开启发送指令
                    NettyChillerControlClient nettyChillerControlClient = new NettyChillerControlClient();
//            nettyChillerControlClient.connect(8081, "192.168.1.131", orderMessageEntityList);
                    nettyChillerControlClient.connect(socketMessage.getPort(), socketMessage.getIP(), orderMessageEntityList);
                    // 开启线程监测标志
                    QuerySendThread querySendThread = new QuerySendThread();
                    querySendThread.start();
                    if (Constant.SEND_STATUS) {
                        result = "success";
                    } else {
                        result = "fail";
                    }
                } else {
                    result = "fail";
                }
            } else {
                result = "冷却塔关机还没有超过8分钟";
            }
        } else {
            // 生成指令
            orderMessageEntityList = getReadOrder485.createOrder(changeValues);
            if (orderMessageEntityList.size() != 0) {
                // 开启发送指令
                NettyChillerControlClient nettyChillerControlClient = new NettyChillerControlClient();
//            nettyChillerControlClient.connect(8081, "192.168.1.131", orderMessageEntityList);
                nettyChillerControlClient.connect(socketMessage.getPort(), socketMessage.getIP(), orderMessageEntityList);
                // 开启线程监测标志
                QuerySendThread querySendThread = new QuerySendThread();
                querySendThread.start();
                if (Constant.SEND_STATUS) {
                    result = "success";
                } else {
                    result = "fail";
                }
            } else {
                result = "fail";
            }
        }
        Constant.CONTROL_WEB_FLAG = false;
        // 不需要停止采集
//        Constant.WEB_FLAG = false;
//        // 延迟5秒处理，等待线程处理数据
        Thread.sleep(500);
//        // 重新开启定时采集
//        quartzManager.resumeAllJob();
//          quartzManager.resumeJob("DDC","JobDDCGroup");
      } catch (InterruptedException | ParseException e) {
        e.printStackTrace();
        result = "fail";
      }
      // 异常情况处理
        return HttpResult.ok(result);
    }

    /**
     * 查询网关信息
     * @param requestJson
     * @return
     */
    @PostMapping("/gateWay/getGateWayList")
    public HttpResult gateWayList(@RequestBody String requestJson){
        JSONObject jsonObject =JSONObject.parseObject(requestJson);
        Integer operator = null;
        Integer grade = null;
        if(jsonObject.get("operator") != null){
            if(jsonObject.get("operator").equals("中国移动")){
                operator = 0;
            }else if(jsonObject.get("operator").equals("中国联通")){
                operator = 1;
            }else if(jsonObject.get("operator").equals("中国电信")){
                operator = 2;
            }
        }
        if(jsonObject.get("grade") != null){
          if(jsonObject.get("grade").equals("正常")){
              grade = 0;
          }else if(jsonObject.get("grade").equals("不在线")){
              grade = 1;
          }else if(jsonObject.get("grade").equals("异常")){
              grade = 2;
          }
        }
        List<GatewayManageEntity> gateWayList = gatewayManageService.queryByOther(grade,operator);
        JSONObject tableData = new JSONObject();
        tableData.put("tableData",gateWayList);
        return HttpResult.ok(tableData);
//        System.out.println(requestJson);
//        return null;
    }

    /**
     * 新增或更新网关信息
     * @param reqestJson
     * @return
     */
    @PostMapping("/gateWay/addOrUpdateGateWayInfo")
    public HttpResult addOrUpdateGateWayInfo(@RequestBody GatewayManageEntity reqestJson){
        try {
            System.out.println(reqestJson.toString());
            gatewayManageService.addOrUpdateGateWayInfo(reqestJson);
            return HttpResult.ok();
        } catch (Exception e) {
            e.printStackTrace();
            return HttpResult.error(e.getMessage());
        }
    }

    /**
     * 删除网关/基表/基表参数信息
     * @param requestJson
     * @return
     */
    @PostMapping("/device/deleteDeviceInfo")
    public HttpResult deleteDeviceInfo(@RequestBody String requestJson){
        JSONObject jsonObject =JSONObject.parseObject(requestJson);
        System.out.println(jsonObject.get("deviceId"));
        try {
            deviceManageService.deleteDeviceInfo((Integer)jsonObject.get("deviceId"),jsonObject.get("deviceType").toString());
            return HttpResult.ok();
        } catch (Exception e) {
            e.printStackTrace();
            return HttpResult.error(e.getMessage());
        }
    }

    /**
     * 查询基表管理中的基表数据
     * @param requestJson
     * @return
     */
    @PostMapping("/baseMeter/getBaseMeterList")
    public HttpResult getBaseMeterList(@RequestBody String requestJson){
        JSONObject jsonObject =JSONObject.parseObject(requestJson);
        List<DeviceManageEntity> list = deviceManageService.getDeviceByOther(jsonObject.get("deviceNum").toString());
        JSONObject tableData = new JSONObject();
        tableData.put("tableData",list);
        return HttpResult.ok(tableData);
    }

    /**
     * 增加或更新基表信息数据
     * @param requestJson
     * @return
     */
    @PostMapping("/baseMeter/addOrUpdateBaseMeter")
    public HttpResult addOrUpdateBaseMeter(@RequestBody DeviceManageEntity requestJson){
        try {
            deviceManageService.addOrUpdateBaseMeter(requestJson);
            return HttpResult.ok();
        } catch (Exception e) {
            e.printStackTrace();
            return HttpResult.error("检查输入的设备码是否有误！");
        }
    }

    /**
     * 查询基表参数管理中的基表参数数据
     * @param requestJson
     * @return
     */
    @PostMapping("/baseMeterParam/getBaseMeterParamList")
    public HttpResult getBaseMeterParamList(@RequestBody String requestJson){
        JSONObject jsonObject =JSONObject.parseObject(requestJson);
        System.out.println(jsonObject.get("baseMeterType"));
        String baseMeterType = (String) jsonObject.get("baseMeterType");
        List<DeviceParameterEntity> list = deviceParamService.getBaseMeterParamList(baseMeterType);
        JSONObject tableData = new JSONObject();
        tableData.put("tableData",list);
        return HttpResult.ok(tableData);
    }

    /**
     * 添加或更新基表参数信息
     * @param requestJson
     * @return
     */
    @PostMapping("/baseMeterParam/addOrUpdateBaseMeterParam")
    public HttpResult addOrUpdateBaseMeterParam(@RequestBody DeviceParameterEntity requestJson){
        try {
            deviceParamService.addOrUpdateBaseMeterParam(requestJson);
            return HttpResult.ok();
        } catch (Exception e) {
            e.printStackTrace();
            return HttpResult.error(e.getMessage());
        }
    }


}
