package com.mh.user.controller;

import com.mh.common.http.HttpResult;
import com.mh.user.annotation.SysLogger;
import com.mh.user.entity.EnergyEntity;
import com.mh.user.model.EnergyModel;
import com.mh.user.model.EnergyParam;
import com.mh.user.service.BuildingService;
import com.mh.user.service.EnergyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("energy")
public class EnergyController {

    @Autowired
    EnergyService energyService;

    @Autowired
    BuildingService buildingService;

    @PostMapping("/save")
    public HttpResult saveEnergy(@RequestBody EnergyEntity energyEntity, @RequestParam(value = "type") int type){
       try{
           energyService.saveEnergy(energyEntity,type);
           return  HttpResult.ok();
       }catch (Exception e){
           return  HttpResult.error("保存出错!");
       }

    }

    @PostMapping("/update")
    public HttpResult updateEnergy(@RequestBody EnergyEntity energyEntity, @RequestParam(value = "type") int type){
        try{
            energyService.updateEnergy(energyEntity,type);
            return  HttpResult.ok();
        }catch (Exception e){
            return  HttpResult.error("修改出错!");
        }

    }

    @PostMapping("/delete")
    public HttpResult deleteEnergy(@RequestParam(value = "curDate") String curDate,
                                   @RequestParam(value = "buildingId") String buildingId,
                                   @RequestParam(value = "type") int type){
        try{
            energyService.deleteEnergy(curDate,buildingId,type);
            return  HttpResult.ok();
        }catch (Exception e){
            return  HttpResult.error("删除出错!");
        }

    }

    @SysLogger(value="中央热水",optDesc = "数据分析查询")
    @PostMapping("/query")
    public HttpResult queryEnergy(@RequestBody EnergyParam energyParam){
        try{
            List<EnergyEntity> list;
            list=energyService.queryEnergy(energyParam);
            int count=energyService.getEnergyCount(energyParam);
            return  HttpResult.ok(count,list);
        }catch (Exception e){
            e.printStackTrace();
            return  HttpResult.error("查询出错!");
        }
    }

    //主界面水、电、单耗查询
    @SysLogger(value="中央热水",optDesc = "主界面水、电、单耗查询")
    @PostMapping("/queryGroup")
    public HttpResult queryEnergyGroup(@RequestParam(value = "curDate",required = true) String curDate,
                                       @RequestParam(value = "type",required = true,defaultValue = "1") int type){
        try{
            List<EnergyEntity> list;
            list=energyService.queryEnergyGroup(curDate,type);
            return  HttpResult.ok(list);
        }catch (Exception e){
            e.printStackTrace();
            return  HttpResult.error("查询出错!");
        }

    }

    //查询每天的用量
    @SysLogger(value="中央热水",optDesc = "查询每天的用量")
    @PostMapping("/queryDay")
    public HttpResult queryDayEnergy(@RequestParam(value = "buildingId",required = false) String buildingId,
                                     @RequestParam(value = "startDate",required = false) String startDate,
                                     @RequestParam(value = "endDate",required = false) String endDate,
                                     @RequestParam(value = "page") int page,
                                     @RequestParam(value = "limit") int limit){
        try{
            List<EnergyEntity> list;
            list=energyService.queryDayEnergy(buildingId,startDate,endDate,page,limit);
            int count=energyService.getDayEnergyCount(buildingId,startDate,endDate,page,limit);
            return  HttpResult.ok(count,list);
        }catch (Exception e){
            e.printStackTrace();
            return  HttpResult.error("查询出错!");
        }
    }
}
