package com.mh.user.controller;

import com.mh.common.http.HttpResult;
import com.mh.common.page.PageRequest;
import com.mh.user.annotation.SysLogger;
import com.mh.user.constants.SysConstants;
import com.mh.user.mapper.SysUserRoleMapper;
import com.mh.user.model.SysRole;
import com.mh.user.model.SysUser;
import com.mh.user.model.SysUserRole;
import com.mh.user.service.SysRoleService;
import com.mh.user.service.SysUserService;
import com.mh.user.utils.PasswordUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 用户控制器
 * @author ljf
 * @date 2020-04-25
 */
@RestController
@RequestMapping("user")
public class SysUserController {

	@Autowired
	private SysUserService sysUserService;

	@Autowired
	private SysRoleService sysRoleService;

	@Autowired
	private SysUserRoleMapper sysUserRoleMapper;

	@SysLogger(value="用户管理",optDesc = "添加或者编辑用户")
	@PostMapping(value="/save")
	public HttpResult save(@RequestBody SysUser record) {
		SysUser user = sysUserService.findById(record.getId());
		if(user != null) {
			if(SysConstants.ADMIN.equalsIgnoreCase(user.getUserName())) {
				return HttpResult.error("超级管理员不允许修改!");
			}
		}
		if(record.getPassword() != null) {

			String salt = PasswordUtils.getSalt();
			if(user == null) {
				// 新增用户
				if(sysUserService.findByName(record.getUserName()) != null) {
					return HttpResult.error("用户名已存在!");
				}
				String password = PasswordUtils.encode(record.getPassword(), salt);
				record.setSalt(salt);
				record.setPassword(password);
				record.setStatus(Byte.parseByte("1"));
			} else {
				SysUser user1=sysUserService.findByName(record.getUserName());
				if(user1!=null && user1.getId()!=record.getId()){
					return HttpResult.error("修改的用户名不能与存在的用户名相同!");
				}
				// 修改用户, 且修改了密码
				if(!record.getPassword().equals(user.getPassword())) {
					String password = PasswordUtils.encode(record.getPassword(), salt);
					record.setSalt(salt);
					record.setPassword(password);
				}
			}
		}

		return HttpResult.ok(sysUserService.save(record));
	}

	@PostMapping(value="/delete")
	public HttpResult delete(@RequestBody List<SysUser> records) {
		for(SysUser record:records) {
			SysUser sysUser = sysUserService.findById(record.getId());
			if(sysUser != null && SysConstants.ADMIN.equalsIgnoreCase(sysUser.getUserName())) {
				return HttpResult.error("超级管理员不允许删除!");
			}
		}
		return HttpResult.ok(sysUserService.delete(records));
	}


	@SysLogger(value="用户管理",optDesc = "删除用户")
	@PostMapping(value="/deleteById")
	public HttpResult deleteById(@RequestBody SysUser record) {
		SysUser sysUser = sysUserService.findById(record.getId());
		if(sysUser != null && SysConstants.ADMIN.equalsIgnoreCase(sysUser.getUserName())) {
			return HttpResult.error("超级管理员不允许删除!");
		}else{
			return HttpResult.ok(sysUserService.delete(record));
		}
	}

	@GetMapping(value="/findByName")
	public HttpResult findByUserName(@RequestParam String name) {

		return HttpResult.ok(sysUserService.findByName(name));
	}


	@GetMapping(value="/findPermissions")
	public HttpResult findPermissions(@RequestParam String systemID,@RequestParam String name) {
		return HttpResult.ok(sysUserService.findPermissions(systemID,name));
	}

	@GetMapping(value="/findUserRoles")
	public HttpResult findUserRoles(@RequestParam Long userId) {
		return HttpResult.ok(sysUserService.findUserRoles(userId));
	}

	@PostMapping(value="/findPage")
	public HttpResult findPage(@RequestBody PageRequest pageRequest) {
		return HttpResult.ok(sysUserService.findPage(pageRequest));
	}

	@PostMapping(value="/queryUsers")
	public HttpResult queryUsers(@RequestParam(value = "userName", required = false) String userName,
								 @RequestParam int page,
								 @RequestParam int limit) {
		try{
			List<SysUser> list=sysUserService.queryUsers(userName,page,limit);
			int count=sysUserService.getCount(userName,page,limit);
			return HttpResult.ok(count,list);
		}catch (Exception e){
			return HttpResult.error();
		}

	}

}
