package com.mh.user.job;

import com.mh.user.service.chillers.DealDataService;
import com.mh.user.utils.ExchangeStringUtil;
import com.mh.user.utils.SimpleWeather;
import org.apache.log4j.Logger;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author ljf
 * @title ：定时处理采集回来的历史数据
 * @description ：
 * @updateTime 2020-07-28
 * @throws ：
 */
@Component
public class DealDataJob {

    private final DealDataService dealDataService;
    private final SimpleWeather simpleWeather=new SimpleWeather();
    private static final Logger log = Logger.getLogger(DealDataJob.class);
    public DealDataJob(DealDataService dealDataService) {
        this.dealDataService = dealDataService;
    }

    /**
     * 定时处理汇总数据：每2分钟处理一次，所有项目统一处理
     */
    @Scheduled(cron = "25 0/2 * * * ?") //每2分钟一次
    public void ProEnergy() {
        try {
            SimpleDateFormat sdf1=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date date=new Date();
            String curDate=sdf1.format(date);
//            dealDataService.proEnergy(curDate);//中央热水yyyy-MM-dd HH:mm:ss

            String m=curDate.substring(15,16);
            if (Integer.parseInt(m)==0){
                curDate=curDate.substring(0,16)+":00";
            }else if(Integer.parseInt(m)==2){
                curDate=curDate.substring(0,16)+":00";
            }else if(Integer.parseInt(m)==4){
                curDate=curDate.substring(0,16)+":00";
            }else if(Integer.parseInt(m)==6){
                curDate=curDate.substring(0,16)+":00";
            }else if(Integer.parseInt(m)==8){
                curDate=curDate.substring(0,16)+":00";
            }
            dealDataService.proEnergyData(curDate);//中央空调yyyy-MM-dd HH:mm:ss
            log.info("---------数据分析定时汇总，每二分钟！"+curDate);
        } catch (Exception e) {
//            e.printStackTrace();
        }
    }
    /**
     * 定时处理数据：每十五分钟处理一次
     */
    @Scheduled(cron = "0 0/16 * * * ?")
    public void dealData() {
        try {
            SimpleDateFormat sdf1=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date date=new Date();
            String curDate=sdf1.format(date);
            dealDataService.proDeviceState(curDate);
            log.info("---------设备状态定时汇总，每十五分钟！"+curDate);
        } catch (Exception e) {
            log.error("设备状态定时汇总异常", e);
        }
    }

    /**
     * 定时处理数据：每分钟处理一次，单个项目分别处理
     */
    @Scheduled(cron = "0 0/1 * * * ?")
    public void proDataResult() {
        try {
            String curDate= ExchangeStringUtil.dateTime(1,"");
            List<String> list=dealDataService.queryProjectId("1");
            if (!list.isEmpty()){
                for(String projectID:list){
                    dealDataService.proDataResult(curDate,projectID); //1珠江酒店，2广合，3广大
                }
            }
            log.info("---------能效监测定时汇总，每一分钟！"+curDate);
        } catch (Exception e) {
            log.error("能效监测定时汇总异常", e);
        }
    }

    /**
     * 定时处理数据：获取环境温度和湿度,所有项目共有
     */
    @Scheduled(cron = "0 0 0/1 * * ?") //1小时保存一次温度、湿度变化值
//    @Scheduled(cron = "0 0/1 * * * ?") //1分钟
    public void saveTempHumidity() {
        try {
            SimpleDateFormat sdf1=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date date=new Date();
            Map<String, Object> map= SimpleWeather.queryWeather("广州");
            String curDate=sdf1.format(date);
            String temperature=map.get("temperature").toString();
            String humidity=map.get("humidity").toString();
            dealDataService.saveTempHumidity(curDate,temperature,humidity);
            log.info(curDate+","+temperature+"℃,"+humidity+"%");
        } catch (Exception e) {
//            e.printStackTrace();
            log.error("调用获取环境温湿度接口失败！", e);
        }
    }

    /**
     * 定时处理数据：每天处理一次
     */
    @Scheduled(cron = "0 0 0 1/1 * ?")
//    @Scheduled(cron = "0 0/1 * * * ?") //1分钟
    public void proEnergyDaySum() {
        try {
            SimpleDateFormat sdf1=new SimpleDateFormat("yyyy-MM-dd");
            Date date=new Date();
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.add(Calendar.DATE, -1);// 减少一天
            String curDate=sdf1.format(calendar.getTime());
            List<String> list=dealDataService.queryProjectId("2");
            if (!list.isEmpty()){
                for(String projectID:list){
                    dealDataService.proEnergyDaySum(curDate,projectID);
                }
            }
            log.info("---------中央热水生产概况汇总，每一小时！"+curDate);
        } catch (Exception e) {
            log.error("中央热水生产概况定时汇总异常", e);
        }
    }


    /**
     * 定时删除历史流水记录（删除前三个月的记录）
     */
//    @Scheduled(cron = "0 0 0 1 1/1 ? ")
//    public void deleteDataHistory() {
//        try {
//            dealDataService.deleteChillersDataHistory();
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//    }

}
