package com.mh.user.mapper;

import com.mh.user.entity.AlarmInfoEntity;
import com.mh.user.entity.AlarmValueSetEntity;
import com.mh.user.mapper.provider.AlarmInfoProvider;
import org.apache.ibatis.annotations.*;
import org.springframework.security.core.parameters.P;

import java.util.List;

@Mapper
public interface AlarmInfoMapper {

    /**
     * 报警管理
     * 保存报警信息
     * @param alarmInfoEntity
     */
    @Insert("insert into alarm_info(building_id,building_name,alarm_time,alarm_device,alarm_type,operate) values (" +
            "#{buildingId},#{buildingName},#{alarmTime},#{alarmDevice},#{alarmType},#{operate})")
    void saveAlarmInfo(AlarmInfoEntity alarmInfoEntity);

    /**
     * 报警管理
     * 根据日期修改报警信息
     * @param alarmInfoEntity
     */
    @Update("<script>" +
            "  update Alarm_Info set  " +
            "  <if test='buildingId!=null'>  building_id = #{buildingId} </if>" +
            "  <if test='buildingName!=null'> , building_name = #{buildingName} </if>" +
            "  <if test='alarmTime!=null'> , alarm_time = #{alarmTime} </if>" +
            "  <if test='alarmDevice!=null'> , alarm_device = #{alarmDevice} </if>" +
            "  <if test='alarmType!=null'> , alarm_type = #{alarmType} </if>" +
            "  <if test='operate!=null'> , operate = #{operate} </if>" +
            "   where id = #{id} " +
            "</script>")
    void updateAlarmInfo(AlarmInfoEntity alarmInfoEntity);

    /**
     * 报警管理
     * 根据日期、设备、楼栋查询报警信息
     * @param alarmTime
     * @param deviceType
     * @param buildingId
     * @param page
     * @param limit
     * @return
     */
    @SelectProvider(type = AlarmInfoProvider.class,method = "queryAlarmInfo")
    @Results({
            @Result(column = "building_id",property = "buildingId" ),
            @Result(column = "building_name", property = "buildingName"),
            @Result(column = "alarm_time", property = "alarmTime"),
            @Result(column = "device_type", property = "deviceType"),
            @Result(column = "alarm_type", property = "alarmType"),
            @Result(column = "deal_state", property = "dealState"),
            @Result(column = "id", property = "id"),
            @Result(column = "device_addr", property = "deviceAddr")
    })
    List<AlarmInfoEntity> queryAlarmInfo(@Param("alarmTime") String alarmTime,
                                         @Param("deviceType") String deviceType,
                                         @Param("buildingId") String buildingId,
                                         @Param("dealState") String dealState,
                                         @Param("page") int page,
                                         @Param("limit") int limit);

    /**
     * 报警管理
     * 根据日期、设备类型查询记录数
     * @return
     */
    @SelectProvider(type = AlarmInfoProvider.class,method = "getAlarmInfoCount")
    int getAlarmInfoCount(@Param("alarmTime") String alarmTime,
                          @Param("deviceType") String deviceType,
                          @Param("buildingId") String buildingId,
                          @Param("dealState") String dealState,
                          @Param("page") int page,
                          @Param("limit") int limit);

    /**
     * 报警管理
     * 根据报警编号删除报警信息
     * @param id
     */
    @Delete("delete from Alarm_Info where id=#{id}")
    void deleteAlarmInfo(@Param("id") String id);
    ///////////////////////////////////////
    //警报设置
    @Insert("insert into alarm_value_set(water_level,water_temp,elect_water,building_id) values (" +
            "#{waterLevel},#{waterTemp},#{electWater},#{buildingId})")
    void saveAlarmValueSet(AlarmValueSetEntity alarmValueSetEntity);

    //修改
    @Update("<script>" +
            "  update alarm_value_set set  " +
            "  <if test='waterLevel!=null'>  water_level = #{waterLevel} </if>" +
            "  <if test='waterTemp!=null'>  water_temp = #{waterTemp} </if>" +
            "  <if test='electWater!=null'> , elect_water = #{electWater} </if>" +
            "   where building_id = #{buildingId} " +
            "</script>")
    void updateAlarmValueSet(AlarmValueSetEntity alarmValueSetEntity);

    //查询报警值设置记录数
    @Select("select count(*) from alarm_value_set  where  building_id=#{buildingId}")
    int getAlarmValueSetCount(@Param("buildingId") String buildingId);

    //删除记录
    @Delete("delete from alarm_value_set")
    void delAlarmValueSet();

    //////////////////////////////////////////
    //修改状态
    @Update("update alarm_info set deal_state=#{dealState} where id=#{id}")
    void updateDealState(@Param("id") String id,@Param("dealState") String dealState);

}
