package com.mh.user.mapper;

import com.mh.user.entity.BuildingEntity;
import com.mh.user.mapper.provider.BuildingProvider;
import com.mh.user.model.BuildingModel;
import org.apache.ibatis.annotations.*;

import java.util.List;

@Mapper
public interface BuildingMapper {

    /**
     * 楼栋管理模块
     * 保存楼栋信息
     * @param buildingEntity
     */
    @Insert("insert into building(building_id,building_name,levels_count,begin_level,house_count,bed_count,check_in_count,area_id,remarks) values (" +
            " #{buildingId},#{buildingName},#{levelsCount},#{beginLevel},#{houseCount},#{bedCount},#{checkInCount},#{areaId},#{remarks})")
    int saveBuilding(BuildingEntity buildingEntity);

    /**
     * 楼栋管理模块
     * 保存楼栋信息
     * @param buildingEntity
     */
    @Update("<script>" +
            "  update building set  " +
            "  <if test='buildingName!=null'>  building_name = #{buildingName} </if>" +
            "  <if test='levelsCount!=null'> , levels_count = #{levelsCount} </if>" +
            "  <if test='beginLevel!=null'> , begin_level = #{beginLevel} </if>" +
            "  <if test='houseCount!=null'> , house_count = #{houseCount} </if>" +
            "  <if test='bedCount!=null'> , bed_count = #{bedCount} </if>" +
            "  <if test='checkInCount!=null'> , check_in_count = #{checkInCount} </if>" +
            "  <if test='areaId!=null'> , area_id = #{areaId} </if>" +
            "  <if test='remarks!=null'> , remarks = #{remarks} </if>" +
            "   where id = #{id} " +
            "</script>")
    int updateBuilding(BuildingEntity buildingEntity);

    /**
     * 楼栋管理模块：
     * 查询楼栋资料
     * @param page
     * @param limit
     * @return
     */
    @Results(value = {
            @Result(property="buildingId",column="building_id"),
            @Result(property="buildingName",column="building_name"),
            @Result(property="levelsCount",column="levels_count"),
            @Result(property="beginLevel",column="begin_level"),
            @Result(property ="houseCount",column ="house_count"),
            @Result(property ="bedCount",column ="bed_count"),
            @Result(property="checkInCount",column="check_in_count"),
            @Result(property="areaId",column="area_id"),
            @Result(property="remarks",column="remarks")
    })

    @SelectProvider(type = BuildingProvider.class,method = "queryBuilding")
    List<BuildingEntity> queryBuilding(@Param("buildingId") String buildingId,@Param("page") int page, @Param("limit") int limit);

    /**
     * 楼栋管理模块：
     * 获取楼栋信息查询的总条数
     * @return
     */
    @SelectProvider(type = BuildingProvider.class,method = "getCount")
    int getCount(@Param("buildingId") String buildingId,@Param("page") int page, @Param("limit") int limit);

    //查询所有楼栋编号和名称
    @Results(value = {
            @Result(property="buildingId",column="id"),
            @Result(property="buildingName",column="building_name")
    })
    @Select("select * from building ")
    List<BuildingModel> selectBuildingName();

    //查询楼栋编号
//    @Results(value = {
//            @Result(property="buildingId",column="building_id")
//    })
    @Select("select id from building where  building_name=#{buildingName}")
    String selectBuildingId(@Param("buildingName") String buildingName);
    /**
     * 楼栋管理模块：
     * 根据名称删除楼栋信息
     * @param id
     */
    @Delete("delete from building where id=#{id}")
    int deleteBuilding(String id);

}
