package com.mh.user.mapper;

import com.mh.user.entity.DataResultClEntity;
import com.mh.user.entity.DataResultEntity;
import com.mh.user.entity.DataResultChEntity;
import com.mh.user.mapper.provider.DataResultProvider;
import com.mh.user.model.ChillerModel;
import org.apache.ibatis.annotations.*;

import java.util.List;

@Mapper
public interface DataResultMapper {

    //保存水电表等数据
    @Insert("insert into data_result(device_addr,deviceType,project_id,lastValue,lastDate,cur_value,cur_date,ratio,calc_value,grade) values (" +
            " #{deviceAddr},#{deviceType},#{projectID},#{lastValue},#{lastDate},#{curValue},#{curDate},#{ratio},#{calcValue},#{grade})")
    void saveDataResult(DataResultEntity dataResultEntity);

    @Update("<script>" +
            "  update data_result set  " +
            "  <if test='deviceAddr!=null'>  device_addr = #{deviceAddr} </if>" +
            "  <if test='deviceType!=null'> , deviceType = #{deviceType} </if>" +
            "  <if test='projectID!=null'> , project_id = #{projectID} </if>" +
            "  <if test='lastValue!=null'> , lastValue = #{lastValue} </if>" +
            "  <if test='lastDate!=null'> , lastDate = #{lastDate} </if>" +
            "  <if test='curValue!=null'> , cur_value = #{curValue} </if>" +
            "  <if test='ratio!=null'> , ratio = #{ratio} </if>" +
            "  <if test='calcValue!=null'> , calc_value = #{calcValue} </if>" +
            "  <if test='grade!=null'> , grade = #{grade} </if>" +
            "   where cur_date=#{curDate} and  device_addr=#{deviceAddr}  and  deviceType=#{deviceType}" +
            "</script>")
    void updateDataResult(DataResultEntity dataResultEntity);

    @Results(id="rs", value = {
            @Result(property="deviceAddr",column="device_addr"),
            @Result(property="deviceType",column="deviceType"),
            @Result(property="projectID",column="project_id"),
            @Result(property="lastValue",column="lastValue"),
            @Result(property ="lastDate",column ="lastDate"),
            @Result(property ="curValue",column ="cur_value"),
            @Result(property="curDate",column="cur_date"),
            @Result(property="ratio",column="ratio"),
            @Result(property="calcValue",column="calc_Value"),
            @Result(property="grade",column="grade")
    })
    @SelectProvider(type = DataResultProvider.class,method = "queryDataResult")
    List<DataResultEntity> queryDataResult(@Param("projectID") String projectID,
                                           @Param("startDate") String startDate,
                                           @Param("endDate") String endDate,
                                           @Param("deviceType") String deviceType,
                                           @Param("page") int page,
                                           @Param("limit") int limit);


    @SelectProvider(type = DataResultProvider.class,method = "getDataResultCount")
    int getDataResultCount(@Param("projectID") String projectID,
                             @Param("startDate") String startDate,
                             @Param("endDate") String endDate,
                             @Param("deviceType") String deviceType,
                             @Param("page") int page,
                             @Param("limit") int limit);


    //查询单条记录
    @ResultMap("rs")
    @Select("select * from  data_result where cur_date=#{curDate} and  device_addr=#{deviceAddr}  and  deviceType=#{deviceType} ")
    DataResultEntity selectDataResult(@Param("curDate") String curDate,
                         @Param("deviceAddr") String deviceAddr,
                         @Param("deviceType") String deviceType);

    //查询是否存在记录
    @Select("select count(*) from  data_result  where cur_date=#{curDate} and  device_addr=#{deviceAddr}  and  deviceType=#{deviceType} ")
    int selectDataResultCount(@Param("curDate") String curDate,
                         @Param("deviceAddr") String deviceAddr,
                         @Param("deviceType") String deviceType);

    //----------------------------------------------------------------------------------------------
    //保存冷量计数据
    @Insert("insert into data_result_cl(device_addr,deviceType,project_id,lastValue,lastDate,cur_value,cur_date,ratio,calc_value,grade,registerAddr,registerName) values (" +
            " #{deviceAddr},#{deviceType},#{projectID},#{lastValue},#{lastDate},#{curValue},#{curDate},#{ratio},#{calcValue},#{grade},#{registerAddr},#{registerName})")
    void saveDataResultCl(DataResultClEntity dataResultClEntity);

    @Update("<script>" +
            "  update data_result_cl set  " +
            "  <if test='deviceAddr!=null'>  device_addr = #{deviceAddr} </if>" +
            "  <if test='deviceType!=null'> , deviceType = #{deviceType} </if>" +
            "  <if test='projectID!=null'> , project_id = #{projectID} </if>" +
            "  <if test='lastValue!=null'> , lastValue = #{lastValue} </if>" +
            "  <if test='lastDate!=null'> , lastDate = #{lastDate} </if>" +
            "  <if test='curValue!=null'> , cur_value = #{curValue} </if>" +
            "  <if test='ratio!=null'> , ratio = #{ratio} </if>" +
            "  <if test='calcValue!=null'> , calc_value = #{calcValue} </if>" +
            "  <if test='grade!=null'> , grade = #{grade} </if>" +
            "  <if test='registerAddr!=null'> , registerAddr = #{registerAddr} </if>" +
            "  <if test='registerName!=null'> , registerName = #{registerName} </if>" +
            "   where cur_date=#{curDate} and  device_addr=#{deviceAddr}  and  deviceType=#{deviceType}" +
            "</script>")
    void updateDataResultCl(DataResultClEntity dataResultClEntity);

    @Insert("insert into data_result_cl_bak(device_addr,deviceType,project_id,lastValue,lastDate,cur_value,cur_date,ratio,calc_value,grade,registerAddr,registerName) values (" +
            " #{deviceAddr},#{deviceType},#{projectID},#{lastValue},#{lastDate},#{curValue},#{curDate},#{ratio},#{calcValue},#{grade},#{registerAddr},#{registerName})")
    void saveDataResultCl_bak(DataResultClEntity dataResultClEntity);

    //----------------------------------------------------------------------------------------------
    //保存冷量计数据
    @Insert("insert into data_result_ch(deviceAddr,deviceType,funCode,registerAddr,registerName,curValue,curDate,projectID,grade) values (" +
            " #{deviceAddr},#{deviceType},#{funCode},#{registerAddr},#{registerName},#{curValue},#{curDate},#{projectID},#{grade})")
    void saveDataResultCh(DataResultChEntity dataResultChEntity);

    @Update("<script>" +
            "  update data_result_ch set  " +
            "  <if test='deviceAddr!=null'>  deviceAddr = #{deviceAddr} </if>" +
            "  <if test='deviceType!=null'> , deviceType = #{deviceType} </if>" +
            "  <if test='funCode!=null'> , funCode = #{funCode} </if>" +
            "  <if test='registerAddr!=null'> , registerAddr = #{registerAddr} </if>" +
            "  <if test='registerName!=null'> , registerName = #{registerName} </if>" +
            "  <if test='curValue!=null'> , curValue = #{curValue} </if>" +
            "  <if test='projectID!=null'> , projectID = #{projectID} </if>" +
            "  <if test='grade!=null'> , grade = #{grade} </if>" +
            "   where curDate=#{curDate} and  deviceAddr=#{deviceAddr}  and  deviceType=#{deviceType}" +
            "</script>")
    void updateDataResultCh(DataResultChEntity dataResultChEntity);

    @Select("select count(*) from  data_result_ch " +
            " where  curDate=#{curDate}  " +
            " and  deviceAddr=#{deviceAddr}  " +
            " and  registerAddr=#{registerAddr}  " +
            " and  projectID=#{projectID} " +
            " and grade = #{grade} ")
    int selectDataResultChCount(@Param("curDate") String curDate,
                                @Param("deviceAddr") String deviceAddr,
                                @Param("registerAddr") String registerAddr,
                                @Param("projectID") String projectID,
                                @Param("grade") int grade);

    //----------------------------------------------------------------------------------------------------
    @Results(id="rsc", value = {
            @Result(property="deviceAddr",column="deviceAddr"),
            @Result(property="deviceType",column="deviceType"),
            @Result(property="curDate",column="curDate"),
            @Result(property ="curValue",column ="curValue"),
            @Result(property="funCode",column="funCode"),
            @Result(property="registerAddr",column="registerAddr"),
            @Result(property="registerName",column="registerName"),
            @Result(property="grade",column="grade"),
            @Result(property="projectID",column="projectID"),
            @Result(property="projectName",column="project_name")
    })
    @SelectProvider(type = DataResultProvider.class,method = "queryDataResultOneMi")
    List<DataResultChEntity> queryDataResultOneMi(@Param("projectID") String projectID,
                                                  @Param("startDate") String startDate,
                                                  @Param("curDate") String curDate,
                                                  @Param("page") int page,
                                                  @Param("limit") int limit);

    //查询记录数
//    @Select("select count(*) from  data_result_one_mi  where  projectID=#{projectID}  and  curDate>=#{startDate} and curDate<=#{curDate} ")
    @SelectProvider(type = DataResultProvider.class,method = "dataResultOneMiCount")
    int dataResultOneMiCount(@Param("projectID") String projectID,@Param("startDate") String startDate,@Param("curDate") String curDate);

    @ResultMap("rsc")
    @SelectProvider(type = DataResultProvider.class,method = "queryDataResultFiveMi")
    List<DataResultChEntity> queryDataResultFiveMi(@Param("projectID") String projectID,
                                                   @Param("startDate") String startDate,
                                                   @Param("curDate") String curDate,
                                                   @Param("page") int page,
                                                   @Param("limit") int limit);


//    @Select("select  count(*) from  data_result_five_mi  where  projectID=#{projectID} and  curDate>=#{startDate} and curDate<=#{curDate} ")
    @SelectProvider(type = DataResultProvider.class,method = "dataResultFiveMiCount")
    int dataResultFiveMiCount(@Param("projectID") String projectID,@Param("startDate") String startDate,@Param("curDate") String curDate);



    @ResultMap("rsc")
    @SelectProvider(type = DataResultProvider.class,method = "queryDataResultFifteenMi")
    List<DataResultChEntity> queryDataResultFifteenMi(@Param("projectID") String projectID,
                                                      @Param("startDate") String startDate,
                                                      @Param("curDate") String curDate,
                                                      @Param("page") int page,
                                                      @Param("limit") int limit);

    //@Select("select count(*) from  data_result_fifteen_mi  where  projectID=#{projectID} and  curDate>=#{startDate} and curDate<=#{curDate} ")
    @SelectProvider(type = DataResultProvider.class,method = "dataResultFifteenMiCount")
    int dataResultFifteenMiCount(@Param("projectID") String projectID,@Param("startDate") String startDate,@Param("curDate") String curDate);

    @ResultMap("rsc")
    @Select("select t1.projectID,sum(cast (t1.curValue AS decimal(18,2)))as curValue,t1.curDate,t2.project_name from  data_result_ch t1 join project_info t2 on t1.projectID=t2.id " +
            "where  t1.projectID=#{projectID}  and t1.curDate>=#{startDate}  and  t1.curDate<=#{curDate} group by t1.curDate,t1.projectID,t2.project_name  order by t1.curDate  ")
    List<DataResultChEntity> queryDataResultCh(@Param("projectID") String projectID,@Param("startDate") String startDate,@Param("curDate") String curDate);

    //保存冷水机数据
    @Insert("insert into data_result_chiller(deviceAddr,deviceType,funCode,registerAddr,registerName,curValue,curDate,projectID,grade) values (" +
            " #{deviceAddr},#{deviceType},#{funCode},#{registerAddr},#{registerName},#{curValue},#{curDate},#{projectID},#{grade})")
    void saveDataResultChiller(DataResultChEntity dataResultChEntity);

    @Update("<script>" +
            "  update data_result_chiller set  " +
            "  <if test='deviceAddr!=null'>  deviceAddr = #{deviceAddr} </if>" +
            "  <if test='deviceType!=null'> , deviceType = #{deviceType} </if>" +
            "  <if test='funCode!=null'> , funCode = #{funCode} </if>" +
            "  <if test='registerAddr!=null'> , registerAddr = #{registerAddr} </if>" +
            "  <if test='registerName!=null'> , registerName = #{registerName} </if>" +
            "  <if test='curValue!=null'> , curValue = #{curValue} </if>" +
            "  <if test='projectID!=null'> , projectID = #{projectID} </if>" +
            "  <if test='grade!=null'> , grade = #{grade} </if>" +
            "   where curDate=#{curDate}  and  deviceAddr=#{deviceAddr}  and  registerAddr=#{registerAddr}  and  projectID=#{projectID} " +
            "</script>")
    void updateDataResultChiller(DataResultChEntity dataResultChEntity);

    @Select("select count(*) from  data_result_chiller where curDate=#{curDate}  " +
            " and  deviceAddr=#{deviceAddr}  " +
            " and  registerAddr=#{registerAddr}  " +
            " and  projectID=#{projectID} " +
            " and funCode = #{funCode} ")
    int selectDataResultChillerCount(@Param("curDate") String curDate,
                                     @Param("deviceAddr") String deviceAddr,
                                     @Param("registerAddr") String registerAddr,
                                     @Param("projectID") String projectID,
                                     @Param("funCode") String funCode);


    @Insert("insert into data_result_now(deviceAddr,deviceType,funCode,registerAddr,registerName,curValue,curDate,projectID,grade) values (" +
            " #{deviceAddr},#{deviceType},#{funCode},#{registerAddr},#{registerName},#{curValue},#{curDate},#{projectID},#{grade})")
    void saveDataResultNow(DataResultChEntity dataResultChEntity);

    @Delete("delete  from  data_result_now  where  deviceAddr=#{deviceAddr}  and  deviceType = #{deviceType}  and  registerAddr=#{registerAddr}  and  projectID=#{projectID}")
    void deleteDataResultNow(@Param("deviceAddr") String deviceAddr,
                             @Param("deviceType") String deviceType,
                             @Param("registerAddr") String registerAddr,
                             @Param("projectID") String projectID);

    //实时参数读取
    @Select("select * from  data_result_now  where  deviceAddr=#{deviceAddr}  and  deviceType=#{deviceType}  and  projectID=#{projectID} ")
    List<DataResultChEntity> selectDataResultNow(@Param("deviceAddr") String deviceAddr,
                                                 @Param("deviceType") String deviceType,
                                                 @Param("projectID") String projectID);

    @Results({
            @Result(property="deviceAddr",column="deviceAddr"),
            @Result(property="deviceType",column="deviceType"),
            @Result(property="curDate",column="curDate"),
            @Result(property ="curValue",column ="curValue"),
            @Result(property="funCode",column="funCode"),
            @Result(property="registerAddr",column="registerAddr"),
            @Result(property="registerName",column="registerName"),
            @Result(property="grade",column="grade"),
            @Result(property="projectID",column="projectID"),
            @Result(property="projectName",column="project_name")
    })
    @SelectProvider(type = DataResultProvider.class,method = "queryDataResultChiller")
    List<DataResultChEntity> queryDataResultChiller(@Param("projectID") String projectID,
                                                    @Param("deviceAddr") String deviceAddr,
                                                    @Param("registerName") String registerName,
                                                    @Param("startDate") String startDate,
                                                    @Param("curDate") String curDate,
                                                    @Param("page") int page,
                                                    @Param("limit") int limit);

    @SelectProvider(type = DataResultProvider.class,method = "dataResultChillerCount")
    int dataResultChillerCount(@Param("projectID") String projectID,
                               @Param("deviceAddr") String deviceAddr,
                               @Param("registerName") String registerName,
                               @Param("startDate") String startDate,
                               @Param("curDate") String curDate);


    //保存冷水机参数数据
    @Insert("insert into data_chiller(deviceAddr,deviceName,runState,setPoint,enterChw,leaveChw,enterCow,leaveCow,power,ratedPower,chillerAmps,refrigeration,ratedRef,chwFlow,ApproachCow,ApproachChw,cop,ratedCop,copyDate) values (" +
            " #{deviceAddr},#{deviceName},#{projectID},#{lastValue},#{lastDate},#{curValue},#{curDate},#{ratio},#{calcValue},#{grade})")
    void saveDataChiller(ChillerModel chillerModel);

    @Select("select count(1) from data_result_cl_bak " +
            " where cur_date = #{curDate} " +
            " and device_addr = #{deviceAddr}  " +
            " and registerAddr = #{registerAddr} " +
            " and project_id = #{projectId} ")
    int selectDataResultChBakCount(@Param("curDate") String curDate,
                                   @Param("deviceAddr") String deviceAddr,
                                   @Param("registerAddr") String registerAddr,
                                   @Param("projectId") String projectId);
}
