package com.mh.user.mapper;

import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.entity.OrderMessageEntity;
import org.apache.ibatis.annotations.*;

import java.util.List;

@Mapper
public interface DeviceCodeParamMapper {

    //插入指令参数
    @Insert("insert into device_code_param(device_addr,device_name,device_type,data_com,baudrate,brand,fun_code,register_addr,create_time)" +
            "values(#{deviceAddr},#{deviceName},#{deviceType},#{dataCom},#{baudrate},#{brand},#{funCode},#{registerAddr},getDate())")
    void insertDeviceCodeParam(DeviceCodeParamEntity deviceCodeParamEntity);

    //查询指令参数,传入IP和端口
    @Results(id="rs", value = {
            @Result(property="id",column="id"),
            @Result(property="deviceAddr",column="device_addr"),
            @Result(property="deviceType",column="deviceType"),
            @Result(property="strData",column="strData"),
            @Result(property="dataPort",column="dataPort"),
            @Result(property="baudRate",column="baudrate"),
            @Result(property ="brand",column ="brand"),
            @Result(property ="funCode",column ="fun_code"),
            @Result(property ="registerAddr",column ="register_addr"),
            @Result(property ="registerName",column ="registerName"),
            @Result(property ="deviceName",column ="deviceName"),
            @Result(property ="createTime",column ="createTime"),
            @Result(property ="grade",column ="grade"),
            @Result(property ="digit",column ="digit"),
            @Result(property ="projectID",column ="project_id")
    })
    @Select("select id, " +
            "device_addr, " +
            "deviceType, " +
            "strData, " +
            "dataPort, " +
            "baudrate, " +
            "brand, " +
            "fun_code, " +
            "register_addr, " +
            "deviceName, " +
            "createTime, " +
            "grade, " +
            "project_id, " +
            "digit, " +
            "registerName " +
            "   from  device_code_param  where dataPort=#{gatewayPort} order by deviceType ")
    List<DeviceCodeParamEntity> queryCodeParam(@Param("gatewayPort") String gatewayPort);

    //查询指令参数,传入串口
    @ResultMap("rs")
    @Select("select t1.* from  device_code_param t1 join gateway_manage t2  on t1.data_com=t2.data_com  where t2.data_com=#{comName}")
    List<DeviceCodeParamEntity> queryCodeParam2(@Param("comName") String comName);

    //查询所有指令参数
    @ResultMap("rs")
    @Select("select * from  device_code_param order by data_com,deviceType")
    List<DeviceCodeParamEntity> queryCodeParam3();

    //查询所有指令参数
    @ResultMap("rs")
    @Select("select * from  device_code_param  where  project_id=#{projectID} and  deviceType=#{deviceType} and  device_addr=#{deviceAddr} and register_addr=#{registerAddr}")
    DeviceCodeParamEntity queryDeviceCodeParam(@Param("projectID") String projectID,
                                               @Param("deviceType") String deviceType,
                                               @Param("deviceAddr") String deviceAddr,
                                               @Param("registerAddr") String registerAddr);

    //查询记录数
    @Select("select  count(*) from device_code_param  where  project_id=#{projectId} ")
    int queryCount(@Param("projectId") String projectID);

    //查询记录数,所有项目
    @Select("select  count(*) from device_code_param ")
    int queryCount2();

    // 批量插入生成指令参数入数据库
    @Insert("<script>" +
            "insert into device_code_param(" +
            "device_addr, " +
            "deviceName, " +
            "deviceType, " +
            "dataPort, " +
            "baudRate, " +
            "brand, " +
            "fun_code, " +
            "register_addr," +
            "registerName," +
            "createTime," +
            "project_id," +
            "parity," +
            "digit," +
            "grade," +
            "strData" +
            ")" +
            "values " +
            "<foreach collection='deviceCodeParamEntityList' item='item' separator=','>" +
            "(#{item.deviceAddr},#{item.deviceName},#{item.deviceType},#{item.dataPort},#{item.baudRate},#{item.brand},#{item.funCode},#{item.registerAddr},#{item.registerName},getDate(),#{item.projectID},#{item.parity},#{item.digit},#{item.grade},#{item.strData})" +
            "</foreach>" +
            "</script>")
    void insertDeviceCodeParamList(@Param("deviceCodeParamEntityList") List<DeviceCodeParamEntity> deviceCodeParamEntityList);

    //查询插入
    @Insert("insert into device_code_param(device_addr,deviceName,deviceType,dataPort,baudRate,parity,brand,createTime,project_id,digit,grade) select device_addr,deviceName,deviceType,dataPort,baudRate,parity,brand,getDate(),project_id,2,grade  from  device_code " +
            " where  (deviceType='电表' or deviceType='水表') and  project_id=#{ProjectID} ")
    void selectInsertDeviceCodeParam(@Param("ProjectID") String ProjectID);
}
