package com.mh.user.mapper.chillers;

import com.mh.user.entity.ChillersEntity;
import com.mh.user.entity.OrderMessageEntity;
import org.apache.ibatis.annotations.*;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 * @author ljf
 * @title ：
 * @description ：关于冷水机组指令控制的持久化层，mapper层
 * @updateTime 2020-05-20
 * @throws ：
 */
@Component
@Mapper
public interface ChillerControlOrderMapper {

    // 更新基表管理状态值
    @Update("update chillers_register_address set last_value = #{dataStr} and last_time = getdate() where" +
            " register_address = #{registerAddr}")
    void updateChillersRegisterAddress(@Param("dataStr") String dataStr, @Param("registerAddr") String registerAddr);

    // 更新发送状态status: 1(发送成功)，0(发送失败)
    @Update("update order_message set send_time = getdate(), send_num = #{sendNum}, status = #{status} " +
            " where register_address = #{registerAddr} and create_time = #{createTime}")
    void updateOrderMessage(OrderMessageEntity orderMessageEntity);

    // 通过父ID查询出对应的寄存器信息值
    @Select("select a.id, a.register_address, a.register_name, a.other_name, " +
            " (select top 1 gateway_ip from gateway_manage as b where b.data_com = a.data_com) as IP," +
            " (select top 1 port from gateway_manage as b where b.data_com = a.data_com) as port, " +
            " (select top 1 ddc_addr from chillers_register_address as c where c.id = #{id}) as ddc_addr " +
            " from chillers_register_address as a where parent_id = #{id}")
    @Results({
            @Result(column = "id", property = "id"),
            @Result(column = "register_address", property = "registerAddress"),
            @Result(column = "register_name", property = "registerName"),
            @Result(column = "other_name", property = "otherName"),
            @Result(column = "IP", property = "IP"),
            @Result(column = "port", property = "port"),
            @Result(column = "ddc_addr", property = "ddcAddr")
    })
    ChillersEntity queryChiller(@Param("id") int id);

    // 通过父ID查询出对应的寄存器信息值
    @Select("select a.id, a.register_address, a.register_name, a.other_name, " +
            " (select top 1 gateway_ip from gateway_manage as b where b.data_com = a.data_com) as IP," +
            " (select top 1 port from gateway_manage as b where b.data_com = a.data_com) as port, " +
            " (select top 1 ddc_addr from chillers_register_address as c where c.id = 19) as ddc_addr " +
            " from chillers_register_address as a where id = #{id}")
    @Results({
            @Result(column = "id", property = "id"),
            @Result(column = "register_address", property = "registerAddress"),
            @Result(column = "register_name", property = "registerName"),
            @Result(column = "other_name", property = "otherName"),
            @Result(column = "IP", property = "IP"),
            @Result(column = "port", property = "port"),
            @Result(column = "ddc_addr", property = "ddcAddr")
    })
    ChillersEntity queryChillerById(@Param("id") int id);

    // 批量插入生成的采集指令入数据库
    @Insert("<script>" +
            "insert into order_message(" +
            "register_address, " +
            "register_name, " +
            "other_name, " +
            "order_str, " +
            "create_time, " +
            "send_time, " +
            "send_num," +
            "status," +
            "grade" +
            ")" +
            "values " +
            "<foreach collection='orderMessageEntityList' item='item' separator=','>" +
            "(#{item.registerAddr},#{item.registerName},#{item.otherName},#{item.orderStr},#{item.createTime},#{item.sendTime},#{item.sendNum},#{item.status},0)" +
            "</foreach>" +
            "</script>")
    void batchInsertOrderMessage(@Param("orderMessageEntityList") List<OrderMessageEntity> orderMessageEntityList);

    // 查询最新关闭冷却塔指令的时间
    @Select("select top 1 send_time from order_message where status = 1 and register_name like '%冷却塔%' and order_str like '%0000' order by send_time desc")
    Date queryCloseChilledLastTime();

    // 发送指令之后，更新对应的地址对应的状态值
    @Update("update chillers_register_address set last_value = right(#{orderStr},2), last_time = getdate() " +
            " where grade = 3 and register_address = #{registerAddr} and ddc_addr is null")
    void updateChillersBySendDDC(OrderMessageEntity orderMessageEntity);

    // 发送指令之后，更新对应的地址对应的状态值
    @Update("update chillers_register_address set last_value = #{orderStr}, last_time = getdate() " +
            " where grade = 3 and register_address = #{registerAddr} and ddc_addr is null")
    void updateChillersBySendDDC1(OrderMessageEntity orderMessageEntity);

}
