package com.mh.user.mapper.chillers;

import com.mh.user.entity.ChillersEntity;
import com.mh.user.entity.DeviceManageEntity;
import com.mh.user.entity.TableDataEntity;
import com.mh.user.entity.TableInfoEntity;
import com.mh.user.provide.ChillersProvider;
import org.apache.ibatis.annotations.*;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author ljf
 * @title ：
 * @description ：关于冷水机组的持久化层，mapper层
 * @updateTime 2020-05-20
 * @throws ：
 */
@Component
@Mapper
public interface ChillersMapper {

    // 查询对应设备信息
    @Select("select device_code,register_address,last_value,last_time from chillers_register_address\t\n" +
            "\twhere (ddc_addr = '9999' or ddc_addr = '9998') and register_address = '09' and function_code = '1' and last_value = 1" +
//            " and convert(varchar(10),getdate(),23) = convert(varchar(10),last_time,23) " +
            " order by device_code ")
    @Results({
            @Result(column = "device_code", property = "deviceCode"),
            @Result(column = "register_address", property = "registerAddress"),
            @Result(column = "last_value", property = "lastValue"),
            @Result(column = "last_time", property = "lastTime")
    })
    List<ChillersEntity> getRunningChillers();

    // 查询对应设备信息
    @Select("select device_code,register_address,last_value,last_time from chillers_register_address " +
            " where ddc_addr = '9999' " +
            " and (register_address = '0A' or register_address = '0B' or register_address = '0E' or register_address = '0F') " +
            " and function_code = '3' and device_code = #{device_code} ")
    @Results({
            @Result(column = "device_code", property = "deviceCode"),
            @Result(column = "register_address", property = "registerAddress"),
            @Result(column = "last_value", property = "lastValue"),
            @Result(column = "last_time", property = "lastTime")
    })
    List<ChillersEntity> getChillersTemp(@Param("device_code") String device_code);

//    @Select("select id, device_code, device_num, collection_num, register_address, register_name, function_code, digit, other_name," +
//            " data_com, last_value, last_time, create_time, update_time, " +
//            " (select gateway_ip as IP from gateway_manage where data_com = c.data_com) as IP, " +
//            " (select port as port from gateway_manage where data_com = c.data_com) as port " +
//            " from chillers_register_address as c where c.grade = #{grade} order by c.id")
    @SelectProvider(type = ChillersProvider.class, method = "queryChillersByOther")
    @Results({
            @Result(column = "id", property = "id"),
            @Result(column = "device_code", property = "deviceCode"),
            @Result(column = "device_num", property = "deviceNum"),
            @Result(column = "collection_num", property = "collectionNum"),
            @Result(column = "register_address", property = "registerAddress"),
            @Result(column = "register_name", property = "registerName"),
            @Result(column = "function_code", property = "funCode"),
            @Result(column = "digit", property = "digit"),
            @Result(column = "other_name", property = "otherName"),
            @Result(column = "data_com", property = "dataCom"),
            @Result(column = "last_value", property = "lastValue"),
            @Result(column = "last_time", property = "lastTime"),
            @Result(column = "create_time", property = "createTime"),
            @Result(column = "update_time", property = "updateTime"),
            @Result(column = "ddc_addr", property = "ddcAddr"),
            @Result(column = "IP", property = "IP"),
            @Result(column = "port", property = "port")
    })
    // 2 代表DDC采集，1 代表串口通信，DDC地址：9999，9998 代表冷水机组通信
    List<ChillersEntity> queryChillersByOther(@Param("grade") int grade, @Param("ddcAddr") String ddcAddr);

    // 获取采集DDC设备的指令集合
    @Select("select ddc_addr, (select gateway_ip as IP from gateway_manage where data_com = c.data_com) as IP, " +
            " (select port as port from gateway_manage where data_com = c.data_com) as port " +
            " from chillers_register_address as c where c.grade = #{grade} " +
//            " and c.ddc_addr <> 8 and c.ddc_addr <> 0 and c.ddc_addr <> 9 and c.ddc_addr <> 10" +
            " group by ddc_addr,data_com order by convert(int,ddc_addr)")
    @Results({
            @Result(column = "ddc_addr", property = "ddcAddr"),
            @Result(column = "IP", property = "IP"),
            @Result(column = "port", property = "port")
    })
    // 2 代表DDC采集，1 代表串口通信
    List<ChillersEntity> queryChillersDDC(@Param("grade") int grade);


    // 插入冷水机组历史流水表
    @Insert("insert into chillers_data_history(device_code, device_num, collection_num, register_address, register_name, fun_code, data_value, collection_time, local_time, grade, ddc_addr) " +
            "values (#{deviceCode}, #{deviceNum}, #{collectionNum}, #{registerAddress}, #{registerName}, #{funCode}, #{lastValue}, #{lastTime,jdbcType=DATE}, #{lastTime,jdbcType=DATE}, 0, #{ddcAddr})")
    int insertChillersHistory(ChillersEntity chillersEntity);

    // 更新对应的冷水机组
    @Update("update chillers_register_address set last_value = #{lastValue}, last_time = #{lastTime} where collection_num = #{collectionNum} and register_address = #{registerAddress}")
    void updateChillerRegisterAddr(ChillersEntity chillersEntity);

    // 判断是否存在实时记录表
    @Select("select count(id) from data_now where device_num = #{deviceNum} and register_addr = #{registerAddr}")
    int queryDataNowByOthers(ChillersEntity chillersEntity);

    // 插入实时表
    @Insert("insert into data_now(device_num, device_code, cur_flow, cur_date, grade, register_addr, ddc_addr) " +
            "values (#{deviceNum}, #{deviceCode}, #{lastValue}, #{lastTime}, 0, #{registerAddress}, #{ddcAddr})")
    @Options(useGeneratedKeys = true, keyProperty = "id")
    int insertChillerDataNow(ChillersEntity chillersEntity);

    // 更新实时表数据
    @Update("update data_now set cur_flow = #{lastValue}, cur_date = #{lastTime} " +
            " where device_num = #{deviceNum} and register_addr = #{registerAddr} and cur_date <> #{lastTime}")
    void updateDataNow(ChillersEntity chillersEntity);

    // 查询冷水机组的状态，以及表格数据
    @Select("select span, title, info, infos, table_rows as rows from table_second where device_name = #{deviceName} and grade = #{grade} order by sort")
    List<TableDataEntity> queryTopData(@Param("deviceName") String deviceName, @Param("grade") int grade);

    @Select("select rowspan, td1, td2, td3  from table_first where device_name = #{deviceName} and grade = #{grade} order by sort")
    List<TableInfoEntity> queryLeftAndRightData(@Param("deviceName") String deviceName, @Param("grade") int grade);

    // 批量插入历史流水表
    @Insert("<script>" +
            "insert into chillers_data_history(register_address, data_value, collection_time, local_time, grade, ddc_addr)" +
            "values " +
            "<foreach collection='chillersEntityLists' item='item' index='index' separator=','>" +
            "(#{item.registerAddress},convert(numeric(24,2),#{item.lastValue}),getdate(),getdate(),0,#{item.ddcAddr})" +
            "</foreach>" +
            "</script>")
    void batchInsertChillersHistory(@Param("chillersEntityLists") List<ChillersEntity> chillersEntityLists);

    // 批量更新基表管理表内容
    @Update("<script>" +
            "<foreach collection = 'chillersEntityLists' item = 'item' separator=';'>" +
            "update chillers_register_address " +
            "<set> " +
            "   last_value = #{item.lastValue}," +
            "   last_time = #{item.lastTime}, " +
            "</set>" +
            "<where> " +
            "   ddc_addr = #{item.ddcAddr} and register_address = #{item.registerAddress} and last_time &lt;&gt; #{item.lastTime}" +
            "</where>" +
            "</foreach>" +
            "</script>")
    int updateRegisterAddr(@Param("chillersEntityLists") List<ChillersEntity> chillersEntityLists);

    // 批量更新实时记录表内容
    @Update("<script>" +
            "<foreach collection = 'chillersEntityLists' item = 'item' separator=';'>" +
            "update data_now " +
            "<set> " +
            "   cur_flow = #{item.lastValue}," +
            "   cur_date = #{item.lastTime}, " +
            "</set>" +
            "<where> " +
            "   ddc_addr = #{item.ddcAddr} and register_addr = #{item.registerAddress} and cur_date &lt;&gt; #{item.lastTime}" +
//            "   ddc_addr = #{item.ddcAddr} and register_addr = #{item.registerAddress} and cur_date &lt;&gt; #{item.lastTime}" +
            "</where>" +
            "</foreach>" +
            "</script>")
    void batchUpdateDataNow(@Param("chillersEntityLists") List<ChillersEntity> chillersEntityLists);

    // 通过DDC地址查询对应的采集参数设备
    @Select("select register_address from chillers_register_address where ddc_addr = #{ddcAddr} and grade = 2")
    List<String> queryDDCAddr(@Param("ddcAddr") String ddcAddr);

    @Select("select top 1 convert(varchar(20),collection_time,120) from chillers_data_history " +
            " where ddc_addr = #{ddcAddr} and register_address = #{registerAddress} order by collection_time desc ")
    String queryLastChillersHistory(ChillersEntity chillersEntity);

    // 查询对应设备信息
    @Select("select device_code,register_address,last_value,last_time from chillers_register_address\t\n" +
      "\twhere (ddc_addr = '9999' or ddc_addr = '9998') and register_address = '09' and function_code = '1' " +
      " order by device_code ")
    @Results({
      @Result(column = "device_code", property = "deviceCode"),
      @Result(column = "register_address", property = "registerAddress"),
      @Result(column = "last_value", property = "lastValue"),
      @Result(column = "last_time", property = "lastTime")
    })
    List<ChillersEntity> getStopChillers();
}
