package com.mh.user.netty;

import com.github.benmanes.caffeine.cache.Cache;
import com.mh.user.utils.SpringContextUtils;
import io.netty.channel.ChannelHandlerContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author ljf
 * @title ：
 * @description ： 管理客户端信息：连接数、连接时创建的对象、连接时创建的报文
 * @updateTime 2021-01-19
 * @throws ：
 */
@Slf4j
public class ClientManage {

    // 创建一个map容器，管理连接的客户端
    public static Map<String, ChannelHandlerContext> clientMap = new HashMap<>();
    // 创建一个map容器，管理创建的对象
    public static Map<String, EchoServerHandler> echoServerHandlerMap = new HashMap<>();
    // 创建一个容器，管理生成的报文
    public static Map<String, List<String>> orderList = new HashMap<>();

    ApplicationContext applicationContext = SpringContextUtils.getApplicationContext();
    Cache<String, Object> cache = (Cache<String, Object>) applicationContext.getBean("caffeineCache");

    private static ClientManage instance = null;

    // 创建一个私有的构造函数
    private ClientManage(){}

    // 创建
    public static ClientManage getInstance() {
        if (null == instance) {
            return new ClientManage();
        } else {
            return instance;
        }
    }

    /**
     * 保存指令
     * @param DTUId
     * @param list
     */
    public void saveOrderMap(String DTUId, List<String> list) {
        try {
            if (cache.getIfPresent(DTUId) != null) {
                log.info("==> 已存在报文");
            } else {
                cache.put(DTUId, list);
            }
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * 删除map中的指令
     * @param list
     */
    public void removeOrderMap(String DTUId, List<String> list) {
        // 从缓存中删除
        cache.asMap().remove(DTUId, list);
    }

    public void saveEchoServerHandlerMap(String DTUId, EchoServerHandler echoServerHandler) {
        try {
            if (echoServerHandlerMap.get(DTUId) != null && echoServerHandlerMap.get(DTUId).equals(echoServerHandler)) {
                // TODO 已存在
                log.info("==> 已存在报文");
            } else {
                //如果map中没有此ctx 将连接存入map中
                echoServerHandlerMap.put(DTUId, echoServerHandler);
            }
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * 删除map中ChannelHandlerContext
     */
    public void removeEchoServerHandlerMap(EchoServerHandler echoServerHandler) {
        for (String key : echoServerHandlerMap.keySet()) {
            if (echoServerHandlerMap.get(key) != null && echoServerHandlerMap.get(key).equals(echoServerHandler)) {
                echoServerHandlerMap.remove(key);
            }
        }
    }

    public void saveChannelMap(String DTUId, ChannelHandlerContext ctx) {
        try {
            if (ClientManage.clientMap.get(DTUId) != null && ClientManage.clientMap.get(DTUId).equals(ctx)) {
                // TODO 已存在
                log.info("==> 已存在报文");
            } else {
                //如果map中没有此ctx 将连接存入map中
                ClientManage.clientMap.put(DTUId, ctx);
            }
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * 删除map中ChannelHandlerContext
     */
    public void removeChannelMap(ChannelHandlerContext ctx) {
        for (String key : ClientManage.clientMap.keySet()) {
            if (ClientManage.clientMap.get(key) != null && ClientManage.clientMap.get(key).equals(ctx)) {
                ClientManage.clientMap.remove(key);
            }
        }
    }

    // 通过反射修改一个实例的私有属性值
//    public void updateInstanceAttribute(ReceiveCommandDto receiveCommandDto, DTUServiceImpl dtuService) {
//        try {
//            EchoServerHandler echoServerHandler = echoServerHandlerMap.get(receiveCommandDto.getGatewayId());
//            Class<?> aClass = echoServerHandler.getClass();
//
//            //获取方法对象
//            Method getTestMethod1 = aClass.getDeclaredMethod("isSendCommand");
//            //执行方法
//            Object invoke1 = getTestMethod1.invoke(echoServerHandler);
//            log.info("==> 修改前的对象地址==>" + echoServerHandler.hashCode() + " 修改前的状态==>" + invoke1);
//
//            //获取属性对象
//            Field sendCommand = aClass.getDeclaredField("sendCommand");
//            //修改属性值
//            sendCommand.setAccessible(true);
//            sendCommand.set(echoServerHandler,true);
//
//            //获取属性对象
//            Field callback = aClass.getDeclaredField("dtuServiceImpl");
//            //修改属性值
//            callback.setAccessible(true);
//            callback.set(echoServerHandler,dtuService);
//
////            String sendStr = "";
//            //获取属性对象
//            Field sendStrField = aClass.getDeclaredField("sendList");
//            //修改属性值
//            sendStrField.setAccessible(true);
//            sendStrField.set(echoServerHandler,receiveCommandDto.getOrderList());
//            //获取方法对象
//            Method getTestMethod = aClass.getDeclaredMethod("isSendCommand");
//            //执行方法
//            Object invoke = getTestMethod.invoke(echoServerHandler);
//            log.info("==> 修改后的对象地址==>" + echoServerHandler.hashCode() + " 修改后的状态==>" + invoke);
//        } catch (NoSuchFieldException | IllegalAccessException | NoSuchMethodException | InvocationTargetException e) {
//            e.printStackTrace();
//        }
//    }

}
