package com.mh.user.netty;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelOption;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import org.apache.log4j.Logger;

public class EchoServer {

    private final int port;
    private static final Logger log = Logger.getLogger(EchoServer.class);

    public EchoServer(int port) {
        this.port = port;
    }

    public void start() {
        // 创建Even-LoopGroup
        NioEventLoopGroup bossGroup = new NioEventLoopGroup();             // bossGroup 用于监听客户端连接，专门负责与客户端创建连接，并把连接注册到workerGroup的Selector中。
        NioEventLoopGroup workerGroup = new NioEventLoopGroup();           // workerGroup用于处理每一个连接发生的读写事件。
        try {
            ServerBootstrap serverBootstrap = new ServerBootstrap();        // 2、创建Server-Bootstrap
            serverBootstrap.group(bossGroup, workerGroup)
                    .channel(NioServerSocketChannel.class)                  // 3、指定所使用的NIO传输Channel
                    .localAddress(port)                                     // 4、指定端口设置套接字
                    .option(ChannelOption.SO_BACKLOG, 1204)
                    .childHandler(new ServerChannelInitializer());
            ChannelFuture channelFuture = serverBootstrap.bind().sync();    // 6、异步绑定服务器；调用sync()方法，阻塞式等待，直到绑定完成
            log.info("服务器启动开始监听端口:"+port);
            channelFuture.channel().closeFuture().sync();                   // 7、获取Channel的closeFuture，并且阻塞当前线程，直到它完成

        } catch (InterruptedException e) {
            log.error("服务器启动失败", e);
        } finally {
            try {
                bossGroup.shutdownGracefully().sync();                     // 8、关闭EventLoopGroup，关闭所有的资源
                workerGroup.shutdownGracefully().sync();                   // 关闭
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }
}
