package com.mh.user.netty;

import com.mh.user.entity.OrderMessageEntity;
import com.mh.user.service.chillers.GatewayManageService;
import com.mh.user.service.chillers.OrderMessageService;
import com.mh.user.utils.AnalysisReceiveOrder485;
import com.mh.user.constants.Constant;
import com.mh.user.utils.ExchangeStringUtil;
import com.mh.user.utils.GetReadOrder485;
import com.mh.user.utils.SpringBeanUtil;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandlerAdapter;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.timeout.IdleState;
import io.netty.handler.timeout.IdleStateEvent;
import io.netty.util.ReferenceCountUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * @author ljf
 * @title ：
 * @description ：客户端异步消息处理机制
 * @updateTime 2020-05-13
 * @throws ：
 */
@Slf4j
public class NettyChillerControlHandler extends ChannelHandlerAdapter {

    private int num = 0;
    private int size = 0;
    private String receiveStr = "";
    private int sendNum = 0;
    private int idle_count = 1;

    List<OrderMessageEntity> orderMessageEntityList;

    // 调用service
    ApplicationContext context = SpringBeanUtil.getApplicationContext();
    GatewayManageService gatewayManageService = context.getBean(GatewayManageService.class);
    OrderMessageService orderMessageService = context.getBean(OrderMessageService.class);

    public NettyChillerControlHandler(List<OrderMessageEntity> orderMessageEntityList) {
        this.orderMessageEntityList = orderMessageEntityList;
    }

    @Override
    public void channelUnregistered(ChannelHandlerContext ctx) throws Exception {
        log.info("当前channel从EventLoop取消注册");
//        Constant.SEND_STATUS = false;
//        super.channelUnregistered(ctx);
        ctx.close();
    }

    /**
     * 超时处理
     * 如果120秒没有接受客户端的心跳，就触发;
     * 如果超过3次，则直接关闭;
     */
    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object obj) throws Exception {
        if (obj instanceof IdleStateEvent) {
            IdleStateEvent event = (IdleStateEvent) obj;
            if (IdleState.READER_IDLE.equals(event.state())) {  //如果读通道处于空闲状态，说明没有接收到心跳命令
                System.out.println("第" + idle_count + "已经10秒没有接收到服务器的信息了,发送第" + num + "条数据");
                if (num > size - 1) {
                    num = 0;
                    // 关闭连接
                    receiveStr = null;
                    Constant.SEND_STATUS = true;
                    System.out.println("关闭这个不活跃的channel");
                    ctx.close();
                } else if (idle_count > 3) {
                    System.out.println("关闭这个不活跃的channel");
                    num = 0;
                    // 关闭连接
                    receiveStr = null;
                    Constant.SEND_STATUS = false;
                    ctx.close();
                } else {
                    // 发送采集DDC指令
                    // 判断空值
                    if (orderMessageEntityList.get(num).getRegisterAddr() == null ||
                            orderMessageEntityList.get(num).getRegisterAddr().equalsIgnoreCase("")) {
                        num = 0;
                        // 关闭连接
                        receiveStr = null;
                        Constant.SEND_STATUS = true;
                        ctx.close();
                    } else {
                        String sendStr = orderMessageEntityList.get(num).getOrderStr();
//                // 获取采集参数个数
//                size = orderMessageEntityList.size();
                        // 2.发送数据
                        ByteBuf buffer = getByteBuf(ctx, sendStr);
                        ctx.channel().writeAndFlush(buffer);
                        idle_count++;
                    }
                }
            }
        } else {
            super.userEventTriggered(ctx, obj);
        }
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
//        super.exceptionCaught(ctx, cause);
        log.info("通信异常！！");
        cause.printStackTrace();
//        receiveStr = null;
//        Channel incoming = ctx.channel();
//        if (incoming.isActive()) {
//            // 重新发送
//            if (sendNum > 2) {
//                // 通信异常，发送失败
//                log.info("SimpleClient: " + incoming.remoteAddress() + "异常");
//                cause.printStackTrace();
//                Constant.SEND_STATUS = false;
//                ctx.close();
//            } else {
//                // 发送采集DDC指令
//                String sendStr = orderMessageEntityList.get(num).getOrderStr();
//                // 获取采集参数个数
//                size = orderMessageEntityList.size();
//                // 2.发送数据
//                ByteBuf buffer = getByteBuf(ctx,sendStr);
//                ctx.channel().writeAndFlush(buffer);
//                sendNum += 1;
//            }
//        }
//        // 判断发送的下标，如果不等于指令数组大小
//        num = num + 1;
//        if (num > size-1) {
//            num = 0;
//            // 关闭连接
//            receiveStr = null;
//            Constant.SEND_STATUS = true;
//            ctx.close();
//        }
    }


    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        super.channelActive(ctx);
        SimpleDateFormat sdf1=new SimpleDateFormat("yyyy-mm-dd HH:mm:ss");
        Date date=new Date();
        log.info(ctx.channel().remoteAddress() + "  " + sdf1.format(date) + "链接服务端成功！");
        // 截取IP地址
        String IP = ExchangeStringUtil.getMidString(ctx.channel().remoteAddress()+"","/", ":");
        // 截取端口号
        String port = ExchangeStringUtil.getMidString(ctx.channel().remoteAddress()+"",":", "");
        log.info("IP: " + IP + ",端口号： " + port);
        // 更新对应的网关在线情况
        gatewayManageService.updateGatewayManage(IP, port);

        // 发送控制DDC指令
        String sendStr = orderMessageEntityList.get(num).getOrderStr();
        // 获取采集参数个数
        size = orderMessageEntityList.size();

        // 2.发送数据
        ByteBuf buffer = getByteBuf(ctx,sendStr);
        ctx.channel().writeAndFlush(buffer);
    }

    private ByteBuf getByteBuf(ChannelHandlerContext ctx, String sendStr) {
        // 申请一个数据结构存储信息
        ByteBuf buffer = ctx.alloc().buffer();
        // 将信息放入数据结构中
        buffer.writeBytes(ExchangeStringUtil.hexStrToBinaryStr(sendStr));//对接需要16进制
        return buffer;
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
//        Thread.sleep(100);
        ctx.close();
        log.info(ctx.channel().localAddress() + "退出链接！！");
    }

    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
//        super.channelRead(ctx, msg);
        //        ByteBuf buf = (ByteBuf)msg;
//        byte[] req = new byte[buf.readableBytes()];
//        buf.readBytes(req);
//        String body = new String(req, "UTF-8");
        try {
            ByteBuf buf = (ByteBuf)msg;
            byte [] bytes = new byte[buf.readableBytes()];
            buf.readBytes(bytes);//复制内容到字节数组bytes
            buf.clear();
            log.info("获取到的值： " + ExchangeStringUtil.bytesToHexString(bytes));
//            if (bytes.length <= 24) {
            if (bytes.length != 0) {
//            receiveStr = receiveStr.replace("null", "");
//            receiveStr = receiveStr + ExchangeStringUtil.bytesToHexString(bytes);//将接收到的数据转为字符串，此字符串就是客户端发送的字符串
//            log.info(ctx.channel().remoteAddress() + "  " + ctx.channel().localAddress() + " 接受服务器数据：" + receiveStr + "，大小： " + receiveStr.length());
                receiveStr = receiveStr + ExchangeStringUtil.bytesToHexString(bytes);//将接收到的数据转为字符串，此字符串就是客户端发送的字符串
                receiveStr = receiveStr.replace("null", "");
                log.info("接受服务器数据：" + receiveStr + "，大小： " + receiveStr.length());
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            ReferenceCountUtil.release(msg);
        }
    }

    @Override
    public void channelReadComplete(ChannelHandlerContext ctx) throws Exception {
        log.info("数据读取接收完成: " + receiveStr);
        if (receiveStr.length() == 24) {
            if (receiveStr.equalsIgnoreCase(orderMessageEntityList.get(num).getOrderStr())) {
                // 解析采集回来的数据
                log.info("采集完整的报文： " + receiveStr + "，指令下标： " + size);
                // 解析采集的报文
                // 更新发送后的指令
                OrderMessageEntity orderMessageEntity = new OrderMessageEntity();
                orderMessageEntity.setRegisterAddr(orderMessageEntityList.get(num).getRegisterAddr());
                orderMessageEntity.setCreateTime(orderMessageEntityList.get(num).getCreateTime());
                orderMessageEntity.setGrade(1);
                orderMessageEntity.setSendNum(1);
                orderMessageEntity.setStatus(1);
                orderMessageEntity.setOrderStr(orderMessageEntityList.get(num).getOrderStr());
                orderMessageService.updateOrderMessage(orderMessageEntity);

//                // 关闭连接
//                receiveStr = null;
//                Constant.SEND_STATUS = true;
//                ctx.close();

                // 清空receiveStr
                receiveStr = "";
                // 判断发送的下标，如果不等于指令数组大小
                num = num + 1;
                if (num > size - 1) {
                    num = 0;
                    // 关闭连接
                    receiveStr = null;
                    Constant.SEND_STATUS = true;
                    ctx.close();
                } else {
                    Thread.sleep(4000);
                    // 继续发送下一个采集DDC设备指令
                    String sendStr = orderMessageEntityList.get(num).getOrderStr();
                    ByteBuf buffer = getByteBuf(ctx, sendStr);
                    // 2.发送数据
                    ctx.channel().writeAndFlush(buffer);
                    log.info("客户端再次往服务端发送数据" + num + ",报文： " + sendStr);
                }
            }
        } else if ((receiveStr.length() > 24) && (num == 0)) {
            // 发送采集DDC指令
            String sendStr = orderMessageEntityList.get(num).getOrderStr();
            // 获取采集参数个数
            size = orderMessageEntityList.size();
            // 2.发送数据
            ByteBuf buffer = getByteBuf(ctx,sendStr);
            ctx.channel().writeAndFlush(buffer);
            // 清空receiveStr
            receiveStr = "";
            sendNum += 1;
        } else if (sendNum > 2){
            // 更新发送后的指令
            OrderMessageEntity orderMessageEntity = new OrderMessageEntity();
            orderMessageEntity.setRegisterAddr(orderMessageEntityList.get(num).getRegisterAddr());
            orderMessageEntity.setCreateTime(orderMessageEntityList.get(num).getCreateTime());
            orderMessageEntity.setGrade(1);
            orderMessageEntity.setSendNum(sendNum);
            orderMessageEntity.setStatus(0);
            orderMessageEntity.setOrderStr(orderMessageEntityList.get(num).getOrderStr());
            orderMessageService.updateOrderMessage(orderMessageEntity);
            Constant.SEND_STATUS = false;
            ctx.close();
        } else if ((receiveStr.length() > 24)) {
            // 接收采集DDC的数据
            // 解析采集的报文
            AnalysisReceiveOrder485 analysisReceiveOrder485 = new AnalysisReceiveOrder485();
            analysisReceiveOrder485.analysisChillersDDC(receiveStr);

            // 清空receiveStr
            receiveStr = "";
            // 更新发送后的指令
            OrderMessageEntity orderMessageEntity = new OrderMessageEntity();
            orderMessageEntity.setRegisterAddr(orderMessageEntityList.get(num).getRegisterAddr());
            orderMessageEntity.setCreateTime(orderMessageEntityList.get(num).getCreateTime());
            orderMessageEntity.setGrade(1);
            orderMessageEntity.setSendNum(1);
            orderMessageEntity.setStatus(1);
            orderMessageEntity.setOrderStr(orderMessageEntityList.get(num).getOrderStr());
            orderMessageService.updateOrderMessage(orderMessageEntity);

            // 判断发送的下标，如果不等于指令数组大小
            num = num + 1;
//            Thread.sleep(500);
            if (num > size-1) {
                num = 0;
                // 关闭连接
                receiveStr = null;
                Constant.SEND_STATUS = true;
                ctx.close();
            }
        }
        ctx.flush();
    }

}
