package com.mh.user.netty;

import io.netty.bootstrap.Bootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;
import io.netty.handler.timeout.ReadTimeoutHandler;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;

/**
 * @author ljf
 * @title ：
 * @description ：Netty客户端，采集电表
 * @updateTime 2020-05-13
 * @throws ：
 */
@Slf4j
@Setter
@Getter
public class NettyMeterClient {
//        implements Runnable {

    private int port;
    private String host;

    //    构造函数传递值 继承Thread时需要
    public NettyMeterClient(int port, String host) {
        this.port = port;
        this.host = host;
    }

    public NettyMeterClient() {
        super();
    }

    public void connect(int port, String host) throws InterruptedException {
        // 配置客户端NIO线程组
      // 配置客户端NIO线程组
      EventLoopGroup group = new NioEventLoopGroup(1);
      try {
        Bootstrap bootstrap = new Bootstrap();
        bootstrap.group(group).channel(NioSocketChannel.class)
          .option(ChannelOption.TCP_NODELAY, true)
          .option(ChannelOption.CONNECT_TIMEOUT_MILLIS, 3000)
          .option(ChannelOption.RCVBUF_ALLOCATOR, new FixedRecvByteBufAllocator(1024*1024))
                    .handler(new ChannelInitializer<SocketChannel>() {
                        @Override
                        protected void initChannel(SocketChannel socketChannel) {
                            // 基于换行符号
//                            socketChannel.pipeline().addLast(new LineBasedFrameDecoder(1024));
//                            // 解码转String，注意调整自己的编码格式GBK、UTF-8
//                            socketChannel.pipeline().addLast(new StringDecoder(StandardCharsets.UTF_8));
//                            // 解码转String，注意调整自己的编码格式GBK、UTF-8
//                            socketChannel.pipeline().addLast(new StringEncoder(StandardCharsets.UTF_8));
                            // 超过10秒钟没有数据读取自动断开连接
                            socketChannel.pipeline().addLast(new ReadTimeoutHandler(20));
                            // 在管道中添加我们自己的接收数据实现方法
//                            socketChannel.pipeline().addLast(new NettyClientHandler());
                            socketChannel.pipeline().addLast(new NettyMeterClientHandler());
                        }
                    });
            // 发起异步连接操作
            ChannelFuture channelFuture = bootstrap.connect(host, port).sync();
            if (channelFuture.isSuccess()) {
                log.info("connect server  成功---------");
            } else {
                log.info("连接失败！");
                log.info("准备重连！");
//                connect(port, host);
            }

            // 等待客户端连接链路关闭future.channel().closeFuture().sync(); // 阻塞main线程
            channelFuture.channel().closeFuture().sync();
        } catch (Exception e) {
            log.error(e.getMessage());
        } finally {
            group.shutdownGracefully();
//            try {
//                TimeUnit.SECONDS.sleep(5);
//                connect(port, host); // 断线重连
//            } catch (InterruptedException e) {
//                e.printStackTrace();
//            }
        }
    }
//
//    @SneakyThrows
//    @Override
//    public void run() {
//        connect(port, host);
//    }
}
