package com.mh.user.netty;

import com.mh.user.constants.Constant;
import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.entity.DeviceManageEntity;
import com.mh.user.service.DeviceCodeParamService;
import com.mh.user.service.chillers.DeviceManageService;
import com.mh.user.utils.*;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandlerAdapter;
import io.netty.channel.ChannelHandlerContext;
import io.netty.util.ReferenceCountUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * @author ljf
 * @title ：
 * @description ：客户端异步消息处理机制
 * @updateTime 2020-05-13
 * @throws ：
 */
@Slf4j
public class NettyMeterClientHandler extends ChannelHandlerAdapter {


  private int num = 0;
  private int size = 0;
  private String receiveStr = null;
  private String IP = "";
  private String port = "";
  List<DeviceCodeParamEntity> deviceManageEntityList;

  // 调用service
  ApplicationContext context = SpringBeanUtil.getApplicationContext();
  DeviceCodeParamService deviceCodeParamService = context.getBean(DeviceCodeParamService.class);

  AnalysisReceiveOrder485 analysisReceiveOrder485 = new AnalysisReceiveOrder485();

  @Override
  public void channelUnregistered(ChannelHandlerContext ctx) throws Exception {
    log.info("当前channel从EventLoop取消注册");
    super.channelUnregistered(ctx);
  }

  @Override
  public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
//        super.exceptionCaught(ctx, cause);
    log.info("通信异常！！");
//        receiveStr = null;
    Channel incoming = ctx.channel();
    if (incoming.isActive()) {
      log.info("SimpleClient: " + incoming.remoteAddress() + "异常");
      receiveStr = null;
      cause.printStackTrace();
      ctx.close();
//            receiveStr = null;
//            try {
//                TimeUnit.SECONDS.sleep(5);
//                SocketAddress remoteAddress = ctx.channel().remoteAddress();
//                String port = ExchangeStringUtil.endData(remoteAddress.toString(),":");
//                String host = ExchangeStringUtil.splitData(remoteAddress.toString(),"/",":");
//                NettyClient nettyClient = new NettyClient();
//                nettyClient.connect(Integer.parseInt(port), host); // 断线重连
//            } catch (InterruptedException e) {
//                e.printStackTrace();
//            }
    }
  }


  @Override
  public void channelActive(ChannelHandlerContext ctx) throws Exception {
//        super.channelActive(ctx);
    // 添加一个状态值，判断是否继续发送指令 update by ljf on 2020-08-07
    if (Constant.WEB_FLAG) {
      num = 0;
      // 关闭连接
      receiveStr = null;
      ctx.close();
    } else {
      ctx.channel().read();
      SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-mm-dd HH:mm:ss");
      Date date = new Date();
      log.info(ctx.channel().remoteAddress() + "  " + sdf1.format(date) + "链接服务端成功！");

      // 截取IP地址
      IP = ExchangeStringUtil.getMidString(ctx.channel().remoteAddress() + "", "/", ":");
      // 截取端口号
      port = ExchangeStringUtil.getMidString(ctx.channel().remoteAddress() + "", ":", "");
      log.info("IP: " + IP + ",端口号： " + port);

      // 生成对应的采集指令
      // 修改生成指令（冷量计和电量一起采集） update by ljf on 2021-01-27
      deviceManageEntityList = deviceCodeParamService.queryCodeParam(port);
      size = deviceManageEntityList.size();

      log.info("初始连接报文： " + receiveStr);
      receiveStr = "";
      // 保持长连接，封装发送电表工具方法 update by ljf on 2021-01-26
      SendOrderUtils.sendMeterOrder(deviceManageEntityList.get(0),ctx,num,size);
      // 1.创建将要写出的数据
//            String sendStr = "6830043080000068110432326536C816";            // 网络单相电表
//            String sendStr = "FEFEFEFE6880025007000068010243C3B216";  // 广仪三相电表
//            String collectionNum = deviceManageEntityList.get(0).getCollectionNum();
//            String sendStr = GetReadOrder485.createMeterOrder(IP, port,
//                    deviceManageEntityList.get(0).getDataCom(), collectionNum, "1");
////            FileUtils.createFileAndWrite(sendStr, 0);
//            ByteBuf buffer = getByteBuf(ctx, sendStr);
//            // 2.发送数据
//            ctx.channel().writeAndFlush(buffer);
    }

  }

  private ByteBuf getByteBuf(ChannelHandlerContext ctx, String sendStr) {
    // byte类型的数据
//        byte[] bytes = "这里是将要写往服务端的数据".getBytes(Charset.forName("utf-8"));
//        String sendStr = "5803004900021914";          // 冷量计
    // 申请一个数据结构存储信息
    ByteBuf buffer = ctx.alloc().buffer();
    // 将信息放入数据结构中
    buffer.writeBytes(ExchangeStringUtil.hexStrToBinaryStr(sendStr));//对接需要16进制
    return buffer;
  }

  @Override
  public void channelInactive(ChannelHandlerContext ctx) throws Exception {
    Thread.sleep(500);
    receiveStr = null;
    ctx.close();
    log.info(ctx.channel().localAddress() + "退出链接！！");
  }

  @Override
  public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
    try {
      ByteBuf buf = (ByteBuf) msg;
      byte[] bytes = new byte[buf.readableBytes()];
      buf.readBytes(bytes);//复制内容到字节数组bytes
      buf.clear();
      log.info("获取到的值： " + ExchangeStringUtil.bytesToHexString(bytes));
      if (bytes.length <= 62) {
//            if (bytes.length <= 142) {
//            receiveStr = receiveStr.replace("null", "");
//            receiveStr = receiveStr + ExchangeStringUtil.bytesToHexString(bytes);//将接收到的数据转为字符串，此字符串就是客户端发送的字符串
//            log.info(ctx.channel().remoteAddress() + "  " + ctx.channel().localAddress() + " 接受服务器数据：" + receiveStr + "，大小： " + receiveStr.length());
        receiveStr = receiveStr + ExchangeStringUtil.bytesToHexString(bytes);//将接收到的数据转为字符串，此字符串就是客户端发送的字符串
        receiveStr = receiveStr.replace("null", "");
        log.info("接受服务器数据：" + receiveStr + "，大小： " + receiveStr.length());
      }
    } catch (Exception e) {
      e.printStackTrace();
    } finally {
      ReferenceCountUtil.release(msg);
    }
//        super.channelRead(ctx, msg);
    //        ByteBuf buf = (ByteBuf)msg;
//        byte[] req = new byte[buf.readableBytes()];
//        buf.readBytes(req);
//        String body = new String(req, "UTF-8");
//        ByteBuf buf = (ByteBuf)msg;
//        byte [] bytes = new byte[buf.readableBytes()];
//        buf.readBytes(bytes);//复制内容到字节数组bytes
//        log.info("获取到的值： " + ExchangeStringUtil.bytesToHexString(bytes));
//        if (bytes.length != 0) {
////            receiveStr = receiveStr.replace("null", "");
////            receiveStr = receiveStr + ExchangeStringUtil.bytesToHexString(bytes);//将接收到的数据转为字符串，此字符串就是客户端发送的字符串
////            log.info(ctx.channel().remoteAddress() + "  " + ctx.channel().localAddress() + " 接受服务器数据：" + receiveStr + "，大小： " + receiveStr.length());
//            receiveStr = receiveStr + ExchangeStringUtil.bytesToHexString(bytes);//将接收到的数据转为字符串，此字符串就是客户端发送的字符串
//            receiveStr = receiveStr.replace("null", "");
//            log.info("接受服务器数据：" + receiveStr + "，大小： " + receiveStr.length());
//        }
  }

  @Override
  public void channelReadComplete(ChannelHandlerContext ctx) throws Exception {
    log.info("采集电表-数据读取接收完成: " + receiveStr);
//        把receiveStr的"null"值去掉
//        a9fec2c71f9002fefefefe6839025007000068810643c3bb446c338d16c2b8
//        A9 FE C2 C7 1F 90 02 FE FE FE FE 68 39 02 50 07 00 00 68 81 06 43 C3 5B 38 6C 33 21 16 F8 12
    if ((receiveStr.length() == 62)) {
//            log.info(receiveStr);
      analysisReceiveOrder485.analysisMeterOrder485(receiveStr,deviceManageEntityList.get(num));          // 解析电表
      receiveStr = "";
      num = num + 1;
      Thread.sleep(600);
      if (num > size - 1) {
        num = 0;
        receiveStr = null;
        // 关闭连接
        ctx.close();
//              // 保持长连接，封装发送电表工具方法 update by ljf on 2021-01-26
//              SendOrderUtils.sendMeterOrder(deviceManageEntityList.get(num),num,IP,port,ctx);
      } else {
        // 添加一个状态值，判断是否继续发送指令 update by ljf on 2020-08-07
        if (Constant.WEB_FLAG) {
          log.info("有指令下发退出定时采集DDC参数");
          num = 0;
          // 关闭连接
          receiveStr = null;
          ctx.close();
        } else {
          // 封装发送电表工具方法 update by ljf on 2021-01-26
          SendOrderUtils.sendMeterOrder(deviceManageEntityList.get(num),ctx,num,size);
          // 1.创建将要写出的数据
          //            fe fe fe fe 68 80 02 50 07 00 00 68 81 06 43 c3 8c 34 33 33 5c 16
//            String sendStr = "FEFEFE6880025007000068010243C3B216";
//                    String collectionNum = deviceManageEntityList.get(num).getCollectionNum();
//                    String sendStr = GetReadOrder485.createMeterOrder(IP, port,
//                            deviceManageEntityList.get(num).getDataCom(), collectionNum, "1");
//                    ByteBuf buffer = getByteBuf(ctx, sendStr);
//                    // 2.发送数据
//                    ctx.channel().writeAndFlush(buffer);
//                    log.info("客户端再次往服务端发送数据" + num);
        }
      }
    } else if ((receiveStr.length() > 62)) {
      receiveStr = null;
      num = num + 1;
      Thread.sleep(500);
      if (num > size - 1) {
        num = 0;
        receiveStr = null;
        // 关闭连接
        ctx.close();
//              // 保持长连接，封装发送电表工具方法 update by ljf on 2021-01-26
//              SendOrderUtils.sendMeterOrder(deviceManageEntityList.get(num),num,IP,port,ctx);
      } else {
        // 添加一个状态值，判断是否继续发送指令 update by ljf on 2020-08-07
        if (Constant.WEB_FLAG) {
          log.info("有指令下发退出定时采集DDC参数");
          num = 0;
          // 关闭连接
          receiveStr = null;
          ctx.close();
        } else {
          // 封装发送电表工具方法 update by ljf on 2021-01-26
          SendOrderUtils.sendMeterOrder(deviceManageEntityList.get(num),ctx,num,size);
          // 1.创建将要写出的数据
          //            fe fe fe fe 68 80 02 50 07 00 00 68 81 06 43 c3 8c 34 33 33 5c 16
//            String sendStr = "FEFEFE6880025007000068010243C3B216";
//                    String collectionNum = deviceManageEntityList.get(num).getCollectionNum();
//                    String sendStr = GetReadOrder485.createMeterOrder(IP, port,
//                            deviceManageEntityList.get(num).getDataCom(), collectionNum, "1");
//                    ByteBuf buffer = getByteBuf(ctx, sendStr);
//                    // 2.发送数据
//                    ctx.channel().writeAndFlush(buffer);
//                    log.info("客户端再次往服务端发送数据" + num);
        }
      }
    }
    ctx.flush();
  }

}
