package com.mh.user.serialport;

import com.mh.user.constants.Constant;
import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.service.DeviceCodeParamService;
import com.mh.user.utils.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.log4j.Logger;
import org.springframework.context.ApplicationContext;
//import purejavacomm.SerialPort;
import gnu.io.SerialPort;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author nxr
 * @title ：
 * @description ： 串口发送和接收处理,采集类
 * @updateTime 2022-08-10
 * @throws ：
 */

//@Slf4j
public class SerialPortSendReceive {

    private  String receiveStr = null;
    public SerialPort serialPort = null;
    private int size = 0;
    private int baudrate=9600;
    private String parity=null;
    List<DeviceCodeParamEntity> deviceManageEntityList;
    private static final Logger log = Logger.getLogger(SerialPortSendReceive.class);
    // 调用service
    ApplicationContext context = SpringBeanUtil.getApplicationContext();
    DeviceCodeParamService deviceCodeParamService = context.getBean(DeviceCodeParamService.class);
    AnalysisReceiveOrder485 analysisReceiveOrder485 = new AnalysisReceiveOrder485();

    public void serialPortSend(String comName) {
        //查看所有串口
        SerialPortUtil serialPortUtil = SerialPortUtil.getSerialPortUtil();
        ArrayList<String> port = serialPortUtil.findPort();
//        SerialTool serialPortUtil = SerialTool.getSerialPortUtil();
//        ArrayList<String> port = serialPortUtil.findPort();

//        comName=comName.toUpperCase(); //转为大写
//        if (port.contains(comName)){
            try{
                //生成对应的采集指令
                deviceManageEntityList = deviceCodeParamService.queryCodeParam3();
                size = deviceManageEntityList.size();
                for (int i=0;i<size;i++){
                    //判断网页端是否有操作设备的
                    if (Constant.WEB_FLAG) {
                        serialPortUtil.closePort(serialPort);
                        log.info("有指令下发退出定时采集");
                        break;
                    }
                    //获取设备实际波特率
                    baudrate=deviceManageEntityList.get(i).getBaudRate();
                    //获取设备实际校验位
                    parity=deviceManageEntityList.get(i).getParity();
                    comName=deviceManageEntityList.get(i).getDataPort();
                    comName=comName.toUpperCase(); //转为大写
                    if (port.contains(comName)){
                        try{

                            //传入对应的串口参数并打开串口
                            if (parity==null || parity.equals("") || parity.equalsIgnoreCase("none")){
                                serialPort = serialPortUtil.openPort(comName, baudrate, SerialPort.DATABITS_8, SerialPort.PARITY_NONE, SerialPort.PARITY_ODD);
                            }else{
                                serialPort = serialPortUtil.openPort(comName, baudrate, SerialPort.DATABITS_8, SerialPort.PARITY_EVEN, SerialPort.PARITY_ODD);
                            }
                            //向串口发送指令
                            SendOrderUtils.sendSerialPort(deviceManageEntityList.get(i), serialPort);
                            Thread.sleep(5000);

                        }catch (Exception e){
                            e.printStackTrace();
                            serialPortUtil.closePort(serialPort);
                            Thread.sleep(10000);
                        }
                        receiveStr=null;
                        //从串口读取数据
                        byte[] bytes= serialPortUtil.readFromPort(serialPort);
                        try {
                            String byteStr = new String(bytes, 0, bytes.length).trim();
                        } catch (NullPointerException e) {
                            serialPortUtil.closePort(serialPort);
                            Thread.sleep(10000);
                            log.info("串口"+serialPort+"没有数据返回！"+i);
                            continue;
                        }
                        receiveStr = receiveStr + printHexString(bytes);
                        //去掉空格和null
                        receiveStr = receiveStr.replace("null", "");
                        receiveStr = receiveStr.replace(" ", "");
                        log.info("串口"+serialPort+"接受第"+i+"数据：" + receiveStr + "，大小： " + receiveStr.length());
                        //返回值全部变成大写
                        String receiveData = receiveStr.toUpperCase();
                        //截取去掉FE
                        String dataStr;
                        if (receiveData.length()>8){
                            String str1=receiveData.substring(0,8);
                            String str2=receiveData.substring(8);
                            dataStr=str1.replace("FE", "")+str2;
                        }else{
                            dataStr = receiveData.replace("FE", "");
                        }

                        String deviceType=deviceManageEntityList.get(i).getDeviceType();
                        String registerAddr=deviceManageEntityList.get(i).getRegisterAddr();
                        String brand=deviceManageEntityList.get(i).getBrand();//品牌
                        String buildingId=deviceManageEntityList.get(i).getProjectID();

                        System.out.println("=========设备类型："+deviceType+",长度："+dataStr.length());
                        //下面是对返回数据根据不同的设备进行相关解析处理
                        try{
                            if ((dataStr.length() == 36 || dataStr.length() == 44 || dataStr.length()==40 || dataStr.length()==50) && deviceType.equals("电表")) {
                                analysisReceiveOrder485.analysisMeterOrder485(dataStr,deviceManageEntityList.get(i));
                            }else if ((dataStr.length() == 70 || dataStr.length() == 44)  && deviceType.equals("水表")) {
                                analysisReceiveOrder485.analysisWtMeterOrder485(dataStr,registerAddr,brand,buildingId);
                            }

                        }catch (Exception e){
//                    e.printStackTrace();
                            log.error(deviceManageEntityList.get(i).getDeviceType()+"保存数据库失败！"+i);
                            serialPortUtil.closePort(serialPort);
                            System.out.println("关闭"+serialPort);
                            Thread.sleep(10000);
                        }

                        try{
                            serialPortUtil.closePort(serialPort);
                            System.out.println("关闭"+serialPort);
                            Thread.sleep(10000);
                        }catch (Exception e){
                            e.printStackTrace();
                        }
                    }else{
                        log.info("串口："+comName+"不存在!");
                    }
                }

            }catch (Exception e){
                log.error("串口采集异常！");
            }
//        }else {
//            log.info("串口："+comName+"不存在!");
//        }
    }

    /**
     * 字节数组转16进制字符串
     * @param b 字节数组
     * @return 16进制字符串
     */
    public static String printHexString(byte[] b) {
        StringBuilder sbf = new StringBuilder();
        for (byte value : b) {
            String hex = Integer.toHexString(value & 0xFF);
            if (hex.length() == 1) {
                hex = '0' + hex;
            }
            sbf.append(hex.toUpperCase()).append(" ");
        }
        return sbf.toString().trim();
    }

    /**
     * 十六进制字符串转byte[]
     * @param hex 十六进制字符串
     * @return byte[]
     */
    public static byte[] hexStr2Byte(String hex) {
        if (hex == null) {
            return new byte[] {};
        }
        // 奇数位补0
        if (hex.length() % 2 != 0) {
            hex = "0" + hex;
        }

        int length = hex.length();
        ByteBuffer buffer = ByteBuffer.allocate(length / 2);
        for (int i = 0; i < length; i++) {
            String hexStr = hex.charAt(i) + "";
            i++;
            hexStr += hex.charAt(i);
            byte b = (byte) Integer.parseInt(hexStr, 16);
            buffer.put(b);
        }
        return buffer.array();
    }

    /**
     * 16进制转换成为string类型字符串
     * @param s 待转换字符串
     */
    public static String hexStringToString(String s) {
        if (s == null || "".equals(s)) {
            return null;
        }
        s = s.replace(" ", "");
        byte[] baKeyword = new byte[s.length() / 2];
        for (int i = 0; i < baKeyword.length; i++) {
            try {
                baKeyword[i] = (byte) (0xff & Integer.parseInt(s.substring(i * 2, i * 2 + 2), 16));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        try {
            s = new String(baKeyword, StandardCharsets.UTF_8);
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        return s;
    }
}
