package com.mh.user.service.impl;

import com.mh.user.entity.DataResultClEntity;
import com.mh.user.entity.DataResultEntity;
import com.mh.user.entity.DataResultChEntity;
import com.mh.user.entity.DeviceInstallEntity;
import com.mh.user.mapper.DataResultMapper;
import com.mh.user.mapper.DeviceInstallMapper;
import com.mh.user.service.DataResultService;
import com.mh.user.service.DeviceInstallService;
import com.mh.user.utils.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

@Service
public class DataResultServiceImpl implements DataResultService {

    @Autowired
    DataResultMapper dataResultMapper;

    @Autowired
    DeviceInstallMapper deviceInstallMapper;

    @Autowired
    DeviceInstallService deviceInstallService;

    private final SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Override
    public void saveDataResult(DataResultEntity dataResultEntity) {

        try{
            //获取系统日期
            Date date=new Date();
            if (dataResultEntity.getCurDate()==null){
                dataResultEntity.setCurDate(date);
            }
            //从安装表获取设备信息
            DeviceInstallEntity deviceInstallEntity=deviceInstallMapper.selectDevice(dataResultEntity.getDeviceAddr(),dataResultEntity.getDeviceType(),dataResultEntity.getProjectID());
            //判断是否有记录
//            int r = dataResultMapper.selectDataResultCount(sdf1.format(dataResultEntity.getCurDate()),
//                    dataResultEntity.getDeviceAddr(),dataResultEntity.getDeviceType());
//            if (r==0){//插入记录
                DataResultEntity data=new DataResultEntity();
                if (dataResultEntity.getDeviceType().equals("水表") || dataResultEntity.getDeviceType().equals("电表")){
//                    String str=dataResultEntity.getDeviceAddr();
//                    data.setDeviceAddr(str.substring(str.length()-8,str.length())); //通讯地址
                    data.setDeviceAddr(dataResultEntity.getDeviceAddr()); //通讯地址
                }else{
                    data.setDeviceAddr(dataResultEntity.getDeviceAddr()); //通讯地址
                }
                data.setDeviceType(dataResultEntity.getDeviceType()); //设备类型
                String curDate=sdf1.format(dataResultEntity.getCurDate());
                data.setCurDate(sdf1.parse(curDate));//当前日期
                data.setCurValue(dataResultEntity.getCurValue());     //当前读数
                data.setProjectID(deviceInstallEntity.getProjectID());   //楼栋编号
                data.setRatio(deviceInstallEntity.getRatio());             //倍率
                double lastValue=deviceInstallEntity.getLastValue();       //安装基表上次读数
                double calcValue=dataResultEntity.getCurValue()-lastValue; //计算用量
                data.setCalcValue(calcValue);//用量
                if (deviceInstallEntity.getLastDate()!=null){//上次抄表日期
                    String lastDate=sdf1.format(deviceInstallEntity.getLastDate());
                    data.setLastDate(sdf1.parse(lastDate));
                }else{
                    String lastDate=sdf1.format(date);
                    data.setLastDate(sdf1.parse(lastDate));
                }
                data.setLastValue(lastValue);//上次读数
                data.setGrade(dataResultEntity.getGrade());//级别
                dataResultMapper.saveDataResult(data);         //插入新的记录
//            }else {//修改记录的curValue、calcValue
//                DataResultEntity data2=dataResultMapper.selectDataResult(sdf1.format(dataResultEntity.getCurDate()),dataResultEntity.getDeviceAddr(),dataResultEntity.getDeviceType());
//                double lastValue=data2.getLastValue();       //安装基表上次读数
//                double calcValue=dataResultEntity.getCurValue()-lastValue; //计算用量
//                String curDate=sdf1.format(dataResultEntity.getCurDate());
//                data2.setCurDate(sdf1.parse(curDate));//当前日期
//                data2.setCurValue(dataResultEntity.getCurValue());//当前读数
//                data2.setCalcValue(calcValue);//用量
//
//                dataResultMapper.updateDataResult(data2);  //更新记录
//            }
            //修改安装表中lastValue,lastDate的值
            deviceInstallMapper.updateLastValue(deviceInstallEntity.getId(),dataResultEntity.getCurValue(),date);
        }catch (Exception e){
            e.printStackTrace();

        }
    }

    @Override
    public void updateDataResult(DataResultEntity dataResultEntity) {
        dataResultMapper.updateDataResult(dataResultEntity);
    }

    @Override
    public List<DataResultEntity> queryDataResult(String projectID, String startDate, String endDate,String deviceType, int page, int limit) {
        return dataResultMapper.queryDataResult(projectID,startDate,endDate,deviceType,page,limit);
    }

    @Override
    public int getDataResultCount(String projectID, String startDate, String endDate,String deviceType, int page, int limit) {
        return dataResultMapper.getDataResultCount(projectID,startDate,endDate,deviceType,page,limit);
    }
    //-------------------------------------------------------------------------------------
    //保存流量计用量
    @Override
    public void saveDataResultCl(DataResultClEntity dataResultClEntity) throws ParseException {
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try{
            //获取系统日期
            Date date=new Date();
            if (dataResultClEntity.getCurDate()==null){
                dataResultClEntity.setCurDate(date);
            }
            //从安装表获取设备信息
            DeviceInstallEntity deviceInstallEntity=deviceInstallMapper.selectDevice(dataResultClEntity.getDeviceAddr(),dataResultClEntity.getDeviceType(),dataResultClEntity.getProjectID());
            Double lastValue=deviceInstallEntity.getLastValue();
            Double curValue=dataResultClEntity.getCurValue();
            dataResultClEntity.setRatio(deviceInstallEntity.getRatio()); //倍率
//            if ((curValue-lastValue)<1000 && (curValue-lastValue)>=0){
            if ((curValue-lastValue)>=0){
                dataResultMapper.saveDataResultCl(dataResultClEntity);         //插入新的记录
                //修改安装表中lastValue,lastDate的值
                deviceInstallMapper.updateLastValue(deviceInstallEntity.getId(),curValue,date);
            }

        }catch (Exception e){
            e.printStackTrace();
        }
    }

    //保存流量计用量
    @Override
    public void saveDataResultCl_bak(DataResultClEntity dataResultClEntity) throws ParseException {
        try{
            //获取系统日期
            if (dataResultClEntity.getCurDate()==null){
                Date date=new Date();
                dataResultClEntity.setCurDate(date);
            }
            //从安装表获取设备信息
            DeviceInstallEntity deviceInstallEntity=deviceInstallMapper.selectDevice(dataResultClEntity.getDeviceAddr(),dataResultClEntity.getDeviceType(),dataResultClEntity.getProjectID());
            dataResultClEntity.setRatio(deviceInstallEntity.getRatio()); //倍率
            String curDate = DateUtil.dateToString(dataResultClEntity.getCurDate(), "yyyy-MM-dd HH:mm:ss");
            String deviceAddr = deviceInstallEntity.getDeviceAddr();
            String registerAddr = dataResultClEntity.getRegisterAddr();
            String projectID = dataResultClEntity.getProjectID();
            int row=dataResultMapper.selectDataResultChBakCount(curDate,deviceAddr,registerAddr,projectID);//无记录
            if (row==0){
                dataResultMapper.saveDataResultCl_bak(dataResultClEntity);         //插入新的记录
            }

        }catch (Exception e){
            e.printStackTrace();
        }
    }
    @Override
    public void updateDataResultCl(DataResultClEntity dataResultClEntity) {
        dataResultMapper.updateDataResultCl(dataResultClEntity);
    }

    //-------------------------------------------------------------------------------------
    @Override
    public void saveDataResultCh(DataResultChEntity dataResultChEntity) throws ParseException {
        try{
            //从安装表获取设备信息
            String deviceAddr=dataResultChEntity.getDeviceAddr();
            String registerAddr=dataResultChEntity.getRegisterAddr();
            String curDate= DateUtil.dateToString(dataResultChEntity.getCurDate(),"yyyy-MM-dd HH:mm:ss");
            String projectID=dataResultChEntity.getProjectID();
            int row=dataResultMapper.selectDataResultChCount(curDate,deviceAddr,registerAddr,projectID, dataResultChEntity.getGrade());//无记录
            if (row==0){
                dataResultMapper.saveDataResultCh(dataResultChEntity);         //插入新的记录
            }

        }catch (Exception e){
            e.printStackTrace();
        }
    }

    @Override
    public void updateDataResultCh(DataResultChEntity dataResultChEntity) {
        dataResultMapper.saveDataResultCh(dataResultChEntity);
    }

    @Override
    public List<DataResultChEntity> queryDataResultMi(String projectID,String startDate,String curDate,int page,int limit, int type) {
        if (type==0){
            return dataResultMapper.queryDataResultOneMi(projectID,startDate,curDate,page,limit);
        }else if (type==1){
            return dataResultMapper.queryDataResultFiveMi(projectID,startDate,curDate,page,limit);
        }else if (type==2){
            return dataResultMapper.queryDataResultFifteenMi(projectID,startDate,curDate,page,limit);
        }else {
            return dataResultMapper.queryDataResultOneMi(projectID,startDate,curDate,page,limit);
        }
    }

    @Override
    public int dataResultOneMiCount(String projectID,String startDate,String curDate) {
        return dataResultMapper.dataResultOneMiCount(projectID,startDate,curDate);
    }

    @Override
    public int dataResultFiveMiCount(String projectID,String startDate,String curDate) {
        return dataResultMapper.dataResultFiveMiCount(projectID,startDate,curDate);
    }

    @Override
    public int dataResultFifteenMiCount(String projectID,String startDate,String curDate) {
        return dataResultMapper.dataResultFifteenMiCount(projectID,startDate,curDate);
    }

    @Override
    public List<DataResultChEntity> queryDataResultSec(String projectID,String startDate,String curDate) {
        return dataResultMapper.queryDataResultCh(projectID,startDate,curDate);
    }

    @Override
    public void saveDataResultChiller(DataResultChEntity dataResultChEntity) {

        try{

            String deviceAddr=dataResultChEntity.getDeviceAddr();
            String registerAddr=dataResultChEntity.getRegisterAddr();

            String str="";
            if (dataResultChEntity.getCurDate()!=null){
                str=sdf1.format(dataResultChEntity.getCurDate());
            }else{
                Date date=new Date();//获取系统日期
                str=sdf1.format(date);
            }

            String curDate="";
            curDate=str.substring(0 ,16)+":00";
            String projectID=dataResultChEntity.getProjectID();
            dataResultChEntity.setCurDate(sdf1.parse(curDate));
            int row=dataResultMapper.selectDataResultChillerCount(curDate,deviceAddr,registerAddr,projectID, dataResultChEntity.getFunCode());//无记录
            if (row==0){
                dataResultMapper.saveDataResultChiller(dataResultChEntity);       //插入新的记录
            }else{
                dataResultMapper.updateDataResultChiller(dataResultChEntity);     //修改
            }

        }catch (Exception e){
            e.printStackTrace();
        }
    }

    @Override
    public void updateDataResultChiller(DataResultChEntity dataResultChEntity) {
        dataResultMapper.updateDataResultChiller(dataResultChEntity);
    }

    @Override
    public void saveDataResultNow(DataResultChEntity dataResultChEntity) {
        String deviceAddr=dataResultChEntity.getDeviceAddr();
        String registerAddr=dataResultChEntity.getRegisterAddr();
        String deviceType=dataResultChEntity.getDeviceType();
        String projectID=dataResultChEntity.getProjectID();
        dataResultMapper.deleteDataResultNow(deviceAddr,deviceType,registerAddr,projectID);
        dataResultMapper.saveDataResultNow(dataResultChEntity);
    }

    @Override
    public void deleteDataResultNow(String deviceAddr, String deviceType, String registerAddr, String projectID) {
        dataResultMapper.deleteDataResultNow(deviceAddr,deviceType,registerAddr,projectID);
    }

    @Override
    public List<DataResultChEntity> selectDataResultNow(String deviceAddr, String deviceType, String projectID) {
        return dataResultMapper.selectDataResultNow(deviceAddr, deviceType, projectID);
    }

    @Override
    public List<DataResultChEntity> queryDataResultChiller(String projectID, String deviceAddr,String registerName, String startDate, String curDate, int page, int limit) {
        return dataResultMapper.queryDataResultChiller(projectID,deviceAddr, registerName, startDate, curDate, page, limit);
    }

    @Override
    public int dataResultChillerCount(String projectID,String deviceAddr, String registerName, String startDate, String curDate) {
        return dataResultMapper.dataResultChillerCount(projectID,deviceAddr,registerName,startDate,curDate);
    }
}
