package com.mh.user.utils;

import com.alibaba.fastjson.JSON;
import com.mh.user.UserServiceApplication;
import com.mh.user.config.RabbitmqConfig;
import com.mh.user.constants.Constant;
import com.mh.user.entity.*;
import com.mh.user.model.QueueParam;
import com.mh.user.service.DataResultService;
import com.mh.user.service.DeviceCodeParamService;
import com.mh.user.service.ProjectInfoService;
import com.mh.user.service.chillers.ChillersService;
import com.mh.user.service.chillers.DataHistoryService;
import com.mh.user.service.chillers.OrderMessageService;
import com.mh.user.service.impl.DataResultServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.log4j.Logger;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.boot.SpringApplication;
import org.springframework.context.ApplicationContext;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * @author ljf
 * @title ：
 * @description ： 解析485接收的数据
 * @updateTime 2020-04-23
 * @throws ：
 */
//@Slf4j
public class AnalysisReceiveOrder485 {

	private static final Logger log = Logger.getLogger(AnalysisReceiveOrder485.class);
	// 调用service
	ApplicationContext context = SpringBeanUtil.getApplicationContext();
	ChillersService chillersService = context.getBean(ChillersService.class);
	OrderMessageService orderMessageService = context.getBean(OrderMessageService.class);
	ThreadPoolExecutor threadPoolService = ThreadPoolService.getInstance();
	DataResultService dataResultService=context.getBean(DataResultService.class);
	DeviceCodeParamService deviceCodeParamService=context.getBean(DeviceCodeParamService.class);
	RabbitTemplate rabbitTemplate=context.getBean(RabbitTemplate.class);
	ProjectInfoService projectInfoService=context.getBean(ProjectInfoService.class);

	private final SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	private final DecimalFormat df = new DecimalFormat("#.##");

	//解析冷量表
	public void analysisCloudOrder485(final String dataStr1,final DeviceCodeParamEntity deviceCodeParam) {
		threadPoolService.execute(() -> {
			// 去掉空格
			String dataStr = dataStr1.replace(" ", "").toUpperCase();
			// 检验报文
			String checkStr = dataStr.substring(0, dataStr.length() - 4);
			byte[] strOrder = ExchangeStringUtil.hexStrToBinaryStr(checkStr);
			int checkNum = CRC16.CRC16_MODBUS(strOrder);
			String checkWord = ExchangeStringUtil.decToHex(String.valueOf(checkNum));
			checkWord = checkWord.substring(2, 4) + checkWord.substring(0, 2);

			if (checkWord.equalsIgnoreCase(dataStr.substring(dataStr.length() - 4))) {
				//创建SimpleDateFormat对象，指定样式    2019-05-13 22:39:30
				Date date = new Date();
				String dateStr="";
				//保留两位小数处理
				DecimalFormat decimalFormat = new DecimalFormat("0.00");
				// 表号
				String cloudId = ExchangeStringUtil.hexToDec(dataStr.substring(0, 2));
				// 读数
				String data="";
				data = dataStr.substring(dataStr.length() - 8, dataStr.length() - 6)
						+ dataStr.substring(dataStr.length() - 6, dataStr.length() - 4)
						+ dataStr.substring(dataStr.length() - 12, dataStr.length() - 10)
						+ dataStr.substring(dataStr.length() - 10, dataStr.length() - 8);

				DataResultChEntity dataResultCh=new DataResultChEntity();
				DataResultClEntity dataResultCl=new DataResultClEntity();
				String registerAddr=deviceCodeParam.getRegisterAddr();
				if (ExchangeStringUtil.isInDate(date, "00:00:00", "00:00:30")) {
					dateStr= sdf1.format(date);
					dateStr=dateStr.substring(0,17)+"00";

					System.out.println("插入时间00"+dateStr);
				}else if (ExchangeStringUtil.isInDate(date, "00:00:30", "00:00:59")) {
					dateStr= sdf1.format(date);
					dateStr=dateStr.substring(0,17)+"30";

					System.out.println("插入时间30"+dateStr);
				}
				try {
					if(registerAddr.equals("32") || registerAddr.equals("33") || registerAddr.equals("35") || registerAddr.equals("36")){
					  dataResultCh.setDeviceAddr(cloudId);
					  dataResultCh.setDeviceType(deviceCodeParam.getDeviceType());

					  dataResultCh.setCurDate(sdf1.parse(dateStr));
					  data = decimalFormat.format(Math.abs(ExchangeStringUtil.hexToSingle(data)));//十六进制字符串转IEEE754浮点型
					  dataResultCh.setCurValue(data);
					  dataResultCh.setRegisterAddr(deviceCodeParam.getRegisterAddr());
					  dataResultCh.setRegisterName(deviceCodeParam.getRegisterName());
					  dataResultCh.setGrade(deviceCodeParam.getGrade());
					  dataResultCh.setProjectID(deviceCodeParam.getProjectID());
					  dataResultCh.setGrade(deviceCodeParam.getGrade());
					  String projectName=projectInfoService.selectName(deviceCodeParam.getProjectID());
					  log.info("冷量计==>"+cloudId+",读数==>"+dataResultCh.getCurValue()+",项目名称==>"+projectName);
					  dataResultService.saveDataResultCh(dataResultCh);
					  log.info("冷量计保存数据库成功!项目名称:"+projectName);
				  }else if (registerAddr.equals("31") || registerAddr.equals("34")){
					  dataResultCl.setDeviceAddr(cloudId);
					  dataResultCl.setDeviceType(deviceCodeParam.getDeviceType());
					  dataResultCh.setCurDate(sdf1.parse(dateStr));
					  long lData= Long.parseLong(ExchangeStringUtil.hexToDec(data));
					  dataResultCl.setCurValue(lData);//字符串转整型
					  dataResultCl.setRegisterAddr(deviceCodeParam.getRegisterAddr());
					  dataResultCl.setRegisterName(deviceCodeParam.getRegisterName());
					  dataResultCl.setGrade(deviceCodeParam.getGrade());
					  dataResultCl.setProjectID(deviceCodeParam.getProjectID());
					  String projectName=projectInfoService.selectName(deviceCodeParam.getProjectID());
					  log.info("冷量计==>" + cloudId+",累计读数==>" + lData+",项目名称==>"+projectName);
					  if (lData<99999999){
					  	if (lData!=2231365){
							dataResultService.saveDataResultCl(dataResultCl);
						}
					  }
					  log.info("冷量计保存数据成功!项目名称:"+projectName);
					  dataResultService.saveDataResultCl_bak(dataResultCl);
                  }
				} catch (Exception e) {
					e.printStackTrace();
					log.error(e);
				}
			}else{
				log.info("冷量计校验失败===>" + dataStr);
			}
		});
	}

	public String analysisCloudQueue(QueueParam queueParam) {
		String rec="fail";
        String dataStr1=queueParam.getRecData(); //采集返回数据
		// 去掉空格
		String dataStr = dataStr1.replace(" ", "").toUpperCase();
		// 检验报文
		String checkStr = dataStr.substring(0, dataStr.length() - 4);
		byte[] strOrder = ExchangeStringUtil.hexStrToBinaryStr(checkStr);
		int checkNum = CRC16.CRC16_MODBUS(strOrder);
		String checkWord = ExchangeStringUtil.decToHex(String.valueOf(checkNum));
		checkWord = checkWord.substring(2, 4) + checkWord.substring(0, 2);

		if (checkWord.equalsIgnoreCase(dataStr.substring(dataStr.length() - 4))) {
			log.info("cloud message check success ===>" + dataStr);
			Date date = new Date();
			String dateStr="";
			//保留两位小数处理
			DecimalFormat decimalFormat = new DecimalFormat("0.00");
			// 表号
			String cloudId = ExchangeStringUtil.hexToDec(dataStr.substring(0, 2));
            String projectID=queueParam.getProjectID();
            String registerAddr=queueParam.getRegisterAddr();
            String deviceType=queueParam.getDeviceType();
			DeviceCodeParamEntity deviceCodeParam=deviceCodeParamService.queryDeviceCodeParam(projectID,deviceType,cloudId,registerAddr);
			// 读数
			String data="";
			data = dataStr.substring(dataStr.length() - 8, dataStr.length() - 6)
					+ dataStr.substring(dataStr.length() - 6, dataStr.length() - 4)
					+ dataStr.substring(dataStr.length() - 12, dataStr.length() - 10)
					+ dataStr.substring(dataStr.length() - 10, dataStr.length() - 8);

			DataResultChEntity dataResultCh=new DataResultChEntity();
			DataResultClEntity dataResultCl=new DataResultClEntity();
			try {
				if(registerAddr.equals("32") || registerAddr.equals("33") || registerAddr.equals("35")){
					dataResultCh.setDeviceAddr(cloudId);
					dataResultCh.setDeviceType(deviceCodeParam.getDeviceType());
					if (ExchangeStringUtil.isInDate(date, "00:00:00", "00:00:30")) {
						dateStr= sdf1.format(date);
						dateStr=dateStr.substring(0,17)+"00";
						dataResultCh.setCurDate(sdf1.parse(dateStr));
					}else if (ExchangeStringUtil.isInDate(date, "00:00:30", "00:00:59")) {
						dateStr= sdf1.format(date);
						dateStr=dateStr.substring(0,17)+"30";
						dataResultCh.setCurDate(sdf1.parse(dateStr));
						System.out.println("插入时间30"+dateStr);
					}

					data = decimalFormat.format(ExchangeStringUtil.hexToSingle(data));//十六进制字符串转IEEE754浮点型
					dataResultCh.setCurValue(data);
					dataResultCh.setRegisterAddr(deviceCodeParam.getRegisterAddr());
					dataResultCh.setRegisterName(deviceCodeParam.getRegisterName());
					dataResultCh.setGrade(deviceCodeParam.getGrade());
					dataResultCh.setProjectID(deviceCodeParam.getProjectID());
					dataResultCh.setGrade(deviceCodeParam.getGrade());
					String projectName=projectInfoService.selectName(deviceCodeParam.getProjectID());
					log.info("cloudID==>" + cloudId+",curValue==>"+dataResultCh.getCurValue()+",projectName==>"+projectName);
					dataResultService.saveDataResultCh(dataResultCh);
					log.info("cloud save data success!projectName:"+projectName);
				}else if (registerAddr.equals("31") || registerAddr.equals("34")){
					dataResultCl.setDeviceAddr(cloudId);
					dataResultCl.setDeviceType(deviceCodeParam.getDeviceType());
					dateStr= sdf1.format(date);
					dataResultCl.setCurDate(sdf1.parse(dateStr));
					Long lData=Long.valueOf(ExchangeStringUtil.hexToDec(data));
					dataResultCl.setCurValue(lData);//字符串转整型
					dataResultCl.setRegisterAddr(deviceCodeParam.getRegisterAddr());
					dataResultCl.setRegisterName(deviceCodeParam.getRegisterName());
					dataResultCl.setGrade(deviceCodeParam.getGrade());
					dataResultCl.setProjectID(deviceCodeParam.getProjectID());
					String projectName=projectInfoService.selectName(deviceCodeParam.getProjectID());
					log.info("cloudID==>" + cloudId+",curValue34==>"+lData+",projectName==>"+projectName);
					if (lData<99999999){
						if (lData!=2231365){
							dataResultService.saveDataResultCl(dataResultCl);
						}
					}
					log.info("cloud save data success!projectName:"+projectName);
					dataResultService.saveDataResultCl_bak(dataResultCl);
				}
                rec="success";
			} catch (Exception e) {
				e.printStackTrace();
				log.error(e);
			}
		}
        return rec;
	}
	/**
	 * 解析电表返回的数据
	 * @param dataStr
	 */
	public void analysisMeterOrder485(final String dataStr,final DeviceCodeParamEntity deviceCodeParam) {
		threadPoolService.execute(() -> {
			//时间，创建SimpleDateFormat对象，指定样式    2019-05-13 22:39:30
			SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			Date date = new Date();
			String dateStr = sdf1.format(date);

			if (dataStr.length() == 36 || dataStr.length() == 40 || dataStr.length() == 44 || dataStr.length() == 50) {
				//检验报文
				String checkStr = dataStr.substring(0, dataStr.length() - 4); //减去校验码
				String checkNum = ExchangeStringUtil.makeChecksum(checkStr);  //生成校验码
				//返回的校验码与重新生成的校验码进行校验
				if (checkNum.equalsIgnoreCase(dataStr.substring(dataStr.length() - 4, dataStr.length() - 2))) {
					//表号，12位
					String meterId = checkStr.substring(12, 14) + checkStr.substring(10, 12) + checkStr.substring(8, 10)
							+ checkStr.substring(6, 8) + checkStr.substring(4, 6) + checkStr.substring(2, 4);
					meterId = String.format("%012d", Long.parseLong(meterId));
					//读数
					String data = "";
					StringBuilder stringBuilder = new StringBuilder();
					if (dataStr.length() == 36){
						for (int i = 0; i < 4; i++) {
							String data1 = checkStr.substring(32 - 2 * (i + 1), 32 - 2 * i);
							stringBuilder.append(data1);
						}
					}else {
						for (int i = 0; i < 4; i++) {
							String data1 = checkStr.substring(36 - 2 * (i + 1), 36 - 2 * i);
							stringBuilder.append(data1);
						}
					}
					data = stringBuilder.toString();
					data = ExchangeStringUtil.cutThree(data);
					// 0 代表前面补充0,4 代表长度为4,d 代表参数为正数型
					if (data.length()==6){
						data = String.format("%06d", Long.parseLong(data));
						data = data.substring(0, 2) + "." + data.substring(2, 6);
					}else if (data.length()==8){
						data = String.format("%08d", Long.parseLong(data));
						data = data.substring(0, 6) + "." + data.substring(6, 8);
					}

					try {
						DataResultEntity dataResultEntity = new DataResultEntity();
						dataResultEntity.setDeviceAddr(meterId);//通讯编号
						dataResultEntity.setDeviceType("电表");//类型
						dataResultEntity.setProjectID(deviceCodeParam.getProjectID());//所属项目
						dataResultEntity.setCurValue(Double.parseDouble(data));     //当前读数
						dataResultEntity.setCurDate(sdf1.parse(dateStr));   //当前日期
						dataResultEntity.setGrade(deviceCodeParam.getGrade());
						dataResultService.saveDataResult(dataResultEntity);
						String projectName=projectInfoService.selectName(deviceCodeParam.getProjectID());
						log.info("电表==>" + meterId+",读数==>" + data+",项目名称==>"+projectName);
						log.info("电表保存数据成功!项目名称:"+projectName);
					} catch (Exception e) {
						e.printStackTrace();
						log.error(e);
					}
				}else {
					log.info("电表校验失败===>"+ dataStr);
				}
			}
		});
	}

	public String analysisMeterQueue(QueueParam queueParam) {
		SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//		Date date = new Date();
//		String dateStr = sdf1.format(date);
		String dateStr=queueParam.getCopyTime();
//		log.info("dateTime:" + dateStr);
		String rec="fail";
        String dataStr=queueParam.getRecData();
		if (dataStr.length() == 32 || dataStr.length() == 36 || dataStr.length() == 40 || dataStr.length() == 44 || dataStr.length() == 50) {
			//检验报文
			String checkStr = dataStr.substring(0, dataStr.length() - 4); //减去校验码
			String checkNum = ExchangeStringUtil.makeChecksum(checkStr);  //生成校验码
			//返回的校验码与重新生成的校验码进行校验
			if (checkNum.equalsIgnoreCase(dataStr.substring(dataStr.length() - 4, dataStr.length() - 2))) {
				log.info("meter message check success ===>" + dataStr);
				//表号，12位
				String meterId = checkStr.substring(12, 14) + checkStr.substring(10, 12) + checkStr.substring(8, 10)
						+ checkStr.substring(6, 8) + checkStr.substring(4, 6) + checkStr.substring(2, 4);
				meterId = String.format("%012d", Long.parseLong(meterId));
				String projectID=queueParam.getProjectID();
				String registerAddr=queueParam.getRegisterAddr();
				String deviceType=queueParam.getDeviceType();
				DeviceCodeParamEntity deviceCodeParam=deviceCodeParamService.queryDeviceCodeParam(projectID,deviceType,meterId,registerAddr);
				//读数
				String data = "";
				StringBuilder stringBuilder = new StringBuilder();
				if (dataStr.length() == 36){
					for (int i = 0; i < 4; i++) {
						String data1 = checkStr.substring(32 - 2 * (i + 1), 32 - 2 * i);
						stringBuilder.append(data1);
					}
				}else if (dataStr.length() == 34){
					for (int i = 0; i < 3; i++) {
						String data1 = checkStr.substring(30 - 2 * (i + 1), 30 - 2 * i);
						stringBuilder.append(data1);
					}
				}else {
					for (int i = 0; i < 4; i++) {
						String data1 = checkStr.substring(36 - 2 * (i + 1), 36 - 2 * i);
						stringBuilder.append(data1);
					}
				}
				data = stringBuilder.toString();
				data = ExchangeStringUtil.cutThree(data);
				// 0 代表前面补充0,4 代表长度为4,d 代表参数为正数型
				if (data.length()==6){
					data = String.format("%06d", Long.parseLong(data));
					data = data.substring(0, 2) + "." + data.substring(2, 6);
				}else if (data.length()==8){
					data = String.format("%08d", Long.parseLong(data));
					data = data.substring(0, 6) + "." + data.substring(6, 8);
				}

				try {
					DataResultEntity dataResultEntity = new DataResultEntity();
					dataResultEntity.setDeviceAddr(meterId);//通讯编号
					dataResultEntity.setDeviceType("电表");//类型
					dataResultEntity.setProjectID(deviceCodeParam.getProjectID());//所属项目
					dataResultEntity.setCurValue(Double.parseDouble(data));     //当前读数
					dataResultEntity.setCurDate(sdf1.parse(dateStr));   //当前日期
					dataResultEntity.setGrade(deviceCodeParam.getGrade());
					dataResultService.saveDataResult(dataResultEntity);
					String projectName=projectInfoService.selectName(deviceCodeParam.getProjectID());
					log.info("meterID==>" + meterId+",meter curValue==>" + data+",projectName==>"+projectName);
					log.info("meter save data success!projectName:"+projectName);
				} catch (Exception e) {
					e.printStackTrace();
					log.error(e);
				}
				rec="success";
			}else {
				log.info("meter message check fail ===>"+ dataStr);
			}
		}
		return rec;
	}

	// 水表
	public void analysisWtMeterOrder485(final String dataStr,final String registerAddr,final String brand,String projectID) {
		threadPoolService.execute(() -> {
			//时间,创建SimpleDateFormat对象，指定样式 2019-05-13 22:39:30
			SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			Date date = new Date();
			String dateStr = sdf1.format(date);
			log.info("time==>" + dateStr);

			// 读数
			String data = "";
			String meterId="";
			if (dataStr.length() == 44 || dataStr.length() == 70) {
				// 检验报文
				String checkStr = dataStr.substring(0, dataStr.length() - 4);//减去校验码
				String checkNum = ExchangeStringUtil.makeChecksum(checkStr); //生成校验码
				if (checkNum.equalsIgnoreCase(dataStr.substring(dataStr.length() - 4, dataStr.length() - 2))) {
					log.info("wtMeter message check success ===>" + dataStr);
					// 表号
					meterId =checkStr.substring(16, 18)+checkStr.substring(14, 16)+ checkStr.substring(12, 14) + checkStr.substring(10, 12)
							+ checkStr.substring(8, 10)+ checkStr.substring(6, 8) + checkStr.substring(4, 6);
					meterId = String.format("%014d", Long.parseLong(meterId));
					log.info("meterID==>" + meterId);
					StringBuilder stringBuilder = new StringBuilder();
					for (int i = 0; i < 4; i++) {
						//String data1 = checkStr.substring(checkStr.length() - 2 * (i + 1), checkStr.length() - 2 * i);
						String data1 = checkStr.substring(36 - 2 * (i + 1), 36 - 2 * i);
						stringBuilder.append(data1);
					}
					data = stringBuilder.toString();
					// 0 代表前面补充0，4 代表长度为4，d 代表参数为正数型
					data = String.format("%08d", Long.parseLong(data));
					data = data.substring(0, 6) + "." + data.substring(6, 8);
				}else{
					log.info("wtMeter message check fail ===>" + dataStr);
				}
			} else if(dataStr.length() == 18){ //脉冲水表
				//检验报文
				String checkStr = dataStr.substring(0, dataStr.length() - 4);

				byte[] strOrder = ExchangeStringUtil.hexStrToBinaryStr(checkStr);
				int checkNum = CRC16.CRC16_MODBUS(strOrder);
				String checkWord = ExchangeStringUtil.decToHex(String.valueOf(checkNum));
				checkWord = checkWord.substring(2, 4) + checkWord.substring(0, 2);
				String sValue=null;
				if (checkWord.equalsIgnoreCase(dataStr.substring(dataStr.length() - 4))) {
					log.info("wtMeter message check success ===>" + dataStr);
					meterId = ExchangeStringUtil.hexToDec(checkStr.substring(0, 2));
					log.info("meterID==>" + meterId);
					if (checkStr.substring(2, 4).equalsIgnoreCase("03")) {
						data=String.valueOf(Integer.parseInt(ExchangeStringUtil.hexToDec(checkStr.substring(6, 14)))/10); //读数
					}
				}else {
					log.info("wtMeter message check fail ===>" + dataStr);
				}
			}

			log.info("wtMeter curValue==>" + data);
			try {
				DataResultEntity dataResultEntity = new DataResultEntity();
//				if (meterId.length()>8){
//					dataResultEntity.setDeviceAddr(meterId.substring(meterId.length()-8,meterId.length()));//通讯编号
//				}else{
					dataResultEntity.setDeviceAddr(meterId);//通讯编号
//				}
				dataResultEntity.setDeviceType("水表");
				dataResultEntity.setCurValue(Double.parseDouble(data));     //当前读数
				dataResultEntity.setCurDate(sdf1.parse(dateStr));           //当前日期
				dataResultEntity.setProjectID(projectID);
				dataResultService.saveDataResult(dataResultEntity);
				log.info("wtMeter save data success!projectID:"+projectID);
			} catch (Exception e) {
				e.printStackTrace();
				log.error(e);
			}
		});
	}
	/**
	 * 解析冷水机组返回的数据
	 */
	public void analysisChillerOrder485(final String dataStr,final DeviceCodeParamEntity deviceCodeParam) {
	  if (!Constant.CONTROL_WEB_FLAG) {
//        threadPoolService.execute(() -> {
		  String checkStr = dataStr.substring(0, dataStr.length() - 4); // 检验报文
		  byte[] strOrder = ExchangeStringUtil.hexStrToBinaryStr(checkStr);
		  int checkNum = CRC16.CRC16_MODBUS(strOrder);
		  String checkWord = ExchangeStringUtil.addZeroForNum(ExchangeStringUtil.decToHex(String.valueOf(checkNum)), 4);
		  checkWord = checkWord.substring(2, 4) + checkWord.substring(0, 2);

		  if (checkWord.equalsIgnoreCase(dataStr.substring(dataStr.length() - 4))) {
			  Date date = new Date();
			  String dateStr = sdf1.format(date);
			  // 冷水机组的地址
			  String chillerAddr = ExchangeStringUtil.hexToDec(dataStr.substring(0, 2));
			  DataResultChEntity dataResultCh=new DataResultChEntity();
			  String data = "";
			  if (dataStr.substring(2, 4).equalsIgnoreCase("01")) {            // 读取状态
				if (dataStr.length() == 12) {
				  // 状态
				  data = ExchangeStringUtil.hexToDec(dataStr.substring(6, 8));
				  try{
					  // 赋值给dataResultCh
					  initialDataResultCh(deviceCodeParam, dataResultCh, chillerAddr, dateStr, data);
				  }catch (Exception e){
					  e.printStackTrace();
					  log.error("冷水机报错：{}", e);
				  }
				}
			  } else if (dataStr.substring(2, 4).equalsIgnoreCase("03") || dataStr.substring(2, 4).equalsIgnoreCase("04")) {
				if (dataStr.length() == 14) {
				  if (deviceCodeParam.getDigit() == 1) {
					// 数据
					data = df.format(Integer.parseInt(ExchangeStringUtil.hexToDec(dataStr.substring(6, 10))) * 0.1);
				  } else if (deviceCodeParam.getDigit() == 2) {
					// 数据
					data = df.format(Integer.parseInt(ExchangeStringUtil.hexToDec(dataStr.substring(6, 10))) * 0.01);
				  } else {
					// 数据
					data = ExchangeStringUtil.hexToDec(dataStr.substring(6, 10));
				  }
				  try{
					 // 赋值给dataResultCh
					 initialDataResultCh(deviceCodeParam, dataResultCh, chillerAddr, dateStr, data);
				  }catch (Exception e){
						e.printStackTrace();
						log.error("冷水机保存异常：", e);
				  }
				}
			}
		  }else{
			  log.info("冷水机校验失败===>" + dataStr);
		  }
//		});
      }
	}

	/**
	 * 格式化数据
	 * @param deviceCodeParam
	 * @param dataResultCh
	 * @param chillerAddr
	 * @param dateStr
	 * @param data
	 * @throws ParseException
	 */
	private void initialDataResultCh(DeviceCodeParamEntity deviceCodeParam, DataResultChEntity dataResultCh, String chillerAddr, String dateStr, String data) throws ParseException {
		dataResultCh.setDeviceAddr(chillerAddr);
		dataResultCh.setDeviceType(deviceCodeParam.getDeviceType());
		dataResultCh.setCurDate(sdf1.parse(dateStr));
		dataResultCh.setCurValue(data);
		dataResultCh.setRegisterAddr(deviceCodeParam.getRegisterAddr());
		dataResultCh.setRegisterName(deviceCodeParam.getRegisterName());
		dataResultCh.setGrade(deviceCodeParam.getGrade());
		dataResultCh.setFunCode(deviceCodeParam.getFunCode());
		dataResultCh.setProjectID(deviceCodeParam.getProjectID());
		String projectName=projectInfoService.selectName(deviceCodeParam.getProjectID());
		log.info("冷水机:" + chillerAddr + ",状态:" + data +",项目名称:"+projectName);
		dataResultService.saveDataResultChiller(dataResultCh);
		dataResultService.deleteDataResultNow(deviceCodeParam.getDeviceAddr(), deviceCodeParam.getDeviceType(), deviceCodeParam.getRegisterAddr(), deviceCodeParam.getProjectID());
		log.info("冷水机保存成功!项目名称:"+projectName);
	}

	public void analysisChillersDDC(final String receiveData1) throws InterruptedException {
		// 去掉空格
//        c0a8020a1f90ff04fb0000000100010200000300000400000500000600000700140800000901cb0a00000b00000c00000d00000e00000f0000100000110025120000130910cabc
		String receiveData = receiveData1.replace(" ", "");
		receiveData = receiveData.toUpperCase();
		// 检验报文
		String checkStr = receiveData.substring(0, receiveData.length() - 4);
		byte[] strOrder = ExchangeStringUtil.hexStrToBinaryStr(checkStr);
		int checkNum = CRC16.CRC16_MODBUS(strOrder);
		String checkWord = ExchangeStringUtil.decToHex(String.valueOf(checkNum));
		checkWord = ExchangeStringUtil.addZeroForNum(checkWord, 4);
		checkWord = checkWord.substring(2, 4) + checkWord.substring(0, 2);

		if (checkWord.equalsIgnoreCase(receiveData.substring(receiveData.length() - 4))) {
			List<ChillersEntity> chillersEntityList = new ArrayList<>();
			// IP地址
			String ip = receiveData.substring(0, 8);
			// 16进制转IP地址
			ip = ExchangeStringUtil.hexToDec(ip.substring(0, 2)) + "." +
					ExchangeStringUtil.hexToDec(ip.substring(2, 4)) + "." +
					ExchangeStringUtil.hexToDec(ip.substring(4, 6)) + "." +
					ExchangeStringUtil.hexToDec(ip.substring(6, 8));
			// 端口号
			String port = receiveData.substring(8, 12);
			// 结束符
			String endStr = receiveData.substring(12, 14);
			// DDC设备地址
			String ddcAddr = receiveData.substring(14, 16);
			ddcAddr = ExchangeStringUtil.hexToDec(ddcAddr);
//      log.info("DDC地址： " + ddcAddr);
			// 通过DDC地址查询对应的采集参数设备
			List<String> stringList = chillersService.queryDDCAddr(ddcAddr);

			// DDC地址反码
			String reversDDCAddr = receiveData.substring(16, 18);
			// 开始解析各个寄存器地址读数
			String registerAddr = "";
			String readData = "";
			for (int i = 0; i < (receiveData.length() - 22) / 6; i++) {
				// 寄存器地址+数据域（00 + 0000）
				String readStr = receiveData.substring(18 + 6 * i, 18 + 6 * (i + 1));
				// 寄存器地址
				registerAddr = registerAddr + readStr.substring(0, 2);
				// 数据
				readData = readData + readStr.substring(2, 6);
				// 判断是否存在寄存器
				boolean flag = queryRegisterAddr(stringList, registerAddr);
				//创建SimpleDateFormat对象，指定样式    2019-05-13 22:39:30
				SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.sss");
				Date date = new Date();
				String dateStr = sdf1.format(date);
				if (flag) {
//                    long register = Long.parseLong(registerAddr, 16);
					long data = Long.parseLong(readData, 16);
//          log.info("寄存器地址： " + registerAddr + "，数据： " + data);
					// 新new一个实体类
					ChillersEntity chillersEntity = new ChillersEntity();
					chillersEntity.setDdcAddr(ddcAddr);
					chillersEntity.setIP(ip);
					chillersEntity.setLastValue(String.valueOf(data));
					chillersEntity.setLastTime(dateStr);
					chillersEntity.setRegisterAddress(registerAddr);
					chillersEntity.setGrade(0);
					chillersEntityList.add(chillersEntity);
					registerAddr = "";
					readData = "";
				}
			}

			Thread.sleep(1000);
			if (Constant.CONTROL_WEB_FLAG) {
				// 1、读取历史流水表数据，判断最新存储的时间是否是超过5分钟 update by ljf on 2020-08-10
				String lastTime = chillersService.queryLastChillersHistory(chillersEntityList.get(chillersEntityList.size() - 1));
				try {
					if ( lastTime == null || lastTime.equalsIgnoreCase("") ) {
						// 2、超过5分钟则进行插入操作 批量插入历史流水表
						chillersService.batchInsertChillersHistory(chillersEntityList);
					} else {
						int subtract = TimeDifferenceUtil.dValue(lastTime);
//					int subtract = dValue(lastTime);
						if (subtract >= 30) {
							// 2、超过5分钟则进行插入操作 批量插入历史流水表
							chillersService.batchInsertChillersHistory(chillersEntityList);
						}
					}
					// 3、否则仅仅更新对应的实时记录表
					// 避免出现RPC 请求中提供了过多的参数。最多应为 2100报错，循环分批次插入update by ljf on 2019-08-23
					int resultInt = 0;
					int batchCount = 5;// 每批commit的个数
					int batchLastIndex = batchCount;// 每批最后一个的下标
					try {
						for (int index = 0; index < chillersEntityList.size(); ) {
							Thread.sleep(500);
							if (batchLastIndex >= chillersEntityList.size()) {
								batchLastIndex = chillersEntityList.size();
								resultInt = resultInt * chillersService.batchUpdateRegisterAddr(chillersEntityList.subList(index, batchLastIndex));
								System.out.println("index:" + index + " batchLastIndex:" + batchLastIndex);
								break;// 数据插入完毕，退出循环
							} else {
								resultInt = resultInt * chillersService.batchUpdateRegisterAddr(chillersEntityList.subList(index, batchLastIndex));
								System.out.println("index:" + index + " batchLastIndex:" + batchLastIndex);
								index = batchLastIndex;// 设置下一批下标
								batchLastIndex = index + (batchCount - 1);
							}
						}
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				} catch (ParseException e) {
					e.printStackTrace();
				}
			} else {
				threadPoolService.execute(() -> {
					// 1、读取历史流水表数据，判断最新存储的时间是否是超过5分钟 update by ljf on 2020-08-10
					String lastTime = chillersService.queryLastChillersHistory(chillersEntityList.get(chillersEntityList.size() - 1));
					try {
						if ( lastTime == null || lastTime.equalsIgnoreCase("") ) {
							// 2、超过5分钟则进行插入操作 批量插入历史流水表
							chillersService.batchInsertChillersHistory(chillersEntityList);
						} else {
							int subtract = TimeDifferenceUtil.dValue(lastTime);
//					int subtract = dValue(lastTime);
							if (subtract >= 30) {
								// 2、超过5分钟则进行插入操作 批量插入历史流水表
								chillersService.batchInsertChillersHistory(chillersEntityList);
							}
						}
						// 3、否则仅仅更新对应的实时记录表
						// 避免出现RPC 请求中提供了过多的参数。最多应为 2100报错，循环分批次插入update by ljf on 2019-08-23
						int resultInt = 0;
						int batchCount = 5;// 每批commit的个数
						int batchLastIndex = batchCount;// 每批最后一个的下标
						try {
							for (int index = 0; index < chillersEntityList.size(); ) {
								Thread.sleep(800);
								if (batchLastIndex >= chillersEntityList.size()) {
									batchLastIndex = chillersEntityList.size();
									resultInt = resultInt * chillersService.batchUpdateRegisterAddr(chillersEntityList.subList(index, batchLastIndex));
									System.out.println("index:" + index + " batchLastIndex:" + batchLastIndex);
									break;// 数据插入完毕，退出循环
								} else {
									resultInt = resultInt * chillersService.batchUpdateRegisterAddr(chillersEntityList.subList(index, batchLastIndex));
									System.out.println("index:" + index + " batchLastIndex:" + batchLastIndex);
									index = batchLastIndex;// 设置下一批下标
									batchLastIndex = index + (batchCount - 1);
								}
							}
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					} catch (ParseException e) {
						e.printStackTrace();
					}
				});
			}
		}
	}

	public static int dValue(String lastDate) throws ParseException {
		SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		Date lastTime = format.parse(lastDate);
		long min = lastTime.getTime();
		Calendar calendar = Calendar.getInstance();
		long min1 = calendar.getTimeInMillis();
		long subtract = min1 - min;
//        System.out.println("相减值： " + subtract/(1000*60));
		return (int) subtract / (1000 * 60);
	}

	public void analysisSendOrder(String receiveData, OrderMessageEntity orderMessageEntity) {
		// 解析返回来的报文
		if (receiveData.startsWith("000000000006010600")) {
			// 获取寄存器地址
			String registerAddr = receiveData.substring(18, 20);
			// 更新对应的状态值
			String dataStr = receiveData.substring(20, 24);
			dataStr = ExchangeStringUtil.hexToDec(dataStr);
			try {
				// 更新数据库对应的发送状态值
				orderMessageService.updateOrderMessage(orderMessageEntity);
				// 以及设备管理表对应的值
				orderMessageService.updateChillersRegisterAddress(dataStr, orderMessageEntity.getRegisterAddr());
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	// 判断是否存在寄存器地址
	public Boolean queryRegisterAddr(List<String> stringList, String registerAddr) {
		boolean flag = false;
		for (int i = 0; i < stringList.size(); i++) {
			if (stringList.get(i).equalsIgnoreCase(registerAddr)) {
				flag = true;
				break;
			}
		}
		return flag;
	}

	public void sendQueue(QueueParam queueParam){

		Map map=new HashMap();
		map.put("recData",queueParam.getRecData());
		map.put("deviceType",queueParam.getDeviceType());
		map.put("registerAddr",queueParam.getRegisterAddr());
		map.put("projectID",queueParam.getProjectID());
		map.put("copyTime",queueParam.getCopyTime());
		String jsonString = JSON.toJSONString(map); //map转json字符串
		rabbitTemplate.convertAndSend(RabbitmqConfig.EXCHANGE_NAME,RabbitmqConfig.ROUTINGKEY_NAME,jsonString);
		log.info("发送"+queueParam.getDeviceType()+"数据到交换机"+RabbitmqConfig.EXCHANGE_NAME);
	}
}
