package com.mh.user.utils;

import com.mh.user.entity.*;
import com.mh.user.serialport.SerialPortThread;
import com.mh.user.service.DeviceCodeParamService;
import com.mh.user.service.DeviceInstallService;
import com.mh.user.service.RegisterAddrService;
import com.mh.user.service.chillers.OrderMessageService;
import org.apache.log4j.Logger;
import org.springframework.context.ApplicationContext;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static com.mh.user.utils.ExchangeStringUtil.doubleTrans1;

/**
 * @author ljf
 * @title ：
 * @description ：
 * @updateTime 2020-04-22
 * @throws ：
 */
//@Slf4j
public class GetReadOrder485 {

    // 调用service
    ApplicationContext context = SpringBeanUtil.getApplicationContext();
    OrderMessageService orderMessageService = context.getBean(OrderMessageService.class);
    RegisterAddrService registerAddrService = context.getBean(RegisterAddrService.class);
    DeviceInstallService deviceInstallService=context.getBean(DeviceInstallService.class);
    DeviceCodeParamService deviceCodeParamService=context.getBean(DeviceCodeParamService.class);

    private static final Logger log = Logger.getLogger(SerialPortThread.class);
    // 电表
    public static String createMeterOrder(DeviceCodeParamEntity deviceCodeParam){
        String str=null;
        String deviceAddr=deviceCodeParam.getDeviceAddr();
        String registerAddr=deviceCodeParam.getRegisterAddr();

        if (deviceAddr!=null && deviceAddr.length()>0){
            try{
                //0代表前面补充0,12代表长度为12,d代表参数为正数型,基表通讯号
                str = String.format("%012d", Long.parseLong(deviceAddr));
                //转换位置
                str = ExchangeStringUtil.changePosition(str);
                //拼接功能码
                str = "68" + str + "680102";
                String strData="";
                if (registerAddr!=null && registerAddr.length()>0 && registerAddr.equalsIgnoreCase("9010")){
//                    str = "68" + str + "680102";
                    strData = "1090";
                }else if (registerAddr.equalsIgnoreCase("B630")){
                    strData = "30B6";
//                    str = "68" + str + "681104";
//                    strData = "00000302";
                }

                //加33操作
                str = str + ExchangeStringUtil.addThree(strData);
                //检验和
                String checkSum = ExchangeStringUtil.makeChecksum(str);
                str = "FEFEFE" + str + checkSum + "16";

            }catch (Exception e){
                e.printStackTrace();
            }
        }
        log.info("send meter message ===>" + str);
        return str.toUpperCase();
    }

    // 水表
    public static String createWtMeterOrder(DeviceCodeParamEntity deviceCodeParam){
        String deviceAddr=deviceCodeParam.getDeviceAddr();
        String brand=deviceCodeParam.getBrand();
        String str=null;
        if(deviceAddr!=null && deviceAddr.length()>0){
            try{
                if (brand==null || brand.equals("") || brand.equals("艾美柯")){
                    // 0 代表前面补充0,14 代表长度为14,d 代表参数为正数型
                    str = String.format("%014d", Long.parseLong(deviceAddr));//基表通讯号
                    // 转换位置
                    str = ExchangeStringUtil.changePosition(str);
                    // 拼接功能码
                    str = "6810" + str + "0103901F00";   //水表读读数指令
                    // 检验和
                    String checkSum = ExchangeStringUtil.makeChecksum(str);
                    str = "FEFEFE" + str + checkSum + "16";

                }else if(brand.equals("脉冲")){
                    str = ExchangeStringUtil.decToHex(deviceAddr);
                    str=ExchangeStringUtil.addZeroForNum(str,2);
                    str=str+"0302020002";//读表累计值

                    byte[] strOrder = ExchangeStringUtil.hexStrToBinaryStr(str);
                    int checkNum = CRC16.CRC16_MODBUS(strOrder);
                    String checkWord = ExchangeStringUtil.decToHex(String.valueOf(checkNum));
                    checkWord = ExchangeStringUtil.addZeroForNum(checkWord,4);
                    checkWord = checkWord.substring(2,4) + checkWord.substring(0,2);
                    str = str + checkWord;
                }
            }catch (Exception e){
                e.printStackTrace();
            }
        }
        log.info("send wtMeter message ===>" + str);
        return str.toUpperCase();
    }

    // 冷量计
    public static String createCloudOrder(DeviceCodeParamEntity deviceCodeParam){
        // 0 代表前面补充0,2 代表长度为2,d 代表参数为正数型
        String cloudNum=deviceCodeParam.getDeviceAddr();
        String str = ExchangeStringUtil.decToHex(cloudNum);
        str=ExchangeStringUtil.addZeroForNum(str,2);
        String cloudType=deviceCodeParam.getRegisterAddr();
        if (cloudType.equalsIgnoreCase("4")){       //温控设备,地址为十六进制
            str =  str + "0300000002";
        } else if (cloudType.equalsIgnoreCase("31")){       //流量计冷冻水量
            str =  str + "0300080002";
        } else if (cloudType.equalsIgnoreCase("32")){       // 流量计出水温度
            str =  str + "0300490002";
        } else if (cloudType.equalsIgnoreCase("33")){       // 流量计回水温度
            str =  str + "03004B0002";
        } else if (cloudType.equalsIgnoreCase("34")){       // 流量计冷量累积量
            str =  str + "0300500002";
        } else if (cloudType.equalsIgnoreCase("35")){       // 瞬时流量/小时
            str =  str + "0300040002";  //0300000002（秒）、0300020002（分）、0300040002（时）
        } else if (cloudType.equalsIgnoreCase("36")){       // 瞬时冷量
            str =  str + "0300140002";
        }
        byte[] strOrder = ExchangeStringUtil.hexStrToBinaryStr(str);
        int checkNum = CRC16.CRC16_MODBUS(strOrder);
        String checkWord = ExchangeStringUtil.decToHex(String.valueOf(checkNum));
        checkWord = ExchangeStringUtil.addZeroForNum(checkWord,4);
        checkWord = checkWord.substring(2,4) + checkWord.substring(0,2);
        str = str + checkWord;

//        str = ExchangeStringUtil.getCRCStr(str);
        log.info("send cloud message ===>" + str);
        return str.toUpperCase();
    }

    /**
     * @param DeviceCodeParam
     * @return
     */
    public static String createChillersOrder(DeviceCodeParamEntity DeviceCodeParam) {
        String str = null;
        try {
            // 获取冷水机组地址
            String chillerAddr = DeviceCodeParam.getDeviceAddr();
            chillerAddr = ExchangeStringUtil.decToHex(chillerAddr);
            String str1 = ExchangeStringUtil.addZeroForNum(chillerAddr, 2);
            // 获取功能码
            String funCode = DeviceCodeParam.getFunCode();
            str1 = str1 + ExchangeStringUtil.addZeroForNum(ExchangeStringUtil.decToHex(funCode),2);
            // 寄存器地址
            String registerAddr = DeviceCodeParam.getRegisterAddr();
            str1 = str1 + ExchangeStringUtil.addZeroForNum(registerAddr,4) + "0001";
            log.info(str1);
            byte[] strOrder = ExchangeStringUtil.hexStrToBinaryStr(str1);
            int checkNum = CRC16.CRC16_MODBUS(strOrder);
            String checkWord = ExchangeStringUtil.decToHex(String.valueOf(checkNum));
            checkWord = ExchangeStringUtil.addZeroForNum(checkWord,4);
            checkWord = checkWord.substring(2,4) + checkWord.substring(0,2);
            str = str1 + checkWord;
        } catch (Exception e) {
            e.printStackTrace();
        }
        log.info("send chiller message ===>" + str);
        return str.toUpperCase();
    }

    // DDC 设备指令生成
    // 服务端 IP（HEX） + 端口(HEX) + 设备地址号(HEX) + CRC16 校验
    public static String createDDCOrder(ChillersEntity chillersEntity) {
        String result = "";
        // 把IP地址转为16进制
        String ip = ExchangeStringUtil.ipToHex(chillersEntity.getIP());
        // 端口号转为16进制
        String port = ExchangeStringUtil.decToHex(String.valueOf(chillersEntity.getPort()));
        // DDC地址
        String ddcAddr = ExchangeStringUtil.decToHex(chillersEntity.getDdcAddr());
        // 拼接成采集报文
        result = ip + port + ddcAddr;
        // 得出校验位后的值
        result = ExchangeStringUtil.getCRCStr(result);
        return result;
    }

    // 生成对设备操作指令
    public List<OrderMessageEntity> createOrder(List<OrderEntity> orderEntityList) throws InterruptedException, ParseException {
        List<OrderMessageEntity> orderMessageEntityList = new ArrayList<>();
        ChillersEntity chillersEntity = new ChillersEntity();
        // 格式化时间
        SimpleDateFormat sdf1=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        // 判断是否是有修改频率操作：size=2:频率+启停；size=1：启停
        if (orderEntityList.size() == 1) {
            // 启停状态
            for (OrderEntity orderEntity:
                    orderEntityList) {
                if (orderEntity.getType() == 3 || orderEntity.getType() == 4
                || orderEntity.getType().equals("3") || orderEntity.getType().equals("4")) {
                    // 不用查询对应的父ID对应的值
                    chillersEntity = orderMessageService.queryChillerById(orderEntity.getId());
                } else {
                    // 查询父ID是其ID的对应数据
                    // 得到对应的寄存器地址
                    chillersEntity = orderMessageService.queryChiller(orderEntity.getId());
                }
                // 拼接数据库采集操作报文
                StringBuilder orderStr = new StringBuilder("000000000006010600");
                // 比如1.6Mpa传入为160的十六进制
                if (orderEntity.getType() == 5 || orderEntity.getType().equals("5")) {
                    orderStr.append(chillersEntity.getRegisterAddress());
                    orderStr.append("00");
                    orderStr.append(ExchangeStringUtil.addZeroForNum(ExchangeStringUtil.decToHex(doubleTrans1(Double.parseDouble(orderEntity.getParam()) * 100)),2));
                } else if (orderEntity.getType() == 4 || orderEntity.getType().equals("4")) {    // 修改温度*10倍
                    orderStr.append(chillersEntity.getRegisterAddress());
                    orderStr.append(ExchangeStringUtil.addZeroForNum(ExchangeStringUtil.decToHex(doubleTrans1(Double.parseDouble(orderEntity.getParam()) * 10)),4));
                } else {
                    orderStr.append(chillersEntity.getRegisterAddress());
                    orderStr.append(ExchangeStringUtil.addZeroForNum(ExchangeStringUtil.decToHex(orderEntity.getParam()), 4));
                }
                // 存入orderMessageEntity数组中
                OrderMessageEntity orderMessageEntity = new OrderMessageEntity();
                orderMessageEntity.setRegisterAddr(chillersEntity.getRegisterAddress());
                orderMessageEntity.setRegisterName(chillersEntity.getRegisterName());
                orderMessageEntity.setOtherName(chillersEntity.getOtherName());
                orderMessageEntity.setOrderStr(orderStr.toString());
                orderMessageEntity.setSendTime(null);
                orderMessageEntity.setSendNum(0);
                orderMessageEntity.setStatus(0);
                orderMessageEntity.setGrade(0);
                orderMessageEntity.setCreateTime(sdf1.parse(sdf1.format(new Date())));
                orderMessageEntityList.add(orderMessageEntity);
            }
        } else if (orderEntityList.size() == 2) {
            // 启停
            String status = "";
            // 频率
            String rate = "";
            // 查询父ID是其ID的对应数据
            for (OrderEntity orderEntity:
                    orderEntityList) {
                if (orderEntity.getType() == 1 || orderEntity.getType().equals("1")) {   // 修改开关状态
                    // 得到对应的寄存器地址
                    // 统一查询启停状态对应的就好
                    chillersEntity = orderMessageService.queryChiller(orderEntity.getId());
                    status = ExchangeStringUtil.decToHex(orderEntity.getParam());
                } else if (orderEntity.getType() == 0 || orderEntity.getType().equals("0")) {    // 修改频率,精确到0.5，发送到DDC需要乘以2，update by ljf on 2020-06-16
                    rate = doubleTrans1(Double.parseDouble(orderEntity.getParam()) * 2);
                    rate = ExchangeStringUtil.decToHex(rate);
                }
            }
            log.info("寄存器地址： " + chillersEntity.getRegisterAddress());
            // 拼接数据库采集操作报文
            String orderStr = "000000000006010600"
                    + chillersEntity.getRegisterAddress()
                    + ExchangeStringUtil.addZeroForNum(rate,2)
                    + ExchangeStringUtil.addZeroForNum(status,2);
            // 存入orderMessageEntity数组中
            OrderMessageEntity orderMessageEntity = new OrderMessageEntity();
            orderMessageEntity.setRegisterAddr(chillersEntity.getRegisterAddress());
            orderMessageEntity.setRegisterName(chillersEntity.getRegisterName());
            orderMessageEntity.setOtherName(chillersEntity.getOtherName());
            orderMessageEntity.setOrderStr(orderStr);
            orderMessageEntity.setSendTime(null);
            orderMessageEntity.setSendNum(0);
            orderMessageEntity.setStatus(0);
            orderMessageEntity.setGrade(0);
            orderMessageEntity.setCreateTime(sdf1.parse(sdf1.format(new Date())));
            orderMessageEntityList.add(orderMessageEntity);
        }
        // 增加读取对应DDC关联设备的状态值 update by ljf on 2020-06-04
        Thread.sleep(1500);
        if (chillersEntity.getDdcAddr() != null || !chillersEntity.getDdcAddr().equalsIgnoreCase("")) {
            String ddcRead = createDDCOrder(chillersEntity);
            // 存入orderMessageEntity数组中
            OrderMessageEntity orderMessageEntity = new OrderMessageEntity();
            orderMessageEntity.setRegisterAddr(chillersEntity.getRegisterAddress());
            orderMessageEntity.setRegisterName(chillersEntity.getRegisterName());
            orderMessageEntity.setOtherName(chillersEntity.getOtherName());
            orderMessageEntity.setOrderStr(ddcRead);
            orderMessageEntity.setSendTime(null);
            orderMessageEntity.setSendNum(0);
            orderMessageEntity.setStatus(0);
            orderMessageEntity.setGrade(0);
            orderMessageEntity.setCreateTime(sdf1.parse(sdf1.format(new Date())));
            orderMessageEntityList.add(orderMessageEntity);
        }
        // 批量插入数据库
        orderMessageService.batchInsertOrderMessage(orderMessageEntityList);
        return orderMessageEntityList;
    }

    // 生成对设备操作指令参数
    public void createOrderParam(String ProjectID){
        List<DeviceInstallEntity> deviceList1;
        List<DeviceCodeParamEntity> deviceCodeParamEntityList = new ArrayList<>();
        int r=0;
        if(ProjectID!=null && !ProjectID.equals("")){
            r=deviceCodeParamService.queryCount(ProjectID);//查询记录数
        }else{
            r=deviceCodeParamService.queryCount2();//查询记录数
        }
        if (r==0){
            if(ProjectID!=null && !ProjectID.equals("")){
                deviceList1=deviceInstallService.selectDeviceParams(ProjectID);//查找多个采集参数的设备，多个
            }else{
                deviceList1=deviceInstallService.selectDeviceParams2();
            }

            System.out.println("========================生成采集参数=======================");
            if (deviceList1.size()>0){
                for (DeviceInstallEntity devices:deviceList1) {
                    if (devices.getDeviceType().equals("冷量计")){
                        //流量计冷冻水
                        DeviceCodeParamEntity deviceCodeParamEntity1 = new DeviceCodeParamEntity();
                        deviceCodeParamEntity1.setRegisterAddr("34");
                        deviceCodeParamEntity1.setFunCode("03");
                        deviceCodeParamEntity1.setDeviceAddr(devices.getDeviceAddr());
                        deviceCodeParamEntity1.setDeviceName(devices.getDeviceName());
                        deviceCodeParamEntity1.setDeviceType(devices.getDeviceType());
                        deviceCodeParamEntity1.setBaudRate(devices.getBaudRate());
                        deviceCodeParamEntity1.setBrand(devices.getBrand());
                        deviceCodeParamEntity1.setDataPort(devices.getDataPort());
                        deviceCodeParamEntity1.setProjectID(devices.getProjectID());
                        deviceCodeParamEntity1.setParity(devices.getParity());
                        deviceCodeParamEntity1.setGrade(devices.getGrade());
                        deviceCodeParamEntity1.setDigit(0); //保留0小数位
                        deviceCodeParamEntity1.setRegisterName("累积量");
                        String strData1=createCloudOrder(deviceCodeParamEntity1);
                        deviceCodeParamEntity1.setStrData(strData1);
                        deviceCodeParamEntityList.add(deviceCodeParamEntity1);//添加到列表
                        //流量计冷量累积量
                        DeviceCodeParamEntity deviceCodeParamEntity2 = new DeviceCodeParamEntity();
                        deviceCodeParamEntity2.setRegisterAddr("35");
                        deviceCodeParamEntity2.setFunCode("03");
                        deviceCodeParamEntity2.setDeviceAddr(devices.getDeviceAddr());
                        deviceCodeParamEntity2.setDeviceName(devices.getDeviceName());
                        deviceCodeParamEntity2.setDeviceType(devices.getDeviceType());
                        deviceCodeParamEntity2.setBaudRate(devices.getBaudRate());
                        deviceCodeParamEntity2.setBrand(devices.getBrand());
                        deviceCodeParamEntity2.setDataPort(devices.getDataPort());
                        deviceCodeParamEntity2.setProjectID(devices.getProjectID());
                        deviceCodeParamEntity2.setParity(devices.getParity());
                        deviceCodeParamEntity2.setDigit(2);   //保留小数位
                        deviceCodeParamEntity2.setRegisterName("瞬时流量");
                        String strData2=createCloudOrder(deviceCodeParamEntity2);
                        deviceCodeParamEntity2.setStrData(strData2);
                        deviceCodeParamEntity2.setGrade(10);
                        deviceCodeParamEntityList.add(deviceCodeParamEntity2);
                        //流量计冷量累积量
                        DeviceCodeParamEntity deviceCodeParamEntity3 = new DeviceCodeParamEntity();
                        deviceCodeParamEntity3.setRegisterAddr("36");
                        deviceCodeParamEntity3.setFunCode("03");
                        deviceCodeParamEntity3.setDeviceAddr(devices.getDeviceAddr());
                        deviceCodeParamEntity3.setDeviceName(devices.getDeviceName());
                        deviceCodeParamEntity3.setDeviceType(devices.getDeviceType());
                        deviceCodeParamEntity3.setBaudRate(devices.getBaudRate());
                        deviceCodeParamEntity3.setBrand(devices.getBrand());
                        deviceCodeParamEntity3.setDataPort(devices.getDataPort());
                        deviceCodeParamEntity3.setProjectID(devices.getProjectID());
                        deviceCodeParamEntity3.setParity(devices.getParity());
                        deviceCodeParamEntity3.setDigit(2);   //保留小数位
                        deviceCodeParamEntity3.setRegisterName("瞬时冷量");
                        String strDat3 =createCloudOrder(deviceCodeParamEntity3);
                        deviceCodeParamEntity3.setStrData(strDat3);
                        deviceCodeParamEntity3.setGrade(11);
                        deviceCodeParamEntityList.add(deviceCodeParamEntity3);

                    }else if (devices.getDeviceType().equals("电表")){
                        DeviceCodeParamEntity deviceCodeParamEntity1 = new DeviceCodeParamEntity();
                        deviceCodeParamEntity1.setRegisterAddr("9010");
                        deviceCodeParamEntity1.setDeviceAddr(devices.getDeviceAddr());
                        deviceCodeParamEntity1.setDeviceName(devices.getDeviceName());
                        deviceCodeParamEntity1.setDeviceType(devices.getDeviceType());
                        deviceCodeParamEntity1.setBaudRate(devices.getBaudRate());
                        deviceCodeParamEntity1.setBrand(devices.getBrand());
                        deviceCodeParamEntity1.setDataPort(devices.getDataPort());
                        deviceCodeParamEntity1.setProjectID(devices.getProjectID());
                        deviceCodeParamEntity1.setParity(devices.getParity());
                        deviceCodeParamEntity1.setGrade(devices.getGrade());
                        deviceCodeParamEntity1.setDigit(2); //保留小数位
                        deviceCodeParamEntity1.setRegisterName("总电量");
                        String strData1=createMeterOrder(deviceCodeParamEntity1);//生成电表指令
                        deviceCodeParamEntity1.setStrData(strData1);
                        deviceCodeParamEntityList.add(deviceCodeParamEntity1);
                    }else if(devices.getDeviceType().equals("冷水机")){
                        List<RegisterAddrEntity> regList=registerAddrService.selectRegisterAddr();
                        if(regList.size()>0){
                            for(RegisterAddrEntity reg:regList){
                                DeviceCodeParamEntity deviceCodeParamEntity1 = new DeviceCodeParamEntity();
                                deviceCodeParamEntity1.setRegisterAddr(reg.getRegisterAddr()); //参数寄存器地址
                                String funFlag=ExchangeStringUtil.addZeroForNum(String.valueOf(reg.getFunFlag()),2);
                                deviceCodeParamEntity1.setFunCode(funFlag); //功能码
                                deviceCodeParamEntity1.setDeviceAddr(devices.getDeviceAddr());
                                deviceCodeParamEntity1.setDeviceName(devices.getDeviceName());
                                deviceCodeParamEntity1.setDeviceType(devices.getDeviceType());
                                deviceCodeParamEntity1.setBaudRate(devices.getBaudRate());
                                deviceCodeParamEntity1.setBrand(devices.getBrand());
                                deviceCodeParamEntity1.setDataPort(devices.getDataPort());
                                deviceCodeParamEntity1.setProjectID(devices.getProjectID());
                                deviceCodeParamEntity1.setParity(devices.getParity());
                                deviceCodeParamEntity1.setGrade(devices.getGrade());

                                deviceCodeParamEntity1.setDigit(reg.getFloatFlag()); //保留小数位
                                deviceCodeParamEntity1.setRegisterName(reg.getRegName()); //寄存器名称
                                String strData1=createChillersOrder(deviceCodeParamEntity1);//生成冷水机指令
                                deviceCodeParamEntity1.setStrData(strData1);
                                deviceCodeParamEntityList.add(deviceCodeParamEntity1);
                            }
                        }
                    }
                }
                deviceCodeParamService.insertDeviceCodeParamList(deviceCodeParamEntityList);
            }
        }
    }
}
