package com.mh.user.utils;

import lombok.extern.slf4j.Slf4j;

/**
 * @author ljf
 * @title ：
 * @description ： 处理解析第一次上来报文和心跳包处理
 * 减少创建实例，采用单例模式
 * @updateTime 2020-10-23
 * @throws ：
 */
@Slf4j
public class HeartUtil {

    // 创建私有构造函数
    private HeartUtil(){}

    // 创建单例
    public static HeartUtil heartUtil = null;

    // 创建过程
    public static HeartUtil getInstance() {
        if (null == heartUtil) {
            heartUtil = new HeartUtil();
        }
        return heartUtil;
    }

    public String firstMessage(String receiveData) {
        receiveData = receiveData.toUpperCase();
        if (receiveData.startsWith("24")) {
            int receiveDataLen = receiveData.length();
            // 截取DTUId
            String DTUId = receiveData.substring(2,10);
            DTUId = ExchangeStringUtil.hexToDec(DTUId);
            // 截取包类型
            // 1) 0x00： 报告 IMSI
            // 2) 0x01： 高优先级切换时间设置反馈
            // 3) 0x02： IP 地址/域名信息设置反馈
            // 4) 0x03： 心跳包
            // 5) 0x04： 心跳时间(单位分钟)设置反馈
            // 6) 0x05： 报告固件版本
            // 7) 0x06： 保留
            // 8) 0x08： 远程 AT 命令反馈
            String frameType = receiveData.substring(10,12);
            if ("00".equalsIgnoreCase(frameType)) {
                // 报告IMSI号,更新最新上线时间
                log.info("提交IMSI号");
            } else if ("03".equalsIgnoreCase(frameType)) {
                // 心跳包，更新最新上线时间
                log.info("心跳包管理");
            }
            // 截取数据长度
            String len = ExchangeStringUtil.hexToDec(receiveData.substring(12,16));
            int dataEndIndex = 16+(Integer.parseInt(len)*2);
            // 截取数据域
            String data = receiveData.substring(16,dataEndIndex);
            // 截取校验位
            String oldCRC16 = receiveData.substring(dataEndIndex,receiveDataLen/2);
            byte[] strOrder = ExchangeStringUtil.hexStrToBinaryStr(receiveData.substring(0,dataEndIndex));
            String myCRC16 = ExchangeStringUtil.decToHex(String.valueOf(CRC16.CRC16_XMODEM(strOrder)));
            if (oldCRC16.equalsIgnoreCase(myCRC16)) {
                // 验证成功，比较两边
                String firstStr = receiveData.substring(0,receiveDataLen/2);
                String secondStr = receiveData.substring(receiveDataLen/2);
                secondStr = ExchangeStringUtil.changePosition(secondStr);
                if (firstStr.equalsIgnoreCase(secondStr)) {
                    // 验证刚刚开始报文或者心跳包
                    // 更新网关管理
                    log.info("网关ID： " + DTUId);
                    return DTUId;
                }
            }

        }
        return null;
    }
}
