package com.mh.user.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.mh.user.entity.ChillersEntity;
import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.entity.DeviceManageEntity;
import com.mh.user.model.OrderParamModel;
import com.mh.user.serialport.SerialPortListener;
import com.mh.user.serialport.SerialPortSendReceive;
import com.mh.user.serialport.SerialPortUtil;
import com.mh.user.serialport.SerialTool;
//import purejavacomm.SerialPort;
import gnu.io.SerialPort;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import lombok.extern.slf4j.Slf4j;
import org.apache.log4j.Logger;

/**
 * @author ljf
 * @title ：
 * @description ： 发送指令工具类
 * @updateTime 2021-01-26
 * @throws ：
 */
//@Slf4j
public class SendOrderUtils {

	private static final Logger log = Logger.getLogger(SendOrderUtils.class);
	/**
	 * 发送采集冷水机组
	 * @param deviceCodeParam
	 * @param ctx
	 * @param num
	 * @param size
	 */
	public static void sendChillersOrder(DeviceCodeParamEntity deviceCodeParam, ChannelHandlerContext ctx, int num, int size) {
		// 继续发送下一个采集冷水机设备指令
		GetReadOrder485 getReadOrder485 = new GetReadOrder485();
		String sendStr = getReadOrder485.createChillersOrder(deviceCodeParam);
		ByteBuf buffer = getByteBuf(ctx, sendStr);
		// 发送数据
		ctx.channel().writeAndFlush(buffer);
		log.info("客户端再次往服务端发送数据" + num + " 数据条数：" + size);
	}

	public static void sendMeterOrCloud(DeviceCodeParamEntity deviceCodeParam, ChannelHandlerContext ctx, int num, int size) {
		if (deviceCodeParam.getDeviceType().equals("电表")) {
			sendMeterOrder(deviceCodeParam,ctx,num,size );
		} else if (deviceCodeParam.getDeviceType().equals("流量计")) {
			sendCloudOrder(deviceCodeParam,ctx,num,size );
		}
	}
	/**
	 * 发送采集电表
	 * @param deviceCodeParam
	 * @param num
	 * @param size
	 * @param ctx
	 */

	public static void sendMeterOrder(DeviceCodeParamEntity deviceCodeParam,
									  ChannelHandlerContext ctx,
	                                  int num,
									  int size) {
		String sendStr="";
		ByteBuf buffer = getByteBuf(ctx, sendStr);
		// 发送数据
		ctx.channel().writeAndFlush(buffer);
		log.info("客户端再次往服务端发送数据" + num + " 数据条数：" + size);
	}
	/**
	 * 发送采集冷量计工具类
	 * @param deviceCodeParam
	 * @param num
	 * @param size
	 * @param ctx
	 */
	public static void sendCloudOrder(DeviceCodeParamEntity deviceCodeParam,
									  ChannelHandlerContext ctx,
									  int num,
									  int size) {
		// 创建将要写出的数据
		String sendStr = GetReadOrder485.createCloudOrder(deviceCodeParam);
		ByteBuf buffer = getByteBuf(ctx, sendStr);
		// 发送数据
		ctx.channel().writeAndFlush(buffer);
		log.info("客户端再次往服务端发送数据" + num + " 数据条数：" + size);
	}

	//向串口发指令数据
	public static void sendSerialPort(DeviceCodeParamEntity deviceCodeParam,SerialPort serialPort ) {

		String deviceType=deviceCodeParam.getDeviceType();
		String sendStr=null;
		if (deviceType.equals("电表")){
			sendStr = GetReadOrder485.createMeterOrder(deviceCodeParam);
		}else if (deviceType.equals("水表")){
			sendStr = GetReadOrder485.createWtMeterOrder(deviceCodeParam);
		}

		String str=sendStr;
        try{
        	byte[] HEX = SerialPortListener.hexStr2Byte(str);
		    SerialPortUtil serialPortUtil = SerialPortUtil.getSerialPortUtil();
		    serialPortUtil.sendToPort(serialPort, HEX);

//			SerialTool serialPortUtil = SerialTool.getSerialPortUtil();
//			serialPortUtil.sendToPort(HEX,serialPort);
		    log.info(deviceType+"向串口"+serialPort+"发送数据>>>>" + str);
		}catch (Exception e){
        	e.printStackTrace();
			log.error("向串口"+serialPort+"发送数据异常！");
		}

	}

	// 发送所有类型采集报文
	public static void sendAllOrder(DeviceCodeParamEntity deviceCodeParam, ChannelHandlerContext ctx, int num, int size) {
//		String deviceType=deviceCodeParam.getDeviceType();
		String sendStr=deviceCodeParam.getStrData();
//		if (deviceType.equals("电表")){
//			sendStr = GetReadOrder485.createMeterOrder(deviceCodeParam);
//		}else if(deviceType.equals("水表")){
//			sendStr = GetReadOrder485.createWtMeterOrder(deviceCodeParam);
//		}else if(deviceType.equals("冷量计")){
//			sendStr = GetReadOrder485.createCloudOrder(deviceCodeParam);
//		}else if(deviceType.equals("冷水机")){
//			sendStr = GetReadOrder485.createChillersOrder(deviceCodeParam);
//		}
		try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			log.error("线程休眠异常", e);
		}
		ByteBuf buffer = getByteBuf(ctx, sendStr);
		// 发送数据
		ctx.channel().writeAndFlush(buffer);
		log.info("sends :"+sendStr+",num:"+num+",records："+size);
	}

	// 发送所有类型采集报文
	public static void sendAllOrder2(DeviceCodeParamEntity deviceCodeParam, ChannelHandlerContext ctx, String snr,String ip, int num, int size) {
		try{
//			String deviceType=deviceCodeParam.getDeviceType();
			String sendStr="";
//			if (deviceType.equals("电表")){
//				sendStr = GetReadOrder485.createMeterOrder(deviceCodeParam);
//			}else if(deviceType.equals("水表")){
//				sendStr = GetReadOrder485.createWtMeterOrder(deviceCodeParam);
//			}else if(deviceType.equals("冷量计")){
//				sendStr = GetReadOrder485.createCloudOrder(deviceCodeParam);
//			}else if(deviceType.equals("冷水机")){
//				sendStr = GetReadOrder485.createChillersOrder(deviceCodeParam);
//			}
			sendStr=deviceCodeParam.getStrData();
			sendStr=ExchangeStringUtil.hexToBase64(sendStr);
			String band=String.valueOf(deviceCodeParam.getBaudRate());
			String parity=deviceCodeParam.getParity();
			//组织
			JSONObject jsonObject= new JSONObject();
			jsonObject.put("snr",snr);
			jsonObject.put("cmd","uData");
			jsonObject.put("parity",parity);
			jsonObject.put("byteSize","8");
			jsonObject.put("stopbits","1");
			jsonObject.put("ip",ip);
			jsonObject.put("ipAddr","192.168.1.202");
			jsonObject.put("name","0000000000");
			jsonObject.put("mode","RS485");
			jsonObject.put("data",sendStr);
			jsonObject.put("baud",band);
			jsonObject.put("addr","100");
			String str=jsonObject.toString();
//			log.info("send json:"+str);
			str=ExchangeStringUtil.strTo16(str);
			ByteBuf buffer = getByteBuf(ctx, str);
			Thread.sleep(1000);
			ctx.channel().writeAndFlush(buffer);// 发送数据
//			log.info("white gateway client sends data to the server==>" + num + ",records==>" +size);
			log.info("发送数据到白色网关==>" + num + ",长度==>" +size);
		}catch (Exception e){
			e.printStackTrace();
		}
	}


	private static ByteBuf getByteBuf(ChannelHandlerContext ctx, String sendStr) {
		// 申请一个数据结构存储信息
		ByteBuf buffer = ctx.alloc().buffer();
		// 将信息放入数据结构中
		buffer.writeBytes(ExchangeStringUtil.hexStrToBinaryStr(sendStr));//对接需要16进制
		return buffer;
	}
}
